@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -27,10 +17,10 @@ from gi.repository import Gtk
 from nbxmpp.client import Client as NBXMPPClient
 from nbxmpp.const import ConnectionType
 from nbxmpp.const import StreamError
+from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 
 from gajim.common import app
-from gajim.common import helpers
 from gajim.common import modules
 from gajim.common import passwords
 from gajim.common.client_modules import ClientModules
@@ -39,7 +29,6 @@ from gajim.common.const import SimpleClientState
 from gajim.common.events import AccountConnected
 from gajim.common.events import AccountDisconnected
 from gajim.common.events import MessageNotSent
-from gajim.common.events import MessageSent
 from gajim.common.events import Notification
 from gajim.common.events import PasswordRequired
 from gajim.common.events import PlainConnection
@@ -48,7 +37,6 @@ from gajim.common.events import StanzaReceived
 from gajim.common.events import StanzaSent
 from gajim.common.helpers import get_account_proxy
 from gajim.common.helpers import get_custom_host
-from gajim.common.helpers import get_idle_status_message
 from gajim.common.helpers import get_resource
 from gajim.common.helpers import Observable
 from gajim.common.helpers import warn_about_plain_connection
@@ -57,6 +45,8 @@ from gajim.common.idle import IdleMonitorManager
 from gajim.common.idle import Monitor
 from gajim.common.structs import OutgoingMessage
 from gajim.common.util.http import create_http_session
+from gajim.common.util.status import get_idle_status_message
+from gajim.common.util.text import to_one_line
 
 from gajim.gtk.util import open_window
 
@@ -66,6 +56,15 @@ log = logging.getLogger('gajim.client')
 IgnoredTlsErrorsT = set[Gio.TlsCertificateFlags] | None
 
 
+def call_counter(func: Any):
+    def helper(self, restart: bool = False) -> Any:
+        if restart:
+            self._connect_machine_calls = 0
+        self._connect_machine_calls += 1
+        return func(self)
+    return helper
+
+
 class Client(Observable, ClientModules):
     def __init__(self, account: str) -> None:
         Observable.__init__(self, log)
@@ -73,9 +72,9 @@ class Client(Observable, ClientModules):
         self._client = None
         self._account = account
         self.name = account
-        self._hostname = app.settings.get_account_setting(self._account,
-                                                          'hostname')
-        self._user = app.settings.get_account_setting(self._account, 'name')
+
+        address = app.settings.get_account_setting(self._account, 'address')
+        self._address = JID.from_string(address)
 
         self._priority = 0
         self._connect_machine_calls = 0
@@ -164,10 +163,11 @@ class Client(Observable, ClientModules):
         log.info('Create new nbxmpp client')
         self._client = NBXMPPClient(log_context=self._account)
         self.connection = self._client
-        self._client.set_domain(self._hostname)
-        self._client.set_username(self._user)
+        self._client.set_domain(self._address.domain)
+        self._client.set_username(self._address.localpart)
         self._client.set_resource(get_resource(self._account))
         self._client.set_http_session(create_http_session())
+        self._client.set_supported_fallback_ns([Namespace.REPLY])
 
         self._client.subscribe('resume-failed', self._on_resume_failed)
         self._client.subscribe('resume-successful', self._on_resume_successful)
@@ -200,11 +200,15 @@ class Client(Observable, ClientModules):
             self.update_presence()
 
         self.notify('state-changed', SimpleClientState.CONNECTED)
+        self.notify('resume-successful')
 
     def _set_client_available(self) -> None:
         self._set_state(ClientState.AVAILABLE)
         app.ged.raise_event(AccountConnected(account=self._account))
 
+        if not app.settings.get_account_setting(self._account, 'autojoin_sync'):
+            self.join_mucs()
+
     def disconnect(self,
                    gracefully: bool,
                    reconnect: bool,
@@ -349,6 +353,12 @@ class Client(Observable, ClientModules):
         # This returns the bare jid
         return nbxmpp.JID.from_string(app.get_jid_from_account(self._account))
 
+    def get_bound_jid(self) -> JID:
+        assert self._client is not None
+        jid = self._client.get_bound_jid()
+        assert jid is not None
+        return jid
+
     def change_status(self, show: str, message: str) -> None:
         if not message:
             message = ''
@@ -366,11 +376,10 @@ class Client(Observable, ClientModules):
             app.settings.set_account_setting(
                 self._account,
                 'last_status_msg',
-                helpers.to_one_line(message))
+                to_one_line(message))
 
         if self._state.is_disconnecting:
-            log.warning("Can\'t change status while "
-                        'disconnect is in progress')
+            log.warning("Can't change status while disconnect is in progress")
             return
 
         if self._state.is_disconnected:
@@ -424,14 +433,12 @@ class Client(Observable, ClientModules):
         if include_muc:
             self.get_module('MUC').update_presence()
 
-    @helpers.call_counter
+    @call_counter
     def connect_machine(self) -> None:
         log.info('Connect machine state: %s', self._connect_machine_calls)
         if self._connect_machine_calls == 1:
-            self.get_module('Delimiter').get_roster_delimiter()
-        elif self._connect_machine_calls == 2:
             self.get_module('Roster').request_roster()
-        elif self._connect_machine_calls == 3:
+        elif self._connect_machine_calls == 2:
             self._finish_connect()
 
     def _finish_connect(self) -> None:
@@ -468,7 +475,7 @@ class Client(Observable, ClientModules):
             return
 
         stanza = self.get_module('Message').build_message_stanza(message)
-        message.stanza = stanza
+        message.set_stanza(stanza)
 
         method = message.contact.settings.get('encryption')
         if not method:
@@ -480,12 +487,15 @@ class Client(Observable, ClientModules):
                 self.get_module('OMEMO').encrypt_message(message)
             except Exception:
                 log.exception('Error')
+                text = message.get_text(with_fallback=False)
+                assert text is not None
                 app.ged.raise_event(
-                    MessageNotSent(client=self._client,
-                                   jid=message.jid,
-                                   message=message.message,
-                                   error=_('Encryption error'),
-                                   time=time.time()))
+                    MessageNotSent(
+                        client=self._client,
+                        jid=str(message.contact.jid),
+                        message=text,
+                        error=_('Encryption error'),
+                        time=time.time()))
                 return
 
             self._send_message(message)
@@ -501,25 +511,8 @@ class Client(Observable, ClientModules):
                                            self._send_message)
 
     def _send_message(self, message: OutgoingMessage) -> None:
-        message.set_sent_timestamp()
-        message.message_id = self.send_stanza(message.stanza)
-
-        log_line_id = None
-        if not message.is_groupchat:
-            log_line_id = self.get_module('Message').log_message(message)
-
-        app.ged.raise_event(
-            MessageSent(jid=message.jid,
-                        account=message.account,
-                        message=message.message,
-                        chatstate=message.chatstate,
-                        timestamp=message.timestamp,
-                        additional_data=message.additional_data,
-                        label=message.label,
-                        correct_id=message.correct_id,
-                        message_id=message.message_id,
-                        msg_log_id=log_line_id,
-                        play_sound=message.play_sound))
+        self.send_stanza(message.get_stanza())
+        self.get_module('Message').store_message(message)
 
     def connect(
         self,
@@ -649,6 +642,17 @@ class Client(Observable, ClientModules):
         active = application.get_property('screensaver-active')
         Monitor.set_extended_away(active)
 
+    def join_mucs(self) -> None:
+        '''Only used when autojoin_sync is False'''
+        self._log.info('Joining all MUCs in all workspaces')
+        for workspace_id in app.settings.get_workspaces():
+            for chat in app.settings.get_workspace_setting(
+                    workspace_id, 'chats'):
+                if chat['account'] != self._account:
+                    continue
+                if chat['type'] == 'groupchat':
+                    self.get_module('MUC').join(chat['jid'])
+
     def cleanup(self) -> None:
         self.disconnect_signals()
         self._destroyed = True
