@@ -12,17 +12,7 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -47,6 +37,9 @@ from gajim.common.file_props import FileProp
 from gajim.common.file_props import FilesProp
 from gajim.common.modules.base import BaseModule
 from gajim.common.socks5 import Socks5SenderClient
+from gajim.common.util.jid import get_full_jid_from_iq
+from gajim.common.util.jid import get_jid_from_iq
+from gajim.common.util.jid import parse_jid
 
 log = logging.getLogger('gajim.c.m.bytestream')
 
@@ -80,7 +73,7 @@ def is_transfer_stopped(file_props: FileProp) -> bool:
         return True
     if file_props.completed:
         return True
-    if not file_props.stopped:
+    if not file_props.stopped:  # noqa: SIM103
         return False
     return True
 
@@ -130,11 +123,11 @@ class Bytestream(BaseModule):
 
     @staticmethod
     def _ft_get_from(iq_obj: Iq) -> str | None:
-        return helpers.get_full_jid_from_iq(iq_obj)
+        return get_full_jid_from_iq(iq_obj)
 
     @staticmethod
     def _ft_get_streamhost_jid_attr(streamhost: nbxmpp.Node) -> str:
-        return helpers.parse_jid(streamhost.getAttr('jid'))
+        return parse_jid(streamhost.getAttr('jid'))
 
     def send_file_approval(self, file_props: FileProp) -> None:
         '''
@@ -378,7 +371,7 @@ class Bytestream(BaseModule):
             if ip_l & (255 << 24 | 240 << 16) == (172 << 24 | 16 << 16):
                 return True
             # 192.168
-            if ip_l & (255 << 24 | 255 << 16) == (192 << 24 | 168 << 16):
+            if ip_l & (255 << 24 | 255 << 16) == (192 << 24 | 168 << 16):  # noqa: SIM103
                 return True
             return False
 
@@ -572,10 +565,10 @@ class Bytestream(BaseModule):
                              _properties: IqProperties
                              ) -> None:
         id_ = iq_obj.getAttr('id')
-        frm = helpers.get_full_jid_from_iq(iq_obj)
+        frm = get_full_jid_from_iq(iq_obj)
         query = iq_obj.getTag('query')
         app.proxy65_manager.error_cb(frm, query)
-        jid = helpers.get_jid_from_iq(iq_obj)
+        jid = get_jid_from_iq(iq_obj)
         id_ = id_[3:]
         file_props = FilesProp.getFilePropBySid(id_)
         if not file_props:
