@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # XEP-0045: Multi-User Chat
 # XEP-0249: Direct MUC Invitations
@@ -18,13 +8,13 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import logging
-from collections import defaultdict
 import time
+from collections import defaultdict
 
 import nbxmpp
+from gi.repository import GLib
 from nbxmpp.const import InviteType
 from nbxmpp.const import StatusCode
 from nbxmpp.errors import StanzaError
@@ -41,26 +31,25 @@ from nbxmpp.structs import StanzaHandler
 from nbxmpp.structs import VoiceRequest
 from nbxmpp.task import Task
 
-from gi.repository import GLib
-
 from gajim.common import app
 from gajim.common import events
 from gajim.common import helpers
 from gajim.common import types
 from gajim.common.const import ClientState
 from gajim.common.const import MUCJoinedState
-from gajim.common.events import MessageModerated
 from gajim.common.events import MucAdded
 from gajim.common.events import MucDecline
 from gajim.common.events import MucInvitation
-from gajim.common.helpers import get_default_muc_config
-from gajim.common.helpers import get_group_chat_nick
-from gajim.common.structs import MUCData
-from gajim.common.structs import MUCPresenceData
-from gajim.common.modules.bits_of_binary import store_bob_data
+from gajim.common.helpers import to_user_string
 from gajim.common.modules.base import BaseModule
+from gajim.common.modules.bits_of_binary import store_bob_data
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.structs import MUCData
+from gajim.common.structs import MUCPresenceData
+from gajim.common.util.datetime import utc_now
+from gajim.common.util.muc import get_default_muc_config
+from gajim.common.util.muc import get_group_chat_nick
 
 log = logging.getLogger('gajim.c.m.muc')
 
@@ -84,7 +73,7 @@ class MUC(BaseModule):
         'approve_voice_request',
         'destroy',
         'request_disco_info',
-        'retract_message'
+        'moderate_message'
     ]
 
     def __init__(self, con: types.Client) -> None:
@@ -101,6 +90,11 @@ class MUC(BaseModule):
                           typ='unavailable',
                           ns=Namespace.MUC_USER,
                           priority=49),
+            StanzaHandler(name='message',
+                          callback=self._on_muc_normal_user_message,
+                          typ='normal',
+                          ns=Namespace.MUC_USER,
+                          priority=49),
             StanzaHandler(name='presence',
                           callback=self._on_error_presence,
                           typ='error',
@@ -109,11 +103,6 @@ class MUC(BaseModule):
                           callback=self._on_subject_change,
                           typ='groupchat',
                           priority=49),
-            StanzaHandler(name='message',
-                          callback=self._on_moderation,
-                          ns=Namespace.FASTEN,
-                          typ='groupchat',
-                          priority=49),
             StanzaHandler(name='message',
                           callback=self._on_config_change,
                           ns=Namespace.MUC_USER,
@@ -188,10 +177,10 @@ class MUC(BaseModule):
                 self._muc_service_jid = info.jid
                 raise nbxmpp.NodeProcessed
 
-    def get_muc_data(self, room_jid: str) -> Optional[MUCData]:
+    def get_muc_data(self, room_jid: JID) -> MUCData | None:
         return self._mucs.get(room_jid)
 
-    def set_password(self, room_jid: str, password: str) -> None:
+    def set_password(self, room_jid: JID, password: str) -> None:
         muc_data = self.get_muc_data(room_jid)
         muc_data.password = password
 
@@ -227,9 +216,9 @@ class MUC(BaseModule):
 
     def _create_muc_data(self,
                          room_jid: str,
-                         nick: Optional[str],
-                         password: Optional[str],
-                         config: Optional[dict[str, Any]]
+                         nick: str | None,
+                         password: str | None,
+                         config: dict[str, Any] | None
                          ) -> MUCData:
         if not nick:
             nick = get_group_chat_nick(self._account, room_jid)
@@ -240,13 +229,13 @@ class MUC(BaseModule):
             if bookmark.password is not None:
                 password = bookmark.password
 
-        return MUCData(room_jid, nick, password, config)
+        return MUCData(room_jid, nick, None, password, config)
 
     def join(self,
-             jid: str,
-             nick: Optional[str] = None,
-             password: Optional[str] = None,
-             config: Optional[dict[str, Any]] = None
+             jid: JID,
+             nick: str | None = None,
+             password: str | None = None,
+             config: dict[str, Any] | None = None
              ) -> None:
         if not app.account_is_available(self._account):
             return
@@ -289,20 +278,30 @@ class MUC(BaseModule):
         muc_data = self._create_muc_data(jid, None, None, config)
         self._mucs[jid] = muc_data
         self._create(muc_data)
-        self._push_muc_added_event(jid)
+        self._push_muc_added_event(jid, select_chat=True)
+
+    def _push_muc_added_event(self,
+                              jid: str,
+                              select_chat: bool = False
+                              ) -> None:
 
-    def _push_muc_added_event(self, jid: str) -> None:
         app.ged.raise_event(MucAdded(account=self._account,
-                                     jid=JID.from_string(jid)))
+                                     jid=JID.from_string(jid),
+                                     select_chat=select_chat))
 
     def _on_disco_result(self, task: Task) -> None:
         try:
             result = task.finish()
         except StanzaError as error:
-            self._log.info('Disco %s failed: %s', error.jid, error.get_text())
-
-            room = self._get_contact(error.jid.bare)
-            room.notify('room-join-failed', error)
+            jid = error.jid
+            self._log.info('Disco %s failed: %s', jid, error.get_text())
+            muc_data = self._mucs.get(jid)
+            muc_data.error = 'join-failed'
+            error_text = helpers.to_user_string(error)
+            muc_data.error_text = error_text
+            self._set_muc_state(jid, MUCJoinedState.NOT_JOINED)
+            room = self._get_contact(jid)
+            room.notify('room-join-failed', error_text)
             return
 
         muc_data = self._mucs.get(result.info.jid)
@@ -347,8 +346,9 @@ class MUC(BaseModule):
         self._con.send_stanza(presence)
 
     def leave(self,
-              room_jid: str,
-              reason: Optional[str] = None
+              room_jid: JID,
+              reason: str | None = None,
+              unregister: bool = False,
               ) -> None:
         self._log.info('Leave MUC: %s', room_jid)
 
@@ -358,10 +358,11 @@ class MUC(BaseModule):
         if muc_data is None:
             return
 
+        self._remove_rejoin_timeout(room_jid)
+
         if muc_data.state.is_not_joined:
             return
 
-        self._remove_rejoin_timeout(room_jid)
 
         self._con.get_module('Presence').send_presence(
             muc_data.occupant_jid,
@@ -374,6 +375,34 @@ class MUC(BaseModule):
         room.set_not_joined()
         room.notify('room-left')
 
+        if not unregister:
+            return
+
+        contact = self._con.get_module('Contacts').get_contact(room_jid)
+        assert isinstance(contact, GroupchatContact)
+        self._con.get_module('Register').unregister(
+            jid=contact.jid,
+            callback=self._unregister_callback,
+        )
+
+    def _unregister_callback(self, task: Task) -> None:
+        try:
+            task.finish()
+        except StanzaError as error:
+            self._log.error('Error while unregistering from MUC: %s',
+                            to_user_string(error))
+
+    def abort_join(self, room_jid: JID) -> None:
+        self._remove_rejoin_timeout(room_jid)
+
+        muc = self._mucs[room_jid]
+        self._con.get_module('Presence').send_presence(
+            muc.occupant_jid,
+            typ='unavailable',
+            caps=False)
+
+        self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
+
     def configure_room(self, room_jid: str) -> None:
         self._nbxmpp('MUC').request_config(room_jid,
                                            callback=self._on_room_config)
@@ -385,7 +414,8 @@ class MUC(BaseModule):
             self._log.info(error)
 
             room = self._get_contact(error.jid.bare)
-            room.notfiy('room-config-failed', error)
+            error_text = helpers.to_user_string(error)
+            room.notfiy('room-config-failed', error_text)
             return
 
         self._log.info('Configure room: %s', result.jid)
@@ -398,7 +428,7 @@ class MUC(BaseModule):
 
     @staticmethod
     def _apply_config(form: SimpleDataForm,
-                      config: Optional[dict[str, Any]] = None
+                      config: dict[str, Any] | None = None
                       ) -> None:
         default_config = get_default_muc_config()
         if config is not None:
@@ -418,26 +448,13 @@ class MUC(BaseModule):
             self._log.info(error)
 
             room = self._get_contact(error.jid.bare)
-            room.notfiy('room-config-failed', error)
+            error_text = helpers.to_user_string(error)
+            room.notfiy('room-config-failed', error_text)
             return
 
         self._con.get_module('Discovery').disco_muc(
             result.jid, callback=self._on_disco_result_after_config)
 
-        # If this is an automatic room creation
-        try:
-            invites = app.automatic_rooms[self._account][result.jid]['invities']
-        except KeyError:
-            return
-
-        user_list = {}
-        for jid in invites:
-            user_list[jid] = {'affiliation': 'member'}
-        self.set_affiliation(result.jid, user_list)
-
-        for jid in invites:
-            self.invite(result.jid, JID.from_string(jid))
-
     def _on_disco_result_after_config(self, task: Task) -> None:
         try:
             result = task.finish()
@@ -452,7 +469,8 @@ class MUC(BaseModule):
         self._log.info('Configuration finished: %s', jid)
 
         room = self._get_contact(jid.bare)
-        event = events.MUCRoomConfigFinished(timestamp=time.time())
+        event = events.MUCRoomConfigFinished(
+            timestamp=utc_now())
 
         assert isinstance(room, GroupchatContact)
         app.storage.events.store(room, event)
@@ -470,7 +488,7 @@ class MUC(BaseModule):
                 status=message,
                 idle_time=idle)
 
-    def change_nick(self, room_jid: str, new_nick: str) -> None:
+    def change_nick(self, room_jid: JID, new_nick: str) -> None:
         status, message, _idle = self._con.get_presence_state()
         self._con.get_module('Presence').send_presence(
             f'{room_jid}/{new_nick}',
@@ -488,11 +506,6 @@ class MUC(BaseModule):
         if muc_data is None:
             return
 
-        if properties.jid.resource != muc_data.nick:
-            self._log.warning('Unknown error presence')
-            self._log.warning(stanza)
-            return
-
         room = self._get_contact(room_jid)
 
         if muc_data.state == MUCJoinedState.JOINING:
@@ -506,7 +519,7 @@ class MUC(BaseModule):
             elif properties.error.condition == 'not-authorized':
                 self._remove_rejoin_timeout(room_jid)
                 self._set_muc_state(room_jid, MUCJoinedState.PASSWORD_REQUEST)
-                room.notify('room-password-required', properties)
+                room.notify('room-password-required')
 
             else:
                 self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
@@ -515,14 +528,14 @@ class MUC(BaseModule):
                     assert isinstance(properties.error, CommonError)
                     muc_data.error_text = helpers.to_user_string(
                         properties.error)
-                    room.notify('room-join-failed', properties.error)
+                    room.notify('room-join-failed', muc_data.error_text)
 
         elif muc_data.state == MUCJoinedState.CREATING:
             self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
             muc_data.error = 'creation-failed'
             assert isinstance(properties.error, CommonError)
             muc_data.error_text = helpers.to_user_string(properties.error)
-            room.notify('room-creation-failed', properties)
+            room.notify('room-creation-failed', muc_data.error_text)
 
         elif muc_data.state == MUCJoinedState.CAPTCHA_REQUEST:
             self._set_muc_state(room_jid, MUCJoinedState.CAPTCHA_FAILED)
@@ -530,18 +543,19 @@ class MUC(BaseModule):
             muc_data.error = 'captcha-failed'
             assert isinstance(properties.error, CommonError)
             muc_data.error_text = helpers.to_user_string(properties.error)
-            room.notify('room-captcha-error', properties.error)
+            room.notify('room-captcha-error', muc_data.error_text)
 
         elif muc_data.state == MUCJoinedState.CAPTCHA_FAILED:
             self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
 
         else:
+            error = helpers.to_user_string(properties.error)
             event = events.MUCRoomPresenceError(
-                timestamp=time.time(),
-                error=properties.error)
+                timestamp=utc_now(),
+                error=error)
             assert isinstance(room, GroupchatContact)
             app.storage.events.store(room, event)
-            room.notify('room-presence-error', properties)
+            room.notify('room-presence-error', event)
 
     def _on_muc_user_presence(self,
                               _con: types.xmppClient,
@@ -559,6 +573,8 @@ class MUC(BaseModule):
         occupant = self._get_contact(properties.jid, groupchat=True)
         room = self._get_contact(properties.jid.bare)
 
+        timestamp = utc_now()
+
         if properties.is_muc_destroyed:
             self._log.info('MUC destroyed: %s', room_jid)
             self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
@@ -566,7 +582,7 @@ class MUC(BaseModule):
             room.set_not_joined()
 
             event = events.MUCRoomDestroyed(
-                timestamp=time.time(),
+                timestamp=timestamp,
                 reason=properties.muc_destroyed.reason,
                 alternate=properties.muc_destroyed.alternate)
             assert isinstance(room, GroupchatContact)
@@ -601,7 +617,7 @@ class MUC(BaseModule):
                                                    occupant)
 
             event = events.MUCNicknameChanged(
-                timestamp=time.time(),
+                timestamp=timestamp,
                 is_self=properties.is_muc_self_presence,
                 new_name=new_occupant.name,
                 old_name=occupant.name)
@@ -617,6 +633,9 @@ class MUC(BaseModule):
             if properties.is_muc_self_presence:
                 self._log.info('Self presence: %s', properties.jid)
                 if muc_data.state == MUCJoinedState.JOINING:
+                    if room.supports(Namespace.OCCUPANT_ID):
+                        muc_data.occupant_id = properties.occupant_id
+
                     if (properties.is_nickname_modified or
                             muc_data.nick != properties.muc_nickname):
                         muc_data.nick = properties.muc_nickname
@@ -625,6 +644,7 @@ class MUC(BaseModule):
 
                 elif muc_data.state == MUCJoinedState.CREATING:
                     if properties.is_new_room:
+                        muc_data.occupant_id = properties.occupant_id
                         self.configure_room(room_jid)
 
             presence = self._process_user_presence(properties)
@@ -638,7 +658,7 @@ class MUC(BaseModule):
             room.set_not_joined()
 
             event = events.MUCRoomKicked(
-                timestamp=time.time(),
+                timestamp=timestamp,
                 status_codes=properties.muc_status_codes,
                 reason=properties.muc_user.reason,
                 actor=properties.muc_user.actor)
@@ -659,6 +679,11 @@ class MUC(BaseModule):
         try:
             presence = self._process_user_presence(properties)
         except KeyError:
+            if (properties.type.is_unavailable
+                    and properties.muc_user.role.is_none):
+                # prosody allows broadcasting "unavailable" presences from
+                # offline room members
+                return
             # Sometimes it seems to happen that we get unavailable presence
             # from occupants we don’t know
             log.warning('Unexpected presence received')
@@ -673,7 +698,7 @@ class MUC(BaseModule):
             presence: MUCPresenceData,
             occupant: GroupchatParticipant) -> None:
 
-        timestamp = time.time()
+        timestamp = utc_now()
 
         if not occupant.is_available and presence.available:
 
@@ -748,22 +773,77 @@ class MUC(BaseModule):
             app.storage.events.store(occupant.room, event)
             signals_and_events.append(('user-status-show-changed', event))
 
+        if occupant.hats != presence.hats:
+            if presence.hats:
+                hats = [h.title for h in presence.hats.get_hats()]
+            else:
+                hats = None
+            event = events.MUCUserHatsChanged(
+                timestamp=timestamp,
+                is_self=properties.is_muc_self_presence,
+                nick=occupant.name,
+                hats=hats,
+            )
+
+            app.storage.events.store(occupant, event)
+            app.storage.events.store(occupant.room, event)
+            signals_and_events.append(('user-hats-changed', event))
+
         occupant.update_presence(presence)
         for signal, event in signals_and_events:
             occupant.notify(signal, event)
 
+    def _on_muc_normal_user_message(self,
+                                    _con: types.xmppClient,
+                                    stanza: Presence,
+                                    properties: MessageProperties) -> None:
+        if not properties.muc_user or not properties.muc_user.jid:
+            return
+
+        if not properties.muc_user.affiliation and not properties.muc_user.role:
+            return
+
+        room_jid = str(properties.muc_jid)
+        if room_jid not in self._mucs:
+            self._log.warning('Message from unknown MUC')
+            self._log.warning(stanza)
+            return
+
+        room = self._get_contact(room_jid)
+
+        if properties.muc_user.nick is not None:
+            nick = properties.muc_user.nick
+        else:
+            contact = self._get_contact(properties.muc_user.jid)
+            nick = contact.name
+
+        event = events.MUCAffiliationChanged(
+            timestamp=utc_now(),
+            nick=nick,
+            affiliation=properties.muc_user.affiliation,
+        )
+        app.storage.events.store(room, event)
+        room.notify('room-affiliation-changed', event)
+
     def _process_user_presence(self,
                                properties: PresenceProperties
                                ) -> MUCPresenceData:
+
         jid = properties.jid
-        muc_presence = MUCPresenceData.from_presence(properties)
+        group_contact = self._client.get_module('Contacts').get_contact(
+            jid.bare, groupchat=True)
+        occupant_id = None
+        if group_contact.supports(Namespace.OCCUPANT_ID):
+            occupant_id = properties.occupant_id
+
+        muc_presence = MUCPresenceData.from_presence(properties, occupant_id)
         if not muc_presence.available:
             self._joined_users[jid.bare].pop(jid.resource)
         else:
             self._joined_users[jid.bare][jid.resource] = muc_presence
         return muc_presence
 
-    def _is_user_joined(self, jid: Optional[JID]) -> bool:
+    def _is_user_joined(self, jid: JID | None) -> bool:
         try:
             self._joined_users[jid.bare][jid.resource]
         except KeyError:
@@ -777,7 +857,7 @@ class MUC(BaseModule):
         self._remove_rejoin_timeout(room_jid)
         self._rejoin_muc.add(room_jid)
         self._log.info('Start rejoin timeout for: %s', room_jid)
-        id_ = GLib.timeout_add_seconds(2, self._rejoin, room_jid)
+        id_ = GLib.timeout_add_seconds(10, self._rejoin, room_jid)
         self._rejoin_timeouts[room_jid] = id_
 
     def _remove_rejoin_timeout(self, room_jid: str) -> None:
@@ -810,20 +890,20 @@ class MUC(BaseModule):
         old_subject = muc_data.subject
 
         muc_subject = properties.muc_subject
+        assert muc_subject is not None
         muc_data.subject = muc_subject
 
-        if muc_subject is not None:
-            if muc_subject.timestamp is None:
-                muc_subject = muc_subject._replace(timestamp=time.time())
+        if muc_subject.timestamp is None:
+            muc_subject = muc_subject._replace(timestamp=time.time())
 
-            if old_subject is None:
-                muc_data.last_subject_timestamp = time.time()
-                room.notify('room-subject', muc_subject)
-            else:
-                # Check if we already showed that subject (rejoin)
-                if old_subject.text != muc_subject.text:
-                    muc_data.last_subject_timestamp = time.time()
-                    room.notify('room-subject', muc_subject)
+        if old_subject is None:
+            muc_data.last_subject_timestamp = time.time()
+            room.notify('room-subject', muc_subject)
+
+        elif old_subject.text != muc_subject.text:
+            # Check if we already showed that subject (rejoin)
+            muc_data.last_subject_timestamp = time.time()
+            room.notify('room-subject', muc_subject)
 
         if muc_data.state == MUCJoinedState.JOINING:
             self._room_join_complete(muc_data)
@@ -831,28 +911,15 @@ class MUC(BaseModule):
 
         raise nbxmpp.NodeProcessed
 
-    def _on_moderation(self,
-                       _con: types.xmppClient,
-                       _stanza: Message,
-                       properties: MessageProperties
-                       ) -> None:
-        if not properties.is_moderation:
-            return
-
-        app.storage.archive.update_additional_data(
-            self._account, properties.moderation.stanza_id, properties)
-
-        app.ged.raise_event(
-            MessageModerated(account=self._account,
-                             jid=properties.jid,
-                             moderation=properties.moderation))
-
-        raise nbxmpp.NodeProcessed
-
-    def cancel_password_request(self, room_jid: str) -> None:
+    def cancel_password_request(self, room_jid: JID) -> None:
         self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
 
-    def _room_join_complete(self, muc_data: MUCData):
+    def _room_join_complete(self, muc_data: MUCData) -> None:
+        # Reset errors from previous tries, otherwise when we are
+        # disconnected from the room, the ChatFunctionPage will be shown
+        muc_data.error = None
+        muc_data.error_text = None
+
         self._set_muc_state(muc_data.jid, MUCJoinedState.JOINED)
         self._remove_rejoin_timeout(muc_data.jid)
 
@@ -939,11 +1006,11 @@ class MUC(BaseModule):
         self._remove_rejoin_timeout(properties.jid)
 
         room = self._get_contact(properties.jid.bare)
-        room.notify('room-captcha-challenge', properties)
+        room.notify('room-captcha-challenge')
 
         raise nbxmpp.NodeProcessed
 
-    def cancel_captcha(self, room_jid: str) -> None:
+    def cancel_captcha(self, room_jid: JID) -> None:
         muc_data = self._mucs.get(room_jid)
         if muc_data is None:
             return
@@ -956,7 +1023,7 @@ class MUC(BaseModule):
         self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
 
     def send_captcha(self,
-                     room_jid: str,
+                     room_jid: JID,
                      form_node: SimpleDataForm
                      ) -> None:
         self._set_muc_state(room_jid, MUCJoinedState.JOINING)
@@ -973,7 +1040,8 @@ class MUC(BaseModule):
                 return
             self._set_muc_state(error.jid, MUCJoinedState.CAPTCHA_FAILED)
             room = self._get_contact(error.jid)
-            room.notify('room-captcha-error', error)
+            error_text = helpers.to_user_string(error)
+            room.notify('room-captcha-error', error_text)
 
     def _on_config_change(self,
                           _con: types.xmppClient,
@@ -988,7 +1056,7 @@ class MUC(BaseModule):
 
         assert properties.muc_status_codes is not None
         event = events.MUCRoomConfigChanged(
-            timestamp=time.time(),
+            timestamp=utc_now(),
             status_codes=properties.muc_status_codes)
 
         assert properties.muc_jid is not None
@@ -1054,7 +1122,7 @@ class MUC(BaseModule):
     def invite(self,
                room: JID,
                jid: JID,
-               reason: Optional[str] = None,
+               reason: str | None = None,
                continue_: bool = False
                ) -> str:
 
@@ -1101,5 +1169,5 @@ class MUC(BaseModule):
             self._remove_rejoin_timeout(room_jid)
 
     def cleanup(self) -> None:
-        super().cleanup()
+        BaseModule.cleanup(self)
         self._remove_all_timeouts()
