@@ -1,29 +1,18 @@
 # Copyright (C) 2019 Philipp Hörist <philipp AT hoerist.com>
 #
-# This file is part of OMEMO Gajim Plugin.
+# This file is part of Gajim.
 #
-# OMEMO Gajim Plugin is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# OMEMO Gajim Plugin is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with OMEMO Gajim Plugin. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # XEP-0384: OMEMO Encryption
 
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
-from typing import Optional
 
 import binascii
 import threading
+from collections.abc import Callable
 from pathlib import Path
 
 from gi.repository import GLib
@@ -38,6 +27,7 @@ from nbxmpp.protocol import JID
 from nbxmpp.protocol import Message
 from nbxmpp.protocol import NodeProcessed
 from nbxmpp.protocol import Presence
+from nbxmpp.structs import EncryptionData
 from nbxmpp.structs import MessageProperties
 from nbxmpp.structs import OMEMOMessage
 from nbxmpp.structs import PresenceProperties
@@ -59,7 +49,6 @@ from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import ged
 from gajim.common import types
-from gajim.common.const import EncryptionData
 from gajim.common.const import EncryptionInfoMsg
 from gajim.common.const import Trust as GajimTrust
 from gajim.common.const import XmppUriQuery
@@ -76,6 +65,7 @@ from gajim.common.modules.util import event_node
 from gajim.common.modules.util import prepare_stanza
 from gajim.common.storage.omemo import OMEMOStorage
 from gajim.common.structs import OutgoingMessage
+from gajim.common.util.decorators import event_filter
 from gajim.common.util.decorators import lru_cache_with_ttl
 
 ALLOWED_TAGS = [
@@ -92,7 +82,10 @@ ALLOWED_TAGS = [
     ('no-permanent-store', Namespace.HINTS),
     ('replace', Namespace.CORRECT),
     ('thread', None),
+    ('reply', Namespace.REPLY),
+    ('fallback', Namespace.FALLBACK),
     ('origin-id', Namespace.SID),
+    ('reactions', Namespace.REACTIONS),
 ]
 
 DeviceIdT = int
@@ -140,7 +133,7 @@ class OMEMO(BaseModule):
         omemo_config = OMEMOConfig(default_prekey_amount=100,
                                    min_prekey_amount=80,
                                    spk_archive_seconds=86400 * 15,
-                                   spk_cycle_seconds=86400,
+                                   spk_cycle_seconds=86400 * 30,
                                    unacknowledged_count=2000)
 
         self._backend = OMEMOSessionManager(
@@ -151,14 +144,17 @@ class OMEMO(BaseModule):
         self._omemo_groupchats: set[str] = set()
         self._muc_temp_store: dict[bytes, str] = {}
 
+    @event_filter(['account'])
     def _on_signed_in(self, _event: SignedIn) -> None:
         self._log.info('Publish our bundle after sign in')
         self.set_bundle()
         self.request_devicelist()
 
+    @event_filter(['account'])
     def _on_muc_disco_update(self, event: MucDiscoUpdate) -> None:
         self._check_if_omemo_capable(str(event.jid))
 
+    @event_filter(['account'])
     def _on_muc_added(self, event: MucAdded) -> None:
         client = app.get_client(event.account)
         contact = client.get_module('Contacts').get_contact(event.jid)
@@ -270,31 +266,38 @@ class OMEMO(BaseModule):
     def _is_omemo_groupchat(self, room_jid: str) -> bool:
         return room_jid in self._omemo_groupchats
 
-    def encrypt_message(self, event: OutgoingMessage) -> bool:
-        if not event.message:
+    def encrypt_message(self, message: OutgoingMessage) -> bool:
+        if not message.has_text():
             return False
 
-        client = app.get_client(self._account)
-        contact = client.get_module('Contacts').get_contact(event.jid)
+        remote_jid = message.contact.jid
 
-        omemo_message = self.backend.encrypt(str(event.jid),
-                                             event.message,
+        client = app.get_client(self._account)
+        contact = client.get_module('Contacts').get_contact(remote_jid)
+
+        text = message.get_text()
+        assert text is not None
+        omemo_message = self.backend.encrypt(str(remote_jid),
+                                             text,
                                              groupchat=contact.is_groupchat)
         if omemo_message is None:
             raise Exception('Encryption error')
 
-        create_omemo_message(event.stanza, omemo_message,
+        create_omemo_message(message.get_stanza(), omemo_message,
                              node_whitelist=ALLOWED_TAGS)
 
-        if event.is_groupchat:
-            self._muc_temp_store[omemo_message.payload] = event.message
-        else:
-            event.xhtml = None
-            event.additional_data['encrypted'] = {
-                'name': 'OMEMO',
-                'trust': GajimTrust[OMEMOTrust.VERIFIED.name]}
+        if message.is_groupchat:
+            self._muc_temp_store[omemo_message.payload] = text
 
-        self._debug_print_stanza(event.stanza)
+        message.set_encryption(
+            EncryptionData(
+                protocol='OMEMO',
+                key='Unknown',
+                trust=GajimTrust[OMEMOTrust.VERIFIED.name]
+            )
+        )
+
+        self._debug_print_stanza(message.get_stanza())
         return True
 
     def encrypt_file(self,
@@ -318,7 +321,7 @@ class OMEMO(BaseModule):
         transfer.size = len(result.payload)
         fragment = binascii.hexlify(result.iv + result.key).decode()
         transfer.set_uri_transform_func(
-            lambda uri: 'aesgcm%s#%s' % (uri[5:], fragment))
+            lambda uri: f'aesgcm{uri[5:]}#{fragment}')
         transfer.set_encrypted_data(result.payload)
         GLib.idle_add(callback, transfer)
 
@@ -379,10 +382,8 @@ class OMEMO(BaseModule):
                 self._log.warning("Can't decrypt own group chat message")
                 return
 
-            plaintext = self._muc_temp_store[properties.omemo.payload]
-            fingerprint = self.backend.get_our_fingerprint()
-            trust = OMEMOTrust.VERIFIED
             del self._muc_temp_store[properties.omemo.payload]
+            return
 
         except DecryptionFailed:
             return
@@ -401,14 +402,15 @@ class OMEMO(BaseModule):
 
         prepare_stanza(stanza, plaintext)
         self._debug_print_stanza(stanza)
-        properties.encrypted = EncryptionData({
-            'name': 'OMEMO',
-            'fingerprint': fingerprint,
-            'trust': GajimTrust[trust.name]})
+        properties.encrypted = EncryptionData(
+            protocol='OMEMO',
+            key=fingerprint or 'Unknown',
+            trust=GajimTrust[trust.name]
+        )
 
     def _process_muc_message(self,
                              properties: MessageProperties
-                             ) -> Optional[str]:
+                             ) -> str | None:
 
         resource = properties.jid.resource
         if properties.muc_ofrom is not None:
@@ -426,7 +428,7 @@ class OMEMO(BaseModule):
 
     def _process_mam_message(self,
                              properties: MessageProperties
-                             ) -> Optional[str]:
+                             ) -> str | None:
 
         self._log.info('Message received, archive: %s', properties.mam.archive)
         if properties.from_muc:
@@ -520,12 +522,11 @@ class OMEMO(BaseModule):
                 self._request_bundle_ttl(jid, device_id)
 
     def _has_trusted_keys(self, jid: str) -> bool:
-        if self.backend.get_identity_infos(
+        infos = self.backend.get_identity_infos(
                 jid,
                 only_active=True,
-                trust=[OMEMOTrust.VERIFIED, OMEMOTrust.BLIND]):
-            return True
-        return False
+                trust=[OMEMOTrust.VERIFIED, OMEMOTrust.BLIND])
+        return bool(infos)
 
     def set_bundle(self, bundle: OMEMOBundle | None = None) -> None:
         if bundle is None:
@@ -567,7 +568,7 @@ class OMEMO(BaseModule):
             jid=JID.from_string(jid),
             message=EncryptionInfoMsg.UNDECIDED_FINGERPRINTS))
 
-    def set_devicelist(self, devicelist: Optional[list[int]] = None) -> None:
+    def set_devicelist(self, devicelist: list[int] | None = None) -> None:
         devicelist_: set[int] = {self.backend.get_our_device()}
         if devicelist is not None:
             devicelist_.update(devicelist)
@@ -584,7 +585,7 @@ class OMEMO(BaseModule):
         self.request_devicelist(jid)
 
     @as_task
-    def request_devicelist(self, jid: Optional[str] = None):
+    def request_devicelist(self, jid: str | None = None):
         _task = yield  # noqa: F841
 
         if jid is None:
@@ -650,6 +651,7 @@ class OMEMO(BaseModule):
         return compose_trust_uri(jid, verified_identities)
 
     def cleanup(self) -> None:
+        BaseModule.cleanup(self)
         self._backend.destroy()
         del self._backend
 
