@@ -1,51 +1,35 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
-from typing import Union
+from typing import Any
 
 from gi.repository import Gdk
-from gi.repository import Gio
 from gi.repository import Gtk
 
-from nbxmpp.protocol import JID
-
 from gajim.common import app
 from gajim.common import ged
+from gajim.common.client import Client
 from gajim.common.const import AvatarSize
-from gajim.common.events import AccountConnected
-from gajim.common.events import AccountDisconnected
+from gajim.common.const import ClientState
+from gajim.common.events import MucDecline
+from gajim.common.events import MucInvitation
 from gajim.common.events import SubscribePresenceReceived
 from gajim.common.events import UnsubscribedPresenceReceived
-from gajim.common.events import MucInvitation
-from gajim.common.events import MucDecline
-from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
 
-from .roster import Roster
-from .status_message_selector import StatusMessageSelector
-from .status_selector import StatusSelector
-from .notification_manager import NotificationManager
-from .builder import get_builder
-from .util import open_window
-from .util import EventHelper
-
-
-ROSTER_MENU_DICT = {
-    'show-offline': _('Show Offline Contacts'),
-    'sort-by-show': _('Sort by Status'),
-}
+from gajim.gtk.builder import get_builder
+from gajim.gtk.menus import get_account_menu
+from gajim.gtk.menus import get_account_notifications_menu
+from gajim.gtk.menus import get_roster_view_menu
+from gajim.gtk.notification_manager import NotificationManager
+from gajim.gtk.roster import Roster
+from gajim.gtk.status_message_selector import StatusMessageSelector
+from gajim.gtk.status_selector import StatusSelector
+from gajim.gtk.util import EventHelper
+from gajim.gtk.util import open_window
 
 
 class AccountPage(Gtk.Box, EventHelper):
@@ -54,37 +38,49 @@ class AccountPage(Gtk.Box, EventHelper):
         EventHelper.__init__(self)
 
         self._account = account
-        self._jid = app.get_jid_from_account(account)
         client = app.get_client(account)
-        self._contact = client.get_module('Contacts').get_contact(self._jid)
+        jid = client.get_own_jid().bare
+        self._contact = client.get_module('Contacts').get_contact(jid)
+        self._contact.connect('avatar-update', self._on_avatar_update)
 
         self._ui = get_builder('account_page.ui')
         self.add(self._ui.paned)
 
+        self._ui.our_jid_label.set_text(jid)
+
         self._status_selector = StatusSelector(account=account)
         self._status_selector.set_halign(Gtk.Align.CENTER)
-        self._ui.account_action_box.add(self._status_selector)
+        self._ui.status_box.add(self._status_selector)
 
         self._status_message_selector = StatusMessageSelector(account=account)
         self._status_message_selector.set_halign(Gtk.Align.CENTER)
-        self._ui.status_message_box.add(self._status_message_selector)
+        self._ui.status_box.add(self._status_message_selector)
 
         self._notification_manager = NotificationManager(account)
         self._ui.account_box.add(self._notification_manager)
 
+        self._ui.notifications_menu_button.set_menu_model(
+            get_account_notifications_menu(account))
+
         self._roster = Roster(account)
         self._ui.roster_box.add(self._roster)
 
         self._ui.paned.set_position(app.settings.get('chat_handle_position'))
         self._ui.paned.connect('button-release-event', self._on_button_release)
 
-        roster_menu = Gio.Menu()
-        for action, label in ROSTER_MENU_DICT.items():
-            roster_menu.append(label, f'win.{action}')
-        self._ui.roster_menu_button.set_menu_model(roster_menu)
+        self._ui.roster_menu_button.set_menu_model(get_roster_view_menu())
+        self._ui.account_page_menu_button.set_menu_model(
+            get_account_menu(account))
 
         self._ui.connect_signals(self)
 
+        client.connect_signal('state-changed', self._on_client_state_changed)
+
+        app.settings.connect_signal(
+            'account_label',
+            self._on_account_label_changed,
+            account)
+
         # pylint: disable=line-too-long
         self.register_events([
             ('subscribe-presence-received', ged.GUI1, self._subscribe_received),
@@ -92,8 +88,6 @@ class AccountPage(Gtk.Box, EventHelper):
              ged.GUI1, self._unsubscribed_received),
             ('muc-invitation', ged.GUI1, self._muc_invitation_received),
             ('muc-decline', ged.GUI1, self._muc_invitation_declined),
-            ('account-connected', ged.GUI2, self._on_account_state),
-            ('account-disconnected', ged.GUI2, self._on_account_state),
         ])
         # pylint: enable=line-too-long
 
@@ -102,28 +96,28 @@ class AccountPage(Gtk.Box, EventHelper):
         self.connect('destroy', self._on_destroy)
 
     def _on_destroy(self, _widget: AccountPage) -> None:
+        self._contact.disconnect_all_from_obj(self)
+        app.settings.disconnect_signals(self)
         app.check_finalize(self)
 
-    def _on_edit_profile(self, _button: Gtk.Button) -> None:
-        open_window('ProfileWindow', account=self._account)
+    def _on_account_label_changed(self, value: str, *args: Any) -> None:
+        self.update()
+
+    def _on_avatar_update(self, *args: Any) -> None:
+        self.update()
 
     def _on_account_settings(self, _button: Gtk.Button) -> None:
         window = open_window('AccountsWindow')
         window.select_account(self._account)
 
-    def _on_adhoc_commands(self, _button: Gtk.Button) -> None:
-        server_jid = JID.from_string(self._jid).domain
-        open_window('AdHocCommands', account=self._account, jid=server_jid)
+    def _on_client_state_changed(self,
+                                 client: Client,
+                                 _signal_name: str,
+                                 state: ClientState
+                                 ) -> None:
 
-    def _on_account_state(self,
-                          event: Union[AccountConnected, AccountDisconnected]
-                          ) -> None:
-
-        if event.account != self._account:
-            return
-
-        self._ui.adhoc_commands_button.set_sensitive(
-            app.account_is_connected(event.account))
+        jid = client.get_own_jid().bare
+        self._ui.our_jid_label.set_text(jid)
 
     def _on_search_changed(self, widget: Gtk.SearchEntry) -> None:
         text = widget.get_text().lower()
@@ -144,6 +138,7 @@ class AccountPage(Gtk.Box, EventHelper):
             self._account, 'account_label')
         self._ui.account_label.set_text(account_label)
 
+        assert isinstance(self._contact, BareContact)
         surface = self._contact.get_avatar(AvatarSize.ACCOUNT_PAGE,
                                            self.get_scale_factor(),
                                            add_show=False)
