@@ -1,30 +1,20 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Optional
 
 from gi.repository import Gtk
 
 from gajim.common import app
 from gajim.common import ged
 from gajim.common.const import AvatarSize
-from gajim.common.helpers import get_client_status
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import ResourceContact
+from gajim.common.util.status import get_client_status
 
 from gajim.gtk.util import EventHelper
 
@@ -76,7 +66,7 @@ class Account(Gtk.ListBoxRow, EventHelper):
         self.get_style_context().add_class('account-sidebar-item')
 
         self.account = account
-        self._account_class: Optional[str] = None
+        self._account_class: str | None = None
 
         self.register_events([
             ('account-enabled', ged.GUI1,
@@ -171,6 +161,7 @@ class AccountAvatar(Gtk.Image):
         self._update_image()
 
     def _update_image(self) -> None:
+        assert not isinstance(self._contact, ResourceContact)
         status = get_client_status(self._account)
         surface = app.app.avatar_storage.get_surface(
             self._contact,
