@@ -21,17 +21,7 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -53,7 +43,6 @@ from gi.repository import Gtk
 from nbxmpp import JID
 from nbxmpp.const import ConnectionProtocol
 from nbxmpp.const import ConnectionType
-from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import InvalidJid
 
 import gajim
@@ -69,16 +58,15 @@ from gajim.common.const import GAJIM_SUPPORT_JID
 from gajim.common.const import GAJIM_WIKI_URI
 from gajim.common.exceptions import GajimGeneralException
 from gajim.common.helpers import load_json
-from gajim.common.helpers import open_uri
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import ResourceContact
+from gajim.common.util.uri import open_uri
 
 from gajim.gtk import menus
 from gajim.gtk import structs
 from gajim.gtk.about import AboutDialog
 from gajim.gtk.accounts import AccountsWindow
 from gajim.gtk.avatar import AvatarStorage
-from gajim.gtk.builder import get_builder
 from gajim.gtk.const import ACCOUNT_ACTIONS
 from gajim.gtk.const import ALWAYS_ACCOUNT_ACTIONS
 from gajim.gtk.const import APP_ACTIONS
@@ -87,8 +75,10 @@ from gajim.gtk.const import MuteState
 from gajim.gtk.const import ONLINE_ACCOUNT_ACTIONS
 from gajim.gtk.dialogs import ConfirmationDialog
 from gajim.gtk.dialogs import DialogButton
+from gajim.gtk.dialogs import ErrorDialog
 from gajim.gtk.dialogs import ShortcutsWindow
 from gajim.gtk.discovery import ServiceDiscoveryWindow
+from gajim.gtk.menus import get_main_menu
 from gajim.gtk.start_chat import StartChatDialog
 from gajim.gtk.util import get_app_window
 from gajim.gtk.util import get_app_windows
@@ -107,7 +97,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
         flags = (Gio.ApplicationFlags.HANDLES_COMMAND_LINE |
                  Gio.ApplicationFlags.CAN_OVERRIDE_APP_ID)
         Gtk.Application.__init__(self,
-                                 application_id='org.gajim.Gajim',
+                                 application_id=app.get_default_app_id(),
                                  flags=flags)
 
         # required to track screensaver state
@@ -203,14 +193,15 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
         self.interface = None
 
-        GLib.set_prgname('org.gajim.Gajim')
+        GLib.set_prgname(app.get_default_app_id())
         if GLib.get_application_name() != 'Gajim':
             GLib.set_application_name('Gajim')
 
     @staticmethod
     def _get_remaining_entry():
         option = GLib.OptionEntry()
-        option.arg = GLib.OptionArg.STRING_ARRAY
+        # https://gitlab.gnome.org/GNOME/pygobject/-/issues/608
+        option.arg = int(GLib.OptionArg.STRING_ARRAY)  # pyright: ignore
         option.arg_data = None
         option.arg_description = ('[URI …]')
         option.flags = GLib.OptionFlags.NONE
@@ -224,15 +215,18 @@ class GajimApplication(Gtk.Application, CoreApplication):
             # to render colored emoji glyphs
             os.environ['PANGOCAIRO_BACKEND'] = 'fontconfig'
 
-        self._init_core()
+        app.ged.register_event_handler(
+            'db-migration', 0, self._on_db_migration)
+
+        if not self._init_core():
+            return
 
         Gtk.IconSize.register('100', 100, 100)
         icon_theme = Gtk.IconTheme.get_default()
         icon_theme.append_search_path(str(configpaths.get('ICONS')))
         load_user_iconsets()
 
-        builder = get_builder('application_menu.ui')
-        self.set_menubar(cast(Gio.Menu, builder.get_object('menubar')))
+        self.set_menubar(get_main_menu())
 
         from gajim.gtk import notification
         notification.init()
@@ -279,16 +273,15 @@ class GajimApplication(Gtk.Application, CoreApplication):
             app.log('uri_handler').info('open %s', uri)
             if not uri.startswith('xmpp:'):
                 continue
-            # remove xmpp:
-            uri = uri[5:]
-            try:
-                jid, cmd = uri.split('?')
-            except ValueError:
-                # No query argument
-                jid, cmd = uri, 'message'
 
             try:
-                jid = JID.from_string(jid)
+                iri, cmd = uri.split('?')
+            except ValueError:
+                # No query argument
+                iri, cmd = uri, 'message'
+
+            try:
+                jid = JID.from_iri(iri)
             except InvalidJid as error:
                 app.log('uri_handler').warning('Invalid JID %s: %s', uri, error)
                 continue
@@ -491,10 +484,20 @@ class GajimApplication(Gtk.Application, CoreApplication):
         for action_name in ONLINE_ACCOUNT_ACTIONS:
             self.set_action_state(f'{account}-{action_name}', new_state)
 
+        # Disable all feature actions on disconnect
         if not new_state:
             for action_name in FEATURE_ACCOUNT_ACTIONS:
                 self.set_action_state(f'{account}-{action_name}', new_state)
 
+    def update_feature_actions_state(self, account: str) -> None:
+        client = app.get_client(account)
+        mam_available = client.get_module('MAM').available
+        blocking_available = client.get_module('Blocking').supported
+
+        self.set_action_state(f'{account}-archive', mam_available)
+        self.set_action_state(f'{account}-blocking', blocking_available)
+        self.set_action_state(f'{account}-block-contact', blocking_available)
+
     def update_app_actions_state(self) -> None:
         active_accounts = bool(app.get_connected_accounts(exclude_local=True))
         self.set_action_state('create-groupchat', active_accounts)
@@ -519,19 +522,11 @@ class GajimApplication(Gtk.Application, CoreApplication):
             self.set_accels_for_action(action, accels)
 
     def _on_feature_discovered(self, event: events.FeatureDiscovered) -> None:
-        if event.feature == Namespace.MAM_2:
-            action = '%s-archive' % event.account
-            self.set_action_state(action, True)
-        elif event.feature == Namespace.BLOCKING:
-            action = '%s-blocking' % event.account
-            self.set_action_state(action, True)
-            action = '%s-block-contact' % event.account
-            self.set_action_state(action, True)
+        self.update_feature_actions_state(event.account)
 
     def create_account(self,
                        account: str,
-                       username: str | None,
-                       domain: str,
+                       address: JID,
                        password: str,
                        proxy_name: str | None,
                        custom_host: tuple[str,
@@ -542,8 +537,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
         CoreApplication.create_account(self,
                                        account,
-                                       username,
-                                       domain,
+                                       address,
                                        password,
                                        proxy_name,
                                        custom_host,
@@ -588,6 +582,9 @@ class GajimApplication(Gtk.Application, CoreApplication):
         if window is not None:
             window.remove_account(account)
 
+    def _on_db_migration(self, _event: events.DBMigration) -> None:
+        open_window('DBMigration')
+
     # Action Callbacks
 
     @staticmethod
@@ -646,7 +643,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
     def _on_services_action(_action: Gio.SimpleAction,
                             param: GLib.Variant) -> None:
         account = param.get_string()
-        server_jid = app.settings.get_account_setting(account, 'hostname')
+        server_jid = app.get_hostname_from_account(account)
         if account not in app.interface.instances:
             app.interface.instances[account] = {'disco': {}}
 
@@ -657,8 +654,8 @@ class GajimApplication(Gtk.Application, CoreApplication):
             try:
                 # Object will add itself to the window dict
                 ServiceDiscoveryWindow(account, address_entry=True)
-            except GajimGeneralException:
-                pass
+            except (GajimGeneralException, RuntimeError) as e:
+                ErrorDialog(_('Error'), str(e))
 
     @staticmethod
     def _on_create_groupchat_action(_action: Gio.SimpleAction,
@@ -723,7 +720,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
     @staticmethod
     def _on_xml_console_action(_action: Gio.SimpleAction,
                                _param: GLib.Variant | None) -> None:
-        open_window('XMLConsoleWindow')
+        open_window('DebugConsoleWindow')
 
     @staticmethod
     def _on_manage_proxies_action(_action: Gio.SimpleAction,
@@ -852,7 +849,8 @@ class GajimApplication(Gtk.Application, CoreApplication):
                                   params: structs.RemoveHistoryActionParams
                                   ) -> None:
         def _remove() -> None:
-            app.storage.archive.remove_history(params.account, params.jid)
+            app.storage.archive.remove_history_for_jid(
+                params.account, params.jid)
 
             app.window.clear_chat_list_row(params.account, params.jid)
             control = app.window.get_control()
@@ -883,4 +881,4 @@ class GajimApplication(Gtk.Application, CoreApplication):
         client.get_module('MUC').leave(params.jid)
         client.get_module('Bookmarks').remove(params.jid)
 
-        app.storage.archive.remove_history(params.account, params.jid)
+        app.storage.archive.remove_history_for_jid(params.account, params.jid)
