@@ -1,25 +1,13 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
-from typing import Optional
-from typing import Union
-
 import functools
 import hashlib
 import logging
+import math
 from collections import defaultdict
 from math import pi
 from pathlib import Path
@@ -36,8 +24,10 @@ from gajim.common import configpaths
 from gajim.common import types
 from gajim.common.const import AvatarSize
 from gajim.common.const import StyleAttr
-from gajim.common.helpers import get_groupchat_name
 from gajim.common.util.classes import Singleton
+from gajim.common.util.image import get_pixbuf_from_file
+from gajim.common.util.image import scale_with_ratio
+from gajim.common.util.muc import get_groupchat_name
 
 from gajim.gtk.const import DEFAULT_WORKSPACE_COLOR
 from gajim.gtk.util import convert_rgb_string_to_float
@@ -45,17 +35,18 @@ from gajim.gtk.util import get_contact_color
 from gajim.gtk.util import get_css_show_class
 from gajim.gtk.util import get_first_graphemes
 from gajim.gtk.util import load_icon_surface
-from gajim.gtk.util import load_pixbuf
 from gajim.gtk.util import make_rgba
 from gajim.gtk.util import rgba_to_float
-from gajim.gtk.util import scale_with_ratio
 
 log = logging.getLogger('gajim.gtk.avatar')
 
 
-AvatarCacheT = dict[Union[JID, str], dict[tuple[int, int, Optional[str]],
+AvatarCacheT = dict[JID | str, dict[tuple[int, int, str | None, str | None],
                                           cairo.ImageSurface]]
 
+CIRCLE_RATIO = 0.18
+CIRCLE_FILL_RATIO = 0.80
+
 
 def generate_avatar_letter(text: str) -> str:
     return get_first_graphemes(text.lstrip(), 1).upper()
@@ -133,6 +124,54 @@ def make_workspace_avatar(letter: str,
     return clip(surface, style)
 
 
+def add_transport_to_avatar(surface: cairo.ImageSurface,
+                            transport_icon: str,
+                            ) -> cairo.ImageSurface:
+
+    scale = surface.get_device_scale()[0]
+    width = surface.get_width()
+    height = surface.get_height()
+
+    transport_surface = load_icon_surface(
+        transport_icon,
+        math.floor(width / scale * CIRCLE_RATIO * 2),
+        int(scale))
+    if transport_surface is None:
+        return surface
+
+    new_surface = cairo.ImageSurface(cairo.Format.ARGB32, width, height)
+    new_surface.set_device_scale(*surface.get_device_scale())
+
+    context = cairo.Context(new_surface)
+    context.set_source_surface(surface, 0, 0)
+    context.paint()
+
+    width = width / scale
+
+    clip_radius = width * CIRCLE_RATIO
+    center_x = clip_radius
+    center_y = clip_radius
+
+    context.set_source_rgb(255, 255, 255)
+    context.set_operator(cairo.Operator.CLEAR)
+    context.arc(center_x, center_y, clip_radius, 0, 2 * pi)
+    context.fill()
+
+    clip_radius *= CIRCLE_FILL_RATIO
+    t_width = transport_surface.get_width() / scale
+    t_height = transport_surface.get_height() / scale
+
+    context.set_source_surface(
+        transport_surface,
+        center_x - t_width / 2,
+        center_y - t_height / 2)
+    context.set_operator(cairo.Operator.OVER)
+    context.arc(center_x, center_y, clip_radius, 0, 2 * pi)
+    context.fill()
+
+    return context.get_target()
+
+
 def add_status_to_avatar(surface: cairo.ImageSurface,
                          show: str
                          ) -> cairo.ImageSurface:
@@ -153,7 +192,7 @@ def add_status_to_avatar(surface: cairo.ImageSurface,
     width = width / scale
     height = height / scale
 
-    clip_radius = width / 5.5
+    clip_radius = width * CIRCLE_RATIO
     center_x = width - clip_radius
     center_y = height - clip_radius
 
@@ -166,7 +205,7 @@ def add_status_to_avatar(surface: cairo.ImageSurface,
     color = convert_rgb_string_to_float(
         app.css_config.get_value(css_color, StyleAttr.COLOR))
 
-    show_radius = clip_radius * 0.75
+    show_radius = clip_radius * CIRCLE_FILL_RATIO
 
     context.set_source_rgb(*color)
     context.set_operator(cairo.Operator.OVER)
@@ -185,8 +224,47 @@ def add_status_to_avatar(surface: cairo.ImageSurface,
     return context.get_target()
 
 
+def merge_avatars(
+    avatar1: cairo.ImageSurface,
+    avatar2: cairo.ImageSurface,
+) -> cairo.ImageSurface:
+    '''
+    Includes avatar2 in the bottom right corner of avatar1
+    '''
+    new_surface = cairo.ImageSurface(
+        cairo.Format.ARGB32,
+        avatar1.get_width(),
+        avatar1.get_height(),
+    )
+    context = cairo.Context(new_surface)
+    context.set_source_surface(avatar1, 0, 0)
+    context.paint()
+
+    context.scale(0.5, 0.5)
+    context.set_source_rgb(255, 255, 255)
+    context.set_operator(cairo.Operator.CLEAR)
+    context.arc(
+        avatar1.get_width() * 1.5,
+        avatar1.get_height() * 1.5,
+        avatar1.get_width() / CIRCLE_FILL_RATIO / 2,
+        0,
+        2 * pi,
+    )
+    context.fill()
+
+    context = cairo.Context(new_surface)
+    context.scale(0.5, 0.5)
+    context.set_source_surface(
+        avatar2,
+        avatar1.get_width(),
+        avatar1.get_height(),
+    )
+    context.paint()
+    return new_surface
+
+
 @functools.lru_cache(maxsize=128)
-def get_show_circle(show: Union[str, types.PresenceShowT],
+def get_show_circle(show: str | types.PresenceShowT,
                     size: int,
                     scale: int
                     ) -> cairo.ImageSurface:
@@ -194,17 +272,20 @@ def get_show_circle(show: Union[str, types.PresenceShowT],
     if not isinstance(show, str):
         show = show.value
 
-    size = size * scale
-    center = size / 2
-    radius = size / 3
+    width = size * scale
+    height = width
 
-    surface = cairo.ImageSurface(cairo.Format.ARGB32, size, size)
+    surface = cairo.ImageSurface(cairo.Format.ARGB32, width, height)
+    surface.set_device_scale(scale, scale)
     context = cairo.Context(surface)
 
     css_color = get_css_show_class(show)
     color = convert_rgb_string_to_float(
         app.css_config.get_value(css_color, StyleAttr.COLOR))
 
+    center = size / 2
+    radius = size / 3
+
     context.set_source_rgb(*color)
     context.set_operator(cairo.Operator.OVER)
     context.arc(center, center, radius, 0, 2 * pi)
@@ -347,19 +428,19 @@ class AvatarStorage(metaclass=Singleton):
     def __init__(self):
         self._cache: AvatarCacheT = defaultdict(dict)
 
-    def invalidate_cache(self, jid: Union[JID, str]) -> None:
+    def invalidate_cache(self, jid: JID | str) -> None:
         self._cache.pop(jid, None)
 
     def get_pixbuf(self,
-                   contact: Union[types.BareContact,
-                                  types.GroupchatContact,
-                                  types.GroupchatParticipant],
+                   contact: (types.BareContact |
+                             types.GroupchatContact |
+                             types.GroupchatParticipant),
                    size: int,
                    scale: int,
-                   show: Optional[str] = None,
+                   show: str | None = None,
                    default: bool = False,
-                   transport_icon: Optional[str] = None,
-                   style: str = 'circle') -> Optional[GdkPixbuf.Pixbuf]:
+                   transport_icon: str | None = None,
+                   style: str = 'circle') -> GdkPixbuf.Pixbuf | None:
 
         surface = self.get_surface(
             contact, size, scale, show, default, transport_icon, style)
@@ -370,28 +451,20 @@ class AvatarStorage(metaclass=Singleton):
                                            size * scale)
 
     def get_surface(self,
-                    contact: Union[types.BareContact,
-                                   types.GroupchatContact,
-                                   types.GroupchatParticipant],
+                    contact: (types.BareContact |
+                              types.GroupchatContact |
+                              types.GroupchatParticipant),
                     size: int,
                     scale: int,
-                    show: Optional[str] = None,
+                    show: str | None = None,
                     default: bool = False,
-                    transport_icon: Optional[str] = None,
+                    transport_icon: str | None = None,
                     style: str = 'circle') -> cairo.ImageSurface:
 
         jid = contact.jid
 
-        if transport_icon is not None:
-            surface = load_icon_surface(transport_icon, size, scale)
-            assert surface is not None
-            if show is not None:
-                surface = add_status_to_avatar(surface, show)
-            self._cache[jid][(size, scale, show)] = surface
-            return surface
-
         if not default:
-            surface = self._cache[jid].get((size, scale, show))
+            surface = self._cache[jid].get((size, scale, show, transport_icon))
             if surface is not None:
                 return surface
 
@@ -399,7 +472,11 @@ class AvatarStorage(metaclass=Singleton):
             if surface is not None:
                 if show is not None:
                     surface = add_status_to_avatar(surface, show)
-                self._cache[jid][(size, scale, show)] = surface
+
+                if transport_icon is not None:
+                    surface = add_transport_to_avatar(surface, transport_icon)
+
+                self._cache[jid][(size, scale, show, transport_icon)] = surface
                 return surface
 
         name = contact.name
@@ -409,7 +486,11 @@ class AvatarStorage(metaclass=Singleton):
             letter, color, size, scale, style=style)
         if show is not None:
             surface = add_status_to_avatar(surface, show)
-        self._cache[jid][(size, scale, show)] = surface
+
+        if transport_icon is not None:
+            surface = add_transport_to_avatar(surface, transport_icon)
+
+        self._cache[jid][(size, scale, show, transport_icon)] = surface
         return surface
 
     def get_muc_surface(self,
@@ -418,17 +499,11 @@ class AvatarStorage(metaclass=Singleton):
                         size: int,
                         scale: int,
                         default: bool = False,
-                        transport_icon: Optional[str] = None,
-                        style: str = 'circle') -> Optional[cairo.ImageSurface]:
-
-        if transport_icon is not None:
-            surface = load_icon_surface(transport_icon, size, scale)
-            assert surface is not None
-            self._cache[jid][(size, scale, None)] = surface
-            return surface
+                        transport_icon: str | None = None,
+                        style: str = 'circle') -> cairo.ImageSurface:
 
         if not default:
-            surface = self._cache[jid].get((size, scale, None))
+            surface = self._cache[jid].get((size, scale, None, transport_icon))
             if surface is not None:
                 return surface
 
@@ -436,8 +511,15 @@ class AvatarStorage(metaclass=Singleton):
             if avatar_sha is not None:
                 surface = self.surface_from_filename(avatar_sha, size, scale)
                 if surface is not None:
+                    # clip first to avoid clipping the transport icon
                     surface = clip(surface, style)
-                    self._cache[jid][(size, scale, None)] = surface
+
+                    if transport_icon is not None:
+                        surface = add_transport_to_avatar(
+                            surface, transport_icon)
+
+                    self._cache[jid][
+                        (size, scale, None, transport_icon)] = surface
                     return surface
 
                 # avatar_sha set, but image is missing
@@ -451,15 +533,18 @@ class AvatarStorage(metaclass=Singleton):
         color = get_contact_color(contact)
         letter = generate_avatar_letter(name)
         surface = generate_default_avatar(letter, color, size, scale, style)
-        self._cache[jid][(size, scale, None)] = surface
+        if transport_icon is not None:
+            surface = add_transport_to_avatar(surface, transport_icon)
+
+        self._cache[jid][(size, scale, None, transport_icon)] = surface
         return surface
 
     def get_workspace_surface(self,
                               workspace_id: str,
                               size: int,
-                              scale: int) -> Optional[cairo.ImageSurface]:
+                              scale: int) -> cairo.ImageSurface | None:
 
-        surface = self._cache[workspace_id].get((size, scale, None))
+        surface = self._cache[workspace_id].get((size, scale, None, None))
         if surface is not None:
             return surface
 
@@ -468,7 +553,7 @@ class AvatarStorage(metaclass=Singleton):
         avatar_sha = app.settings.get_workspace_setting(
             workspace_id, 'avatar_sha')
         if avatar_sha:
-            surface = self._load_surface_from_storage(avatar_sha, size, scale)
+            surface = self.surface_from_filename(avatar_sha, size, scale)
             if surface is not None:
                 return clip(surface, 'round-corners')
 
@@ -479,12 +564,12 @@ class AvatarStorage(metaclass=Singleton):
         rgba = make_rgba(color or DEFAULT_WORKSPACE_COLOR)
         surface = make_workspace_avatar(
             name, rgba_to_float(rgba), size, scale)
-        self._cache[workspace_id][(size, scale, None)] = surface
+        self._cache[workspace_id][(size, scale, None, None)] = surface
         return surface
 
     @staticmethod
-    def _load_for_publish(path: str) -> Optional[tuple[bool, bytes]]:
-        pixbuf = load_pixbuf(path)
+    def _load_for_publish(path: str) -> tuple[bool, bytes] | None:
+        pixbuf = get_pixbuf_from_file(path)
         if pixbuf is None:
             return None
 
@@ -501,7 +586,7 @@ class AvatarStorage(metaclass=Singleton):
         return pixbuf.save_to_bufferv('png', [], [])
 
     @staticmethod
-    def save_avatar(data: bytes) -> Optional[str]:
+    def save_avatar(data: bytes) -> str | None:
         '''
         Save an avatar to the harddisk
 
@@ -516,12 +601,12 @@ class AvatarStorage(metaclass=Singleton):
             with open(path, 'wb') as output_file:
                 output_file.write(data)
         except Exception:
-            log.error('Storing avatar failed', exc_info=True)
+            log.exception('Storing avatar failed')
             return None
         return sha
 
     @staticmethod
-    def get_avatar_path(filename: str) -> Optional[Path]:
+    def get_avatar_path(filename: str) -> Path | None:
         path = configpaths.get('AVATAR') / filename
         if not path.is_file():
             return None
@@ -533,49 +618,33 @@ class AvatarStorage(metaclass=Singleton):
     def surface_from_filename(self,
                               filename: str,
                               size: int,
-                              scale: int) -> Optional[cairo.ImageSurface]:
+                              scale: int) -> cairo.ImageSurface | None:
 
         size = size * scale
         path = self.get_avatar_path(filename)
         if path is None:
             return None
 
-        pixbuf = load_pixbuf(path, size)
+        pixbuf = get_pixbuf_from_file(path, size)
         if pixbuf is None:
             return None
 
         surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf, scale)
         return fit(surface, size)
 
-    def _load_surface_from_storage(self,
-                                   filename: str,
-                                   size: int,
-                                   scale: int) -> Optional[cairo.ImageSurface]:
-
-        size = size * scale
-        path = self.get_avatar_path(filename)
-        if path is None:
-            return None
-
-        pixbuf = load_pixbuf(path, size)
-        if pixbuf is None:
-            return None
-        surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf, scale)
-        return fit(surface, size)
-
     def _get_avatar_from_storage(self,
-                                 contact: Union[types.BareContact,
-                                                types.GroupchatContact,
-                                                types.GroupchatParticipant],
+                                 contact: (types.BareContact |
+                                           types.GroupchatContact |
+                                           types.GroupchatParticipant),
                                  size: int,
                                  scale: int,
-                                 style: str) -> Optional[cairo.ImageSurface]:
+                                 style: str) -> cairo.ImageSurface | None:
 
         avatar_sha = contact.avatar_sha
         if avatar_sha is None:
             return None
 
-        surface = self._load_surface_from_storage(avatar_sha, size, scale)
+        surface = self.surface_from_filename(avatar_sha, size, scale)
         if surface is None:
             return None
         return clip(surface, style)
