@@ -1,37 +1,44 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
-from typing import Optional
+from enum import Enum
+from pathlib import Path
 
 from gi.repository import GObject
 from gi.repository import Gtk
 from gi.repository import Pango
-
+from nbxmpp.modules.dataforms import SimpleDataForm
 from nbxmpp.protocol import InvalidJid
 from nbxmpp.protocol import JID
 from nbxmpp.protocol import validate_resourcepart
 
 from gajim.common import app
-from gajim.common.client import Client
+from gajim.common import types
+from gajim.common.const import SimpleClientState
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
-from gajim.common.types import ChatContactT
 
-from .dataform import DataFormWidget
-from .groupchat_inviter import GroupChatInviter
+from gajim.gtk.dataform import DataFormWidget
+from gajim.gtk.file_transfer_selector import FileTransferSelector
+from gajim.gtk.groupchat_inviter import GroupChatInviter
+
+
+class FunctionMode(Enum):
+    INVITE = 'invite'
+    SEND_FILE = 'send-file'
+    CHANGE_NICKNAME = 'change-nickname'
+    KICK = 'kick'
+    BAN = 'ban'
+    PASSWORD_REQUEST = 'password-request'  # noqa: S105
+    CAPTCHA_REQUEST = 'captcha-request'
+    CAPTCHA_ERROR = 'captcha-error'
+    JOIN_FAILED = 'join-failed'
+    CREATION_FAILED = 'creation-failed'
+    CONFIG_FAILED = 'config-failed'
 
 
 class ChatFunctionPage(Gtk.Box):
@@ -49,11 +56,13 @@ class ChatFunctionPage(Gtk.Box):
         self.set_valign(Gtk.Align.CENTER)
         self.get_style_context().add_class('padding-18')
 
-        self._client: Optional[Client] = None
-        self._contact: Optional[ChatContactT] = None
-        self._mode: Optional[str] = None
+        self._client: types.Client | None = None
+        self._contact: types.ChatContactT | None = None
+        self._mode: FunctionMode | None = None
+        self._data: str | None = None
+        self._ready_state = True
 
-        self._widget: Optional[Gtk.Widget] = None
+        self._widget: Gtk.Widget | None = None
 
         self._heading = Gtk.Label()
         self._heading.set_max_width_chars(30)
@@ -87,22 +96,21 @@ class ChatFunctionPage(Gtk.Box):
 
     def process_escape(self) -> None:
         close_control = self._mode in (
-            'join-failed',
-            'creation-failed',
-            'config-failed',
-            'captcha-error')
+            FunctionMode.CREATION_FAILED,
+            FunctionMode.CONFIG_FAILED,
+            FunctionMode.CAPTCHA_ERROR)
         self.emit('finish', close_control)
 
-    def _clear(self) -> None:
+    def _reset(self) -> None:
+        if self._client is not None:
+            self._client.disconnect_all_from_obj(self)
+
         for child in self._content_box.get_children():
             child.destroy()
 
-    def set_mode(self,
-                 contact: ChatContactT,
-                 mode: str,
-                 data: Optional[str] = None
-                 ) -> None:
-        self._clear()
+        if self._widget is not None:
+            self._widget.destroy()
+
         self._confirm_button.get_style_context().remove_class(
             'destructive-action')
         self._confirm_button.get_style_context().remove_class(
@@ -110,18 +118,31 @@ class ChatFunctionPage(Gtk.Box):
         self._confirm_button.set_sensitive(False)
         self._confirm_button.grab_default()
 
+        self._forget_button.set_sensitive(True)
         self._forget_button.hide()
 
+        self._ready_state = True
+
+    def set_mode(self,
+                 contact: types.ChatContactT,
+                 mode: FunctionMode,
+                 data: str | None = None,
+                 files: list[str] | None = None
+                 ) -> None:
+
+        self._reset()
+
         self._contact = contact
-        self._heading.set_text(self._contact.name)
         self._client = app.get_client(contact.account)
+        self._client.connect_signal(
+            'state-changed', self._on_client_state_changed)
+
         self._mode = mode
         self._data = data
 
-        if self._widget is not None:
-            self._widget.destroy()
+        self._heading.set_text(self._contact.name)
 
-        if mode == 'invite':
+        if mode == FunctionMode.INVITE:
             self._confirm_button.set_label(_('Invite'))
             self._confirm_button.get_style_context().add_class(
                 'suggested-action')
@@ -130,41 +151,52 @@ class ChatFunctionPage(Gtk.Box):
             self._widget.connect('listbox-changed', self._on_ready)
             self._widget.load_contacts()
 
-        elif mode == 'change-nickname':
+        elif mode == FunctionMode.SEND_FILE:
+            self._confirm_button.set_label(_('Send Files'))
+            self._confirm_button.get_style_context().add_class(
+                'suggested-action')
+            self._widget = FileTransferSelector(self._contact, data)
+            self._widget.connect('changed', self._on_ready)
+            if files is not None:
+                self._widget.add_files(files)
+
+        elif mode == FunctionMode.CHANGE_NICKNAME:
             self._confirm_button.set_label(_('Change'))
             self._confirm_button.get_style_context().add_class(
                 'suggested-action')
             self._widget = InputWidget(self._contact, mode)
             self._widget.connect('changed', self._on_ready)
 
-        elif mode == 'kick':
+        elif mode == FunctionMode.KICK:
             self._confirm_button.set_label(_('Kick'))
             self._confirm_button.set_sensitive(True)
             self._confirm_button.get_style_context().add_class(
                 'destructive-action')
             self._widget = InputWidget(self._contact, mode, data)
 
-        elif mode == 'ban':
+        elif mode == FunctionMode.BAN:
             self._confirm_button.set_label(_('Ban'))
             self._confirm_button.set_sensitive(True)
             self._confirm_button.get_style_context().add_class(
                 'destructive-action')
             self._widget = InputWidget(self._contact, mode, data)
 
-        elif mode == 'password-request':
+        elif mode == FunctionMode.PASSWORD_REQUEST:
             self._confirm_button.set_label(_('Join'))
             self._confirm_button.get_style_context().add_class(
                 'suggested-action')
             self._widget = InputWidget(self._contact, mode)
             self._widget.connect('changed', self._on_ready)
 
-        elif mode == 'captcha-request':
+        elif mode == FunctionMode.CAPTCHA_REQUEST:
             self._confirm_button.set_label(_('Join'))
             self._confirm_button.get_style_context().add_class(
                 'suggested-action')
             muc_data = self._client.get_module('MUC').get_muc_data(
                 self._contact.jid)
+            assert muc_data is not None
             form = muc_data.captcha_form
+            assert form is not None
             options = {'no-scrolling': True,
                        'entry-activates-default': True}
             self._widget = DataFormWidget(form, options=options)
@@ -172,23 +204,29 @@ class ChatFunctionPage(Gtk.Box):
             self._widget.show_all()
             self._widget.connect('is-valid', self._on_ready)
 
-        elif mode == 'captcha-error':
+        elif mode == FunctionMode.CAPTCHA_ERROR:
             self._confirm_button.set_label(_('Try Again'))
             self._confirm_button.set_sensitive(True)
             self._confirm_button.get_style_context().add_class(
                 'suggested-action')
             self._widget = ErrorWidget(error_text=data)
 
-        elif mode in ('join-failed', 'creation-failed', 'config-failed'):
+        elif mode in (FunctionMode.JOIN_FAILED,
+                      FunctionMode.CREATION_FAILED,
+                      FunctionMode.CONFIG_FAILED):
             self._confirm_button.set_label(_('Try Again'))
             self._confirm_button.set_sensitive(True)
             self._confirm_button.get_style_context().add_class(
                 'suggested-action')
-            if mode == 'join-failed':
+            if mode == FunctionMode.JOIN_FAILED:
                 is_bookmark = self._client.get_module('Bookmarks').is_bookmark(
                     self._contact.jid)
                 self._forget_button.set_visible(is_bookmark)
-            self._widget = ErrorWidget(error_mode=mode, error_text=data)
+
+            error_text = str(self._contact.jid)
+            if data is not None:
+                error_text = f'{error_text}\n{data}'
+            self._widget = ErrorWidget(mode=mode, error_text=error_text)
 
         assert self._widget is not None
         self._content_box.add(self._widget)
@@ -196,65 +234,87 @@ class ChatFunctionPage(Gtk.Box):
             self._widget.focus()
         elif isinstance(self._widget, DataFormWidget):
             self._widget.focus_first_entry()
+        elif mode == FunctionMode.SEND_FILE:
+            self._confirm_button.grab_focus()
+
+    def _on_client_state_changed(self,
+                                 _client: types.Client,
+                                 _signal_name: str,
+                                 _state: SimpleClientState
+                                 ) -> None:
+
+        self._update_button_state()
 
     def _on_ready(self,
                   _widget: Gtk.Widget,
                   state: bool
                   ) -> None:
-        self._confirm_button.set_sensitive(state)
+
+        self._ready_state = state
+        self._update_button_state()
+
+    def _update_button_state(self) -> None:
+        assert self._contact is not None
+        if app.account_is_connected(self._contact.account):
+            self._confirm_button.set_sensitive(self._ready_state)
+            self._forget_button.set_sensitive(True)
+            return
+
+        self._confirm_button.set_sensitive(False)
+        self._forget_button.set_sensitive(False)
 
     def _on_confirm_clicked(self, _button: Gtk.Button) -> None:
-        if self._mode == 'invite':
+        assert self._client is not None
+        assert self._contact is not None
+
+        if self._mode == FunctionMode.INVITE:
             assert isinstance(self._widget, GroupChatInviter)
             invitees = self._widget.get_invitees()
             for jid in invitees:
                 self._invite(JID.from_string(jid))
 
-        elif self._mode == 'change-nickname':
+        elif self._mode == FunctionMode.SEND_FILE:
+            assert isinstance(self._widget, FileTransferSelector)
+            if self._widget.transfer_resource_required():
+                return
+
+            self._send_files(self._widget.get_catalog())
+
+        elif self._mode == FunctionMode.CHANGE_NICKNAME:
             assert isinstance(self._widget, InputWidget)
             nickname = self._widget.get_text()
-            assert self._client is not None
-            assert self._contact is not None
             self._client.get_module('MUC').change_nick(
                 self._contact.jid, nickname)
 
-        elif self._mode == 'kick':
+        elif self._mode == FunctionMode.KICK:
             assert isinstance(self._widget, InputWidget)
             reason = self._widget.get_text()
-            assert self._client is not None
-            assert self._contact is not None
             self._client.get_module('MUC').set_role(
                 self._contact.jid, self._data, 'none', reason)
 
-        elif self._mode == 'ban':
+        elif self._mode == FunctionMode.BAN:
             assert isinstance(self._widget, InputWidget)
             reason = self._widget.get_text()
-            assert self._client is not None
-            assert self._contact is not None
             self._client.get_module('MUC').set_affiliation(
                 self._contact.jid,
                 {self._data: {'affiliation': 'outcast', 'reason': reason}})
 
-        elif self._mode == 'password-request':
+        elif self._mode == FunctionMode.PASSWORD_REQUEST:
             assert isinstance(self._widget, InputWidget)
             password = self._widget.get_text()
-            assert self._client is not None
-            assert self._contact is not None
             self._client.get_module('MUC').set_password(
                 self._contact.jid, password)
             self._client.get_module('MUC').join(self._contact.jid)
 
-        elif self._mode == 'captcha-request':
+        elif self._mode == FunctionMode.CAPTCHA_REQUEST:
             assert isinstance(self._widget, DataFormWidget)
             form_node = self._widget.get_submit_form()
-            assert self._client is not None
-            assert self._contact is not None
+            assert isinstance(form_node, SimpleDataForm)
             self._client.get_module('MUC').send_captcha(
                 self._contact.jid, form_node)
 
-        elif self._mode in ('join-failed', 'captcha-error'):
-            assert self._client is not None
-            assert self._contact is not None
+        elif self._mode in (FunctionMode.JOIN_FAILED,
+                            FunctionMode.CAPTCHA_ERROR):
             self._client.get_module('MUC').join(self._contact.jid)
 
         self.emit('finish', False)
@@ -263,27 +323,28 @@ class ChatFunctionPage(Gtk.Box):
         assert self._client is not None
         assert self._contact is not None
 
-        if self._mode == 'captcha-request':
-            self._client.get_module('MUC').cancel_captcha(
-                self._contact.jid)
-            self.emit('finish', True)
-            return
+        connected = app.account_is_connected(self._contact.account)
+        close_control = False
 
-        if self._mode == 'password-request':
-            self._client.get_module('MUC').cancel_password_request(
-                self._contact.jid)
-            self.emit('finish', True)
-            return
+        if self._mode == FunctionMode.CAPTCHA_REQUEST:
+            if connected:
+                self._client.get_module('MUC').cancel_captcha(
+                    self._contact.jid)
+            close_control = True
 
-        if self._mode in (
-                'join-failed',
-                'creation-failed',
-                'config-failed',
-                'captcha-error'):
-            self.emit('finish', True)
-            return
+        elif self._mode == FunctionMode.PASSWORD_REQUEST:
+            if connected:
+                self._client.get_module('MUC').cancel_password_request(
+                    self._contact.jid)
+            close_control = True
 
-        self.emit('finish', False)
+        elif self._mode in (
+                FunctionMode.CREATION_FAILED,
+                FunctionMode.CONFIG_FAILED,
+                FunctionMode.CAPTCHA_ERROR):
+            close_control = True
+
+        self.emit('finish', close_control)
 
     def _on_forget_clicked(self, _button: Gtk.Button) -> None:
         assert self._client is not None
@@ -298,10 +359,29 @@ class ChatFunctionPage(Gtk.Box):
             self._contact.jid, invited_jid)
         invited_contact = client.get_module('Contacts').get_contact(
             invited_jid)
+        assert isinstance(invited_contact, BareContact)
         self.emit(
             'message',
             _('%s has been invited to this group chat') % invited_contact.name)
 
+    def _send_files(self, catalog: list[tuple[Path, str, JID]]) -> None:
+        assert self._contact is not None
+        client = app.get_client(self._contact.account)
+
+        # catalog: list[(file Path, transfer method, recipient JID)]
+        for path, method, jid in catalog:
+            if method == 'httpupload':
+                client.get_module('HTTPUpload').send_file(
+                    self._contact, path)
+                continue
+
+            # Send using Jingle
+            app.interface.instances['file_transfers'].send_file(
+                self._contact.account,
+                self._contact,
+                jid,
+                str(path))
+
 
 class InputWidget(Gtk.Box):
 
@@ -310,10 +390,11 @@ class InputWidget(Gtk.Box):
     }
 
     def __init__(self,
-                 contact: ChatContactT,
-                 mode: str,
-                 data: Optional[str] = None
+                 contact: types.ChatContactT,
+                 mode: FunctionMode,
+                 data: str | None = None
                  ) -> None:
+
         Gtk.Box.__init__(self,
                          orientation=Gtk.Orientation.VERTICAL,
                          spacing=12)
@@ -336,21 +417,21 @@ class InputWidget(Gtk.Box):
         self._entry.connect('changed', self._on_entry_changed)
         self.add(self._entry)
 
-        if mode == 'change-nickname':
+        if mode == FunctionMode.CHANGE_NICKNAME:
             heading_label.set_text(_('Change Nickname'))
             sub_label.set_text(_('Enter your new nickname'))
             assert isinstance(self._contact, GroupchatContact)
             self._entry.set_text(self._contact.nickname or '')
 
-        elif mode == 'kick':
+        elif mode == FunctionMode.KICK:
             heading_label.set_text(_('Kick %s') % data)
             sub_label.set_text(_('Reason (optional)'))
 
-        elif mode == 'ban':
+        elif mode == FunctionMode.BAN:
             heading_label.set_text(_('Ban %s') % data)
             sub_label.set_text(_('Reason (optional)'))
 
-        elif mode == 'password-request':
+        elif mode == FunctionMode.PASSWORD_REQUEST:
             heading_label.set_text(_('Password Required'))
             sub_label.set_text(_('Enter a password to join this chat'))
             self._entry.set_input_purpose(Gtk.InputPurpose.PASSWORD)
@@ -365,7 +446,7 @@ class InputWidget(Gtk.Box):
     def _on_entry_changed(self, entry: Gtk.Entry) -> None:
         text = entry.get_text()
 
-        if self._mode == 'change-nickname':
+        if self._mode == FunctionMode.CHANGE_NICKNAME:
             assert isinstance(self._contact, GroupchatContact)
             if not text or text == self._contact.nickname:
                 self.emit('changed', False)
@@ -387,9 +468,10 @@ class InputWidget(Gtk.Box):
 
 class ErrorWidget(Gtk.Box):
     def __init__(self,
-                 error_mode: Optional[str] = None,
-                 error_text: Optional[str] = None
+                 mode: FunctionMode | None = None,
+                 error_text: str | None = None
                  ) -> None:
+
         Gtk.Box.__init__(self,
                          orientation=Gtk.Orientation.VERTICAL,
                          spacing=12)
@@ -400,11 +482,11 @@ class ErrorWidget(Gtk.Box):
         heading = Gtk.Label()
         heading.get_style_context().add_class('bold16')
         heading_text = _('An Error Occurred')
-        if error_mode == 'join-failed':
+        if mode == FunctionMode.JOIN_FAILED:
             heading_text = _('Failed to Join Group Chat')
-        elif error_mode == 'creation-failed':
+        elif mode == FunctionMode.CREATION_FAILED:
             heading_text = _('Failed to Create Group Chat')
-        elif error_mode == 'config-failed':
+        elif mode == FunctionMode.CONFIG_FAILED:
             heading_text = _('Failed to Configure Group Chat')
         heading.set_text(heading_text)
 
