@@ -1,49 +1,47 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 import logging
 import sys
-import time
+from functools import partial
 from urllib.parse import urlparse
 
 from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
+from nbxmpp.errors import StanzaError
 from nbxmpp.protocol import JID
+from nbxmpp.task import Task
 
 from gajim.common import app
 from gajim.common import events
-from gajim.common import helpers
-from gajim.common import preview_helpers
 from gajim.common.commands import ChatCommands
 from gajim.common.const import CallType
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
-from gajim.common.preview_helpers import format_geo_coords
-from gajim.common.preview_helpers import split_geo_uri
+from gajim.common.storage.archive.const import ChatDirection
+from gajim.common.storage.archive.const import MessageType
+from gajim.common.storage.archive.models import Message
 from gajim.common.structs import OutgoingMessage
 from gajim.common.types import ChatContactT
+from gajim.common.util.muc import message_needs_highlight
+from gajim.common.util.preview import filename_from_uri
+from gajim.common.util.preview import format_geo_coords
+from gajim.common.util.preview import guess_simple_file_type
+from gajim.common.util.preview import split_geo_uri
 from gajim.common.util.text import remove_invalid_xml_chars
 
 from gajim.gtk.chat_banner import ChatBanner
 from gajim.gtk.chat_function_page import ChatFunctionPage
 from gajim.gtk.chat_function_page import FunctionMode
+from gajim.gtk.const import DND_TARGET_FLATPAK
+from gajim.gtk.const import DND_TARGET_URI_LIST
 from gajim.gtk.const import TARGET_TYPE_URI_LIST
 from gajim.gtk.control import ChatControl
 from gajim.gtk.dialogs import ErrorDialog
@@ -84,15 +82,8 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         box.add(self._chat_banner)
-        separator1 = Gtk.Separator()
-        separator1.set_margin_start(6)
-        separator1.set_margin_end(6)
-        box.add(separator1)
+        box.add(Gtk.Separator(margin_start=6, margin_end=6))
         box.add(self._chat_control.widget)
-        separator2 = Gtk.Separator()
-        separator2.set_margin_start(6)
-        separator2.set_margin_end(6)
-        box.add(separator2)
         box.add(self._message_action_box)
 
         dnd_icon = Gtk.Image.new_from_icon_name(
@@ -122,9 +113,9 @@ class ChatStack(Gtk.Stack, EventHelper):
         overlay.connect('drag-leave', self._on_drag_leave)
 
         if app.is_flatpak():
-            target = 'application/vnd.portal.files'
+            target = DND_TARGET_FLATPAK
         else:
-            target = 'text/uri-list'
+            target = DND_TARGET_URI_LIST
 
         uri_entry = Gtk.TargetEntry.new(
             target,
@@ -151,7 +142,6 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         self.register_events([
             ('message-received', 85, self._on_message_received),
-            ('gc-message-received', 85, self._on_message_received),
             ('muc-disco-update', 85, self._on_muc_disco_update),
             ('account-connected', 85, self._on_account_state),
             ('account-disconnected', 85, self._on_account_state),
@@ -166,11 +156,7 @@ class ChatStack(Gtk.Stack, EventHelper):
             self._chat_function_page.process_escape()
             return True
 
-        if self._message_action_box.is_correcting:
-            self._message_action_box.toggle_message_correction()
-            return True
-
-        return False
+        return self._message_action_box.process_escape()
 
     def get_chat_control(self) -> ChatControl:
         return self._chat_control
@@ -373,35 +359,57 @@ class ChatStack(Gtk.Stack, EventHelper):
             self._update_chat_actions(self._current_contact)
 
     def _on_message_received(self, event: events.MessageReceived) -> None:
-        if not event.msgtxt or event.properties.is_sent_carbon:
+        if event.from_mam:
             return
 
-        client = app.get_client(event.account)
-        contact = client.get_module('Contacts').get_contact(event.jid)
-        if isinstance(contact, GroupchatContact):
+        if app.window.is_chat_active(event.account, event.jid):
+            if event.message.id is None:
+                return
+
+            client = app.get_client(event.account)
+            contact = client.get_module('Contacts').get_contact(event.jid)
+            assert isinstance(
+                contact, BareContact | GroupchatContact | GroupchatParticipant)
+            client.get_module('ChatMarkers').send_displayed_marker(
+                contact,
+                event.message.id,
+                event.message.stanza_id)
+            return
+
+        message = event.message
+        if message.direction == ChatDirection.OUTGOING:
+            return
+
+        if event.m_type == MessageType.GROUPCHAT:
+            client = app.get_client(event.account)
+            contact = client.get_module('Contacts').get_contact(
+                event.jid, groupchat=True)
+            assert isinstance(contact, GroupchatContact)
             # MUC messages may be received after some delay, so make sure we
             # don't issue notifications for our own messages.
             self_contact = contact.get_self()
             if (self_contact is not None and
-                    self_contact.name == event.properties.muc_nickname):
+                    self_contact.name == message.resource):
                 return
 
-        if app.window.is_chat_active(event.account, event.jid):
-            return
+        self._issue_notification(event.account, message)
 
-        self._issue_notification(event)
+    def _issue_notification(
+        self,
+        account: str,
+        data: Message
+    ) -> None:
 
-    def _issue_notification(self, event: events.MessageReceived) -> None:
-        text = event.msgtxt
-        tim = event.properties.timestamp
-        additional_data = event.additional_data
-        client = app.get_client(event.account)
-        contact = client.get_module('Contacts').get_contact(event.jid)
+        text = data.text
+        assert text is not None
+
+        client = app.get_client(account)
+        contact = client.get_module('Contacts').get_contact(data.remote.jid)
 
         title = _('New message from')
 
         is_previewable = app.preview_manager.is_previewable(
-            text, additional_data)
+            text, data.oob)
         if is_previewable:
             scheme = urlparse(text).scheme
             if scheme == 'geo':
@@ -409,8 +417,8 @@ class ChatStack(Gtk.Stack, EventHelper):
                 text = format_geo_coords(
                     float(location.lat), float(location.lon))
             else:
-                file_name = preview_helpers.filename_from_uri(text)
-                _icon, file_type = preview_helpers.guess_simple_file_type(text)
+                file_name = filename_from_uri(text)
+                _icon, file_type = guess_simple_file_type(text)
                 text = f'{file_type} ({file_name})'
 
         sound: str | None = None
@@ -423,9 +431,9 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         if isinstance(contact, GroupchatContact):
             msg_type = 'group-chat-message'
-            title += f' {event.resource} ({contact.name})'
+            title += f' {data.resource} ({contact.name})'
             assert contact.nickname is not None
-            needs_highlight = helpers.message_needs_highlight(
+            needs_highlight = message_needs_highlight(
                 text, contact.nickname, client.get_own_jid().bare)
             if needs_highlight:
                 sound = 'muc_message_highlight'
@@ -435,29 +443,29 @@ class ChatStack(Gtk.Stack, EventHelper):
             if not contact.can_notify() and not needs_highlight:
                 return
 
-            if contact.can_notify() or needs_highlight:
-                set_urgency_hint(app.window, True)
+            set_urgency_hint(app.window, True)
 
         if isinstance(contact, GroupchatParticipant):
             msg_type = 'private-chat-message'
             title += f' {contact.name} (private in {contact.room.name})'
             sound = 'first_message_received'
 
-        # Is it a history message? Don't want sound-floods when we join.
-        if tim is not None and time.mktime(time.localtime()) - tim > 1:
-            sound = None
-
         assert isinstance(
             contact, BareContact | GroupchatContact | GroupchatParticipant)
         if app.settings.get('notification_preview_message'):
-            if text.startswith('/me'):
+            if text.startswith('/me '):
                 name = contact.name
                 if isinstance(contact, GroupchatContact):
-                    name = event.properties.muc_nickname
+                    name = data.resource
                 text = f'* {name} {text[3:]}'
         else:
             text = ''
 
+        if isinstance(contact, GroupchatContact):
+            resource = data.resource
+        else:
+            resource = None
+
         app.ged.raise_event(
             events.Notification(account=contact.account,
                                 jid=contact.jid,
@@ -465,7 +473,9 @@ class ChatStack(Gtk.Stack, EventHelper):
                                 sub_type=msg_type,
                                 title=title,
                                 text=text,
-                                sound=sound))
+                                sound=sound,
+                                resource=resource)
+        )
 
     def _connect_actions(self) -> None:
         actions = [
@@ -529,6 +539,7 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         app.window.get_action('quote').set_enabled(online)
         app.window.get_action('mention').set_enabled(online)
+        app.window.get_action('reply').set_enabled(online)
 
     def _update_group_chat_actions(self, contact: GroupchatContact) -> None:
         joined = contact.is_joined
@@ -551,7 +562,8 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         app.window.get_action('quote').set_enabled(joined)
         app.window.get_action('mention').set_enabled(joined)
-        app.window.get_action('retract-message').set_enabled(joined)
+        app.window.get_action('reply').set_enabled(joined)
+        app.window.get_action('moderate-message').set_enabled(joined)
 
     def _update_participant_actions(self,
                                     contact: GroupchatParticipant) -> None:
@@ -631,7 +643,7 @@ class ChatStack(Gtk.Stack, EventHelper):
                 assert isinstance(contact, GroupchatContact)
                 resource_contact = contact.get_resource(nick)
                 jid = resource_contact.jid
-            open_window('AdHocCommands', account=account, jid=jid)
+            open_window('AdHocCommands', account=account, jids=[jid])
 
         elif action_name == 'muc-kick':
             assert param is not None
@@ -645,15 +657,28 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         elif action_name == 'muc-change-role':
             assert param is not None
+            assert isinstance(contact, GroupchatContact)
             nick, role = param.get_strv()
-            client.get_module('MUC').set_role(contact.jid, nick, role)
+            client.get_module('MUC').set_role(
+                contact.jid,
+                nick,
+                role,
+                callback=partial(
+                    self._on_affiliation_or_role_change,
+                    contact, jid, role),
+            )
 
         elif action_name == 'muc-change-affiliation':
             assert param is not None
+            assert isinstance(contact, GroupchatContact)
             jid, affiliation = param.get_strv()
             client.get_module('MUC').set_affiliation(
                 contact.jid,
-                {jid: {'affiliation': affiliation}})
+                {jid: {'affiliation': affiliation}},
+                callback=partial(
+                    self._on_affiliation_or_role_change,
+                    contact, jid, affiliation),
+            )
 
         elif action_name == 'muc-request-voice':
             client.get_module('MUC').request_voice(contact.jid)
@@ -666,10 +691,40 @@ class ChatStack(Gtk.Stack, EventHelper):
                 row = view.get_next_message_row(self._last_quoted_id)
 
             if row is not None:
-                self._last_quoted_id = row.log_line_id
+                self._last_quoted_id = row.pk
                 self._message_action_box.insert_as_quote(
                     row.get_text(), clear=True)
 
+    def _on_affiliation_or_role_change(self,
+                                       muc: GroupchatContact,
+                                       jid: JID | str,
+                                       affiliation_or_role: str,
+                                       task: Task) -> None:
+        try:
+            result = task.finish()
+        except StanzaError as error:
+            log.error('Error on affiliation/role change request: %s',
+                      error)
+            if self._current_contact == muc:
+                self._chat_control.add_info_message(
+                    _('An error occurred while trying to make '
+                      '{occupant_jid} {affiliation_or_role}: {error}').format(
+                        occupant_jid=jid,
+                        affiliation_or_role=affiliation_or_role,
+                        error=str(error),
+                    ))
+            else:
+                ErrorDialog(
+                    _('An error occurred while trying to make '
+                      '{occupant_jid} {affiliation_or_role} in group '
+                      '"{group}"').format(
+                        occupant_jid=jid,
+                        affiliation_or_role=affiliation_or_role,
+                        group=muc.name),
+                    str(error))
+        else:
+            log.debug('Affiliation/role change success: %s', result)
+
     def _on_drag_data_received(self,
                                _widget: Gtk.Widget,
                                _context: Gdk.DragContext,
@@ -710,7 +765,7 @@ class ChatStack(Gtk.Stack, EventHelper):
             return False
 
         for targ in context.list_targets():
-            if targ.name() in {'text/uri-list', 'application/vnd.portal.files'}:
+            if targ.name() in {DND_TARGET_URI_LIST, DND_TARGET_FLATPAK}:
                 self._drop_area.set_no_show_all(False)
                 self._drop_area.show_all()
                 return True
@@ -750,7 +805,7 @@ class ChatStack(Gtk.Stack, EventHelper):
         self._chat_control.add_info_message(message)
 
     def _on_send_message(self) -> None:
-        message = self._message_action_box.msg_textview.get_text()
+        message = self._message_action_box.get_text()
         if message.startswith('//'):
             # Escape sequence for chat commands
             message = message[1:]
@@ -783,35 +838,26 @@ class ChatStack(Gtk.Stack, EventHelper):
             return
 
         label = self._message_action_box.get_seclabel()
-
-        correct_id = None
-        if self._message_action_box.is_correcting:
-            correct_id = self._message_action_box.try_message_correction(
-                message)
-            if correct_id is None:
-                return
+        correct_id = self._message_action_box.get_correction_id()
+        reply_data = self._message_action_box.get_message_reply()
 
         chatstate = client.get_module('Chatstate').get_active_chatstate(
             contact)
 
-        type_ = 'chat'
-        if isinstance(contact, GroupchatContact):
-            type_ = 'groupchat'
-
         message_ = OutgoingMessage(account=contact.account,
                                    contact=contact,
-                                   message=message,
-                                   type_=type_,
+                                   text=message,
                                    chatstate=chatstate,
-                                   label=label,
+                                   sec_label=label,
                                    control=self._chat_control,
-                                   correct_id=correct_id)
+                                   correct_id=correct_id,
+                                   reply_data=reply_data)
 
         client.send_message(message_)
 
-        self._message_action_box.msg_textview.clear()
+        self._message_action_box.reset_state_after_send()
+
         self._last_quoted_id = None
-        app.storage.drafts.set(contact, '')
 
     def get_last_message_id(self, contact: ChatContactT) -> str | None:
         return self._message_action_box.get_last_message_id(contact)
@@ -835,7 +881,7 @@ class ChatStack(Gtk.Stack, EventHelper):
         self._chat_control.clear()
 
     def _on_command_signal(self,
-                           chat_commands: ChatCommands,
+                           _chat_commands: ChatCommands,
                            signal_name: str,
                            text: str
                            ) -> None:
@@ -850,7 +896,7 @@ class ChatPlaceholderBox(Gtk.Box):
                          spacing=18)
         self.set_valign(Gtk.Align.CENTER)
         image = Gtk.Image.new_from_icon_name(
-            'org.gajim.Gajim-symbolic', Gtk.IconSize.from_name('100'))
+            'gajim-symbolic', Gtk.IconSize.from_name('100'))
         image.set_opacity(0.2)
         self.add(image)
 
