@@ -1,22 +1,10 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import cast
-from typing import Optional
-from typing import Union
 
 import logging
 from enum import IntEnum
@@ -27,6 +15,7 @@ from gi.repository import GObject
 from gi.repository import Gtk
 from nbxmpp.errors import StanzaError
 from nbxmpp.modules.vcard4 import VCard
+from nbxmpp.namespaces import Namespace
 from nbxmpp.structs import AnnotationNote
 from nbxmpp.structs import SoftwareVersionResult
 from nbxmpp.task import Task
@@ -39,15 +28,16 @@ from gajim.common.const import SimpleClientState
 from gajim.common.events import SubscribedPresenceReceived
 from gajim.common.events import UnsubscribedPresenceReceived
 from gajim.common.ged import EventHelper
-from gajim.common.helpers import get_uf_affiliation
-from gajim.common.helpers import get_uf_role
-from gajim.common.helpers import get_uf_show
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.modules.contacts import ResourceContact
+from gajim.common.util.status import get_uf_show
+from gajim.common.util.user_strings import get_uf_affiliation
+from gajim.common.util.user_strings import get_uf_role
 
 from gajim.gtk.builder import get_builder
+from gajim.gtk.contact_name_widget import ContactNameWidget
 from gajim.gtk.contact_settings import ContactSettings
 from gajim.gtk.dialogs import ConfirmationDialog
 from gajim.gtk.dialogs import DialogButton
@@ -60,7 +50,7 @@ from gajim.gtk.vcard_grid import VCardGrid
 log = logging.getLogger('gajim.gtk.contact_info')
 
 
-ContactT = Union[BareContact, GroupchatParticipant]
+ContactT = BareContact | GroupchatParticipant
 
 
 class Column(IntEnum):
@@ -72,7 +62,7 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
     def __init__(self,
                  account: str,
                  contact: ContactT,
-                 page: Optional[str] = None) -> None:
+                 page: str | None = None) -> None:
 
         Gtk.ApplicationWindow.__init__(self)
         EventHelper.__init__(self)
@@ -103,12 +93,17 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         self._ui.main_stack.connect('notify::visible-child-name',
                                     self._on_stack_child_changed)
 
-        self._ui.name_entry.set_text(contact.name)
-        if contact.is_in_roster:
-            self._ui.edit_name_button.show()
-
         self._load_avatar()
 
+        self._contact_name_widget = ContactNameWidget(
+            contact=self.contact,
+            edit_mode=True
+        )
+        self._contact_name_widget.connect('name-updated', self._on_contact_name_updated)
+        self._ui.contact_name_controls_box.add(self._contact_name_widget)
+
+        self._ui.contact_name_header_label.set_text(self.contact.name)
+
         self._fill_information_page(self.contact)
 
         if isinstance(self.contact, BareContact):
@@ -151,10 +146,6 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
                                  _signal_name: str,
                                  state: SimpleClientState
                                  ) -> None:
-
-        self._ui.edit_name_button.set_sensitive(state.is_connected)
-        self._ui.edit_name_button.set_tooltip_text(
-            _('Not connected') if not state.is_connected else _('Edit Name…'))
         self._ui.subscription_listbox.set_sensitive(state.is_connected)
 
         if state.is_connected:
@@ -185,6 +176,9 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         self.unregister_events()
         app.check_finalize(self)
 
+    def _on_contact_name_updated(self, _widget: ContactNameWidget, name: str) -> None:
+        self._ui.contact_name_header_label.set_text(name)
+
     def _fill_information_page(self, contact: ContactT) -> None:
         self._vcard_grid = VCardGrid(self.account)
         self._ui.vcard_box.add(self._vcard_grid)
@@ -195,9 +189,6 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
 
         jid = str(contact.get_address())
 
-        self._ui.contact_name_label.set_text(contact.name)
-        self._ui.edit_name_button.set_sensitive(
-            self._client.state.is_available)
 
         self._ui.contact_jid_label.set_text(jid)
         self._ui.contact_jid_label.set_tooltip_text(jid)
@@ -223,7 +214,9 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         if note is not None:
             self._ui.textview_annotation.get_buffer().set_text(note.data)
 
-        if app.account_supports_private_storage(self.account):
+        server_disco = self._client.get_module('Discovery').server_info
+        if (server_disco is not None and
+                server_disco.supports(Namespace.PRIVATE)):
             # Hide the Notes page if private storage is not available, because
             # roster notes cannot be stored without.
             # Since there is no disco mechanism for private storage, we rely on
@@ -324,7 +317,7 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         assert not isinstance(surface_1, GdkPixbuf.Pixbuf)
         assert not isinstance(surface_2, GdkPixbuf.Pixbuf)
         self._ui.avatar_image.set_from_surface(surface_1)
-        self._ui.header_image.set_from_surface(surface_2)
+        self._ui.avatar_image_header.set_from_surface(surface_2)
 
     def _set_os_info(self, task: Task) -> None:
         self._tasks.remove(task)
@@ -363,27 +356,9 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
                 jid=self.contact.jid, data=new_annotation)
             self._client.get_module('Annotations').set_note(new_note)
 
-    def _on_edit_name_toggled(self, widget: Gtk.ToggleButton) -> None:
-        active = widget.get_active()
-        self._ui.name_entry.set_sensitive(active)
-        if active:
-            self._ui.name_entry.grab_focus()
-            self._ui.edit_name_button_image.set_from_icon_name(
-                'document-save-symbolic', Gtk.IconSize.BUTTON)
-        else:
-            self._ui.edit_name_button_image.set_from_icon_name(
-                'document-edit-symbolic', Gtk.IconSize.BUTTON)
-            name = self._ui.name_entry.get_text()
-            if name == self.contact.name:
-                return
-
-            assert isinstance(self.contact, BareContact)
-            self._client.get_module('Roster').set_item(
-                self.contact.jid, name, self.contact.groups)
-            self._ui.contact_name_label.set_text(name)
-
-    def _on_name_entry_activate(self, _widget: Gtk.Entry) -> None:
-        self._ui.edit_name_button.set_active(False)
+    def _on_edit_contact_name_header_clicked(self, _widget: Gtk.Button) -> None:
+        self._switcher.set_row('information')
+        self._contact_name_widget.enable_edit_mode()
 
     def _on_from_subscription_switch_toggled(self,
                                              switch: Gtk.Switch,
@@ -539,7 +514,7 @@ class DeviceGrid:
     def widget(self) -> Gtk.Grid:
         return self._ui.devices_grid
 
-    def set_entity_time(self, entity_time: Optional[str]) -> None:
+    def set_entity_time(self, entity_time: str | None) -> None:
         if entity_time is not None:
             self._ui.time_value.set_text(entity_time)
             self._ui.time_value.show()
@@ -547,7 +522,7 @@ class DeviceGrid:
 
         self._check_complete()
 
-    def set_software(self, software: Optional[SoftwareVersionResult]) -> None:
+    def set_software(self, software: SoftwareVersionResult | None) -> None:
         if software is not None:
             software_string = f'{software.name} {software.version}'
             self._ui.software_value.set_text(software_string)
