@@ -1,22 +1,9 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
-from typing import Optional
-from typing import Union
-
 from gi.repository import Gdk
 from gi.repository import Gtk
 
@@ -32,7 +19,7 @@ from gajim.gtk.conversation.code_widget import CodeWidget
 from gajim.gtk.conversation.plain_widget import PlainWidget
 from gajim.gtk.conversation.quote_widget import QuoteWidget
 
-ContentT = Union[ParsingResult, QuoteBlock]
+ContentT = ParsingResult | QuoteBlock
 
 
 class MessageWidget(Gtk.Box):
@@ -60,13 +47,13 @@ class MessageWidget(Gtk.Box):
 
     def add_with_styling(self,
                          text: str,
-                         nickname: Optional[str] = None) -> None:
+                         nickname: str | None = None) -> None:
 
         self._original_text = text
         if len(text) > MAX_MESSAGE_LENGTH:
             text = text[:MAX_MESSAGE_LENGTH]
 
-        if text.startswith('/me') and nickname is not None:
+        if text.startswith('/me ') and nickname is not None:
             self._add_action_phrase(text, nickname)
             if len(self._original_text) > MAX_MESSAGE_LENGTH:
                 self._add_read_more_button(self._original_text)
