@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -18,6 +8,7 @@ import time
 from datetime import datetime
 
 from gi.repository import Gtk
+from gi.repository import Pango
 from nbxmpp.structs import MucSubject
 
 from gajim.common.const import AvatarSize
@@ -41,7 +32,7 @@ class MUCSubject(BaseRow):
         BaseRow.__init__(self, account)
 
         current_timestamp = timestamp or time.time()
-        self.timestamp = datetime.fromtimestamp(current_timestamp)
+        self.timestamp = datetime.fromtimestamp(current_timestamp).astimezone()
         self.db_timestamp = current_timestamp
 
         self.grid.set_halign(Gtk.Align.START)
@@ -65,12 +56,17 @@ class MUCSubject(BaseRow):
         date = ''
         if subject.timestamp is not None:
             time_str = time.strftime('%c', time.localtime(subject.timestamp))
-            date = f' ({time_str})'
+            date = f'{time_str}\n'
 
-        meta_str = f'{author}{date}'
-        meta = Gtk.Label(label=meta_str)
-        meta.set_halign(Gtk.Align.START)
-        meta.set_selectable(True)
+        meta_str = f'{author}\n{date}'
+        meta = Gtk.Label(
+            halign=Gtk.Align.START,
+            label=meta_str,
+            selectable=True,
+            wrap=True,
+            wrap_mode=Pango.WrapMode.WORD_CHAR,
+            xalign=0,
+        )
         meta.get_style_context().add_class('small-label')
         subject_box.add(meta)
 
