@@ -1,22 +1,11 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-
-from datetime import datetime
+# SPDX-License-Identifier: GPL-3.0-only
 
 from gi.repository import Gtk
 
 from gajim.common.i18n import _
+from gajim.common.util.datetime import FIRST_LOCAL_DATETIME
 
 from gajim.gtk.conversation.rows.base import BaseRow
 
@@ -28,7 +17,7 @@ class ScrollHintRow(BaseRow):
         self.set_activatable(False)
 
         self.type = 'system'
-        self.timestamp = datetime.fromtimestamp(0)
+        self.timestamp = FIRST_LOCAL_DATETIME
 
         self.get_style_context().add_class('conversation-system-row')
 
