@@ -1,32 +1,20 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
 from typing import cast
-from typing import Optional
-from typing import Union
 
-from gi.repository import Gtk
+from collections.abc import Callable
+
 from gi.repository import GLib
 from gi.repository import GObject
+from gi.repository import Gtk
 from gi.repository import Pango
-
-from nbxmpp.modules.dataforms import extend_form
 from nbxmpp.modules.dataforms import DataField
+from nbxmpp.modules.dataforms import extend_form
 from nbxmpp.modules.dataforms import MultipleDataForm
 from nbxmpp.modules.dataforms import SimpleDataForm
 from nbxmpp.modules.dataforms import Uri
@@ -35,13 +23,12 @@ from nbxmpp.simplexml import Node
 
 from gajim.common import app
 from gajim.common.i18n import _
-from gajim.common.helpers import open_uri
-
-from .util import MultiLineLabel
-from .util import MaxWidthComboBoxText
-from .util import make_href_markup
-from .util import scale_pixbuf_from_data
+from gajim.common.util.image import scale_pixbuf_from_data
+from gajim.common.util.uri import open_uri
 
+from gajim.gtk.util import make_href_markup
+from gajim.gtk.util import MaxWidthComboBoxText
+from gajim.gtk.util import MultiLineLabel
 
 # Options
 
@@ -56,11 +43,13 @@ from .util import scale_pixbuf_from_data
 
 class DataFormWidget(Gtk.ScrolledWindow):
 
-    __gsignals__ = {'is-valid': (GObject.SignalFlags.RUN_LAST, None, (bool,))}
+    __gsignals__ = {
+        'is-valid': (GObject.SignalFlags.RUN_LAST, None, (bool,)),
+    }
 
     def __init__(self,
-                 form_node: Union[SimpleDataForm, MultipleDataForm],
-                 options: Optional[dict[str, Any]] = None
+                 form_node: SimpleDataForm | MultipleDataForm,
+                 options: dict[str, Any] | None = None
                  ) -> None:
 
         Gtk.ScrolledWindow.__init__(self)
@@ -76,26 +65,34 @@ class DataFormWidget(Gtk.ScrolledWindow):
             self.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.NEVER)
 
         self._form_node = form_node
-        self._form_grid = FormGrid(form_node, options)
+        if self._form_node.is_reported:
+            self._form_widget = DataFormReportedTable(form_node)
+        else:
+            self._form_widget = FormGrid(form_node, options)
+            self._form_widget.connect('form-completed', self._on_form_completed)
         self._original_form_hash = self.get_form_hash()
 
-        self.add(self._form_grid)
+        self.add(self._form_widget)
+
+        self.connect('destroy', self._on_destroy)
 
     @property
-    def title(self) -> Optional[str]:
-        return self._form_grid.title
+    def title(self) -> str | None:
+        return self._form_widget.title
 
     @property
-    def instructions(self) -> Optional[str]:
-        return self._form_grid.instructions
+    def instructions(self) -> str | None:
+        return self._form_widget.instructions
 
     def validate(self) -> None:
-        return self._form_grid.validate(True)
+        return self._form_widget.validate(True)
 
-    def get_form(self) -> Union[SimpleDataForm, MultipleDataForm]:
+    def get_form(self) -> SimpleDataForm | MultipleDataForm:
         return self._form_node
 
-    def get_submit_form(self) -> Union[SimpleDataForm, MultipleDataForm]:
+    def get_submit_form(self) -> SimpleDataForm:
+        if self._form_node.is_reported:
+            raise ValueError('MultipleDataForm cannot be submitted')
         self._form_node.type_ = 'submit'
         return self._form_node
 
@@ -109,16 +106,30 @@ class DataFormWidget(Gtk.ScrolledWindow):
         self._original_form_hash = self.get_form_hash()
 
     def focus_first_entry(self) -> None:
-        for row in range(0, self._form_grid.row_count):
-            widget = self._form_grid.get_child_at(1, row)
+        if self._form_node.is_reported:
+            return
+
+        for row in range(self._form_widget.row_count):
+            widget = self._form_widget.get_child_at(1, row)
             if isinstance(widget, Gtk.Entry):
                 widget.grab_focus_without_selecting()
                 break
 
+    def _on_destroy(self, widget: Gtk.ScrolledWindow) -> None:
+        self._form_widget.destroy()
+
+    def _on_form_completed(self, *_args) -> None:
+        self.get_toplevel().activate_default()
+
 
 class FormGrid(Gtk.Grid):
+
+    __gsignals__ = {
+        'form-completed': (GObject.SignalFlags.RUN_LAST, None, ())
+    }
+
     def __init__(self,
-                 form_node: Union[SimpleDataForm, MultipleDataForm],
+                 form_node: SimpleDataForm,
                  options: dict[str, Any]
                  ) -> None:
 
@@ -128,19 +139,19 @@ class FormGrid(Gtk.Grid):
         self.set_halign(Gtk.Align.CENTER)
         self.row_count = 0
 
-        self.rows: list[Union[SizeAdjustment,
-                              Title,
-                              Instructions,
-                              Field,
-                              ImageMediaField]] = []
+        self.rows: list[(SizeAdjustment |
+                         Title |
+                         Instructions |
+                         Field |
+                         ImageMediaField)] = []
 
         form_width = options.get('form-width', 435)
         self.set_size_request(form_width, -1)
 
         self._data_form = form_node
 
-        self.title: Optional[str] = None
-        self.instructions: Optional[str] = None
+        self.title: str | None = None
+        self.instructions: str | None = None
 
         self._fields = {
             'boolean': BooleanField,
@@ -166,20 +177,19 @@ class FormGrid(Gtk.Grid):
         self._analyse_fields(form_node, options)
         self._parse_form(form_node, options)
 
-    def _add_row(self,
-                 field: Union[SizeAdjustment,
-                              Title,
-                              Instructions,
-                              Field,
-                              ImageMediaField]
-                 ) -> None:
+        self.connect('destroy', self._on_destroy)
+
+    def _add_row(
+        self,
+        field: SizeAdjustment | Title | Instructions | Field | ImageMediaField
+    ) -> None:
 
         field.add(self, self.row_count)
         self.row_count += 1
         self.rows.append(field)
 
     @staticmethod
-    def _analyse_fields(form_node: Union[SimpleDataForm, MultipleDataForm],
+    def _analyse_fields(form_node: SimpleDataForm,
                         options: dict[str, Any]
                         ) -> None:
 
@@ -187,7 +197,7 @@ class FormGrid(Gtk.Grid):
             # Don’t overwrite option
             return
 
-        label_lengths = set([0])
+        label_lengths = {0}
         for field in form_node.iter_fields():
             if field.type_ == 'hidden':
                 continue
@@ -200,10 +210,14 @@ class FormGrid(Gtk.Grid):
         options['right-align'] = max(label_lengths) < 30
 
     def _parse_form(self,
-                    form_node: Union[SimpleDataForm, MultipleDataForm],
+                    form_node: SimpleDataForm,
                     options: dict[str, Any]
                     ) -> None:
 
+        single_field = len(
+            [field for field in form_node.fields
+             if field.type_ not in ('fixed', 'hidden')]) == 1
+
         for field in form_node.iter_fields():
             if field.type_ == 'hidden':
                 continue
@@ -218,7 +232,13 @@ class FormGrid(Gtk.Grid):
                     continue
 
             widget = self._fields[field.type_]
-            self._add_row(widget(field, self, options))
+            if single_field and widget is ListSingleField:
+                row = widget(field, self, options, treeview=True)
+                row.connect(
+                    'row-activated', self._on_single_list_field_activated)
+                self._add_row(row)
+            else:
+                self._add_row(widget(field, self, options))
 
     def _add_media_field(self,
                          field: DataField,
@@ -245,6 +265,15 @@ class FormGrid(Gtk.Grid):
         dataform_widget = cast(DataFormWidget, viewport.get_parent())
         dataform_widget.emit('is-valid', value)
 
+    def _on_single_list_field_activated(self, widget: ListSingleField) -> None:
+        self.emit('form-completed')
+
+    def _on_destroy(self, widget: Gtk.Grid) -> None:
+        for row in self.rows:
+            if isinstance(row, Field):
+                row.destroy()
+        self.rows.clear()
+
 
 class SizeAdjustment:
     def __init__(self, options: dict[str, Any]) -> None:
@@ -254,6 +283,7 @@ class SizeAdjustment:
         left_width = options.get('left-width', 100)
         self._left_box.set_size_request(left_width, -1)
         self._right_box.set_hexpand(True)
+        self._right_box.set_size_request(250, -1)
 
     def add(self, form_grid: FormGrid, row_number: int) -> None:
         form_grid.attach(self._left_box, 0, row_number, 1, 1)
@@ -286,17 +316,19 @@ class Instructions:
         form_grid.attach(self._label, 0, row_number, 2, 1)
 
 
-class Field:
+class Field(GObject.GObject):
     def __init__(self,
                  field: DataField,
                  form_grid: FormGrid,
                  options: dict[str, Any]
                  ) -> None:
 
-        self._widget: Optional[Gtk.Widget] = None
+        GObject.GObject.__init__(self)
+
+        self._widget: Gtk.Widget | None = None
         self._field = field
         self._form_grid = form_grid
-        self._validate_source_id: Optional[int] = None
+        self._validate_source_id: int | None = None
         self._read_only = options.get('read-only', False)
 
         self._label = Gtk.Label(label=field.label)
@@ -346,7 +378,7 @@ class Field:
 
     def _set_warning(self,
                      is_valid: bool,
-                     error: Union[str, InvalidJid]
+                     error: str | InvalidJid
                      ) -> None:
 
         if not self._field.required and not is_valid and not error:
@@ -365,7 +397,7 @@ class Field:
         self._warning_image.set_tooltip_text(str(error))
         self._warning_image.set_visible(not is_valid)
 
-    def _validate(self) -> None:
+    def _validate(self, delay: bool = True) -> None:
         self._form_grid.validate(False)
         if self._validate_source_id is not None:
             GLib.source_remove(self._validate_source_id)
@@ -376,7 +408,14 @@ class Field:
             self._form_grid.validate(is_valid)
             self._validate_source_id = None
 
-        self._validate_source_id = GLib.timeout_add(200, _start_validation)
+        if delay:
+            self._validate_source_id = GLib.timeout_add(200, _start_validation)
+        else:
+            _start_validation()
+
+    def destroy(self) -> None:
+        if self._validate_source_id is not None:
+            GLib.source_remove(self._validate_source_id)
 
 
 class BooleanField(Field):
@@ -390,7 +429,7 @@ class BooleanField(Field):
 
         if self.read_only:
             label = _('Yes') if field.value else _('No')
-            self._widget = Gtk.Label(label=label)
+            self._widget = Gtk.Label(label=label, selectable=True)
             self._widget.set_xalign(0)
         else:
             self._widget = Gtk.CheckButton()
@@ -430,14 +469,33 @@ class FixedField(Field):
 
 
 class ListSingleField(Field):
+
+    __gsignals__ = {
+        'row-activated': (GObject.SignalFlags.RUN_LAST, None, ())
+    }
+
     def __init__(self,
                  field: DataField,
                  form_grid: FormGrid,
-                 options: dict[str, Any]
+                 options: dict[str, Any],
+                 treeview: bool = False,
                  ) -> None:
 
         Field.__init__(self, field, form_grid, options)
 
+        self._unique = treeview
+
+        if self.read_only:
+            self._widget = Gtk.Label(label=field.value, selectable=True)
+            self._widget.set_xalign(0)
+            return
+
+        if treeview:
+            self._setup_treeview(field)
+        else:
+            self._setup_dropdown(field)
+
+    def _setup_dropdown(self, field: DataField) -> None:
         self._widget = MaxWidthComboBoxText()
         self._widget.set_valign(Gtk.Align.CENTER)
         for value, label in field.iter_options():
@@ -452,6 +510,74 @@ class ListSingleField(Field):
         self._field.value = widget.get_active_id()
         self._validate()
 
+    def _setup_treeview(self, field: DataField) -> None:
+        self._treeview = ListSingleTreeView(field, self)
+        self._treeview.connect('row-activated', self._on_row_activated)
+        self._treeview.connect('cursor-changed', self._on_cursor_changed)
+
+        self._widget = Gtk.ScrolledWindow()
+        self._widget.set_propagate_natural_height(True)
+        self._widget.set_hexpand(True)
+        self._widget.set_vexpand(True)
+        self._widget.set_min_content_height(100)
+        self._widget.set_max_content_height(300)
+        self._widget.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
+        self._widget.add(self._treeview)
+
+
+    def _on_cursor_changed(self, *_args: Any) -> None:
+        model, treeiter = self._treeview.get_selection().get_selected()
+        if treeiter is None:
+            return None
+        self._field.value = model[treeiter][1]
+        self._validate(delay=False)
+
+    def _on_row_activated(self, *_args: Any) -> None:
+        self.emit('row-activated')
+
+    def add(self,
+            form_grid: FormGrid,
+            row_number: int
+            ) -> None:
+        if self._unique:
+            form_grid.attach(self._widget, 0, row_number, 2, 1)
+        else:
+            super().add(form_grid, row_number)
+
+
+class ListSingleTreeView(Gtk.TreeView):
+    def __init__(self,
+                 field: DataField,
+                 multi_field: ListSingleField
+                 ) -> None:
+
+        Gtk.TreeView.__init__(self)
+
+        self._field = field
+        self._multi_field = multi_field
+
+        # label, value, tooltip
+        self._store = Gtk.ListStore(str, str, str)
+
+        col = Gtk.TreeViewColumn(field.label)
+        cell = Gtk.CellRendererText()
+        cell.set_property('ellipsize', Pango.EllipsizeMode.END)
+        cell.set_property('width-chars', 80)
+        col.pack_start(cell, True)
+        col.set_attributes(cell, text=0)
+        self.append_column(col)
+
+        self.set_headers_visible(True)
+
+        for option in field.options:
+            label, value = option
+            self._store.append([label, value, label])
+
+        if any(filter(lambda x: len(x[0]) > 80, field.options)):
+            self.set_tooltip_column(2)
+
+        self.set_model(self._store)
+
 
 class ListMultiField(Field):
     def __init__(self,
@@ -463,7 +589,7 @@ class ListMultiField(Field):
         Field.__init__(self, field, form_grid, options)
         self._label.set_valign(Gtk.Align.START)
 
-        self._treeview = ListMutliTreeView(field, self)
+        self._treeview = ListMultiTreeView(field, self)
 
         self._widget = Gtk.ScrolledWindow()
         self._widget.set_propagate_natural_height(True)
@@ -476,7 +602,7 @@ class ListMultiField(Field):
         self._validate()
 
 
-class ListMutliTreeView(Gtk.TreeView):
+class ListMultiTreeView(Gtk.TreeView):
     def __init__(self,
                  field: DataField,
                  multi_field: ListMultiField
@@ -506,18 +632,20 @@ class ListMutliTreeView(Gtk.TreeView):
         cell.connect('toggled', self._toggled)
         col.pack_start(cell, True)
         col.set_attributes(cell, active=3)
-        self.append_column(col)
+
+        if not multi_field.read_only:
+            self.append_column(col)
 
         self.set_headers_visible(False)
 
         for option in field.options:
             label, value = option
+            if multi_field.read_only and value not in field.values:
+                continue
             self._store.append(
                 [label, value, label, value in field.values])
 
-        labels_over_max_width = map(lambda x: len(x) > 40,
-                                    [option[0] for option in field.options])
-        if any(labels_over_max_width):
+        if any(filter(lambda x: len(x[0]) > 40, field.options)):
             self.set_tooltip_column(2)
 
         self.set_model(self._store)
@@ -540,7 +668,7 @@ class ListMutliTreeView(Gtk.TreeView):
         for row in model:
             if not row[3]:
                 continue
-            values.append(row[2])
+            values.append(row[1])
         self._field.values = values
 
 
@@ -578,7 +706,9 @@ class JidMultiField(Field):
         self._scrolled_window.add(self._treeview)
 
         self._widget.pack_start(self._scrolled_window, True, True, 0)
-        self._widget.pack_end(self._toolbar, False, False, 0)
+
+        if not self._read_only:
+            self._widget.pack_end(self._toolbar, False, False, 0)
 
     def _add_clicked(self, _widget: Gtk.ToolButton) -> None:
         model = self._treeview.get_model()
@@ -666,7 +796,7 @@ class TextSingleField(Field):
         Field.__init__(self, field, form_grid, options)
 
         if self.read_only:
-            self._widget = Gtk.Label(label=field.value)
+            self._widget = Gtk.Label(label=field.value, selectable=True)
             self._widget.set_xalign(0)
             self._widget.set_selectable(True)
         else:
@@ -691,9 +821,9 @@ class TextPrivateField(TextSingleField):
                  ) -> None:
 
         TextSingleField.__init__(self, field, form_grid, options)
-        assert isinstance(self._widget, Gtk.Entry)
-        self._widget.set_input_purpose(Gtk.InputPurpose.PASSWORD)
-        self._widget.set_visibility(False)
+        if isinstance(self._widget, Gtk.Entry):
+            self._widget.set_input_purpose(Gtk.InputPurpose.PASSWORD)
+            self._widget.set_visibility(False)
 
 
 class JidSingleField(TextSingleField):
@@ -716,32 +846,34 @@ class TextMultiField(Field):
         Field.__init__(self, field, form_grid, options)
         self._label.set_valign(Gtk.Align.START)
 
-        self._widget = Gtk.ScrolledWindow()
-        self._widget.set_policy(Gtk.PolicyType.NEVER,
-                                Gtk.PolicyType.AUTOMATIC)
-        self._widget.set_propagate_natural_height(True)
-        self._widget.set_min_content_height(100)
-        self._widget.set_max_content_height(300)
+        length = len(field.value)
 
         if self.read_only:
-            self._textview = MultiLineLabel(label=field.value)
-            self._textview.set_selectable(True)
-            self._textview.set_xalign(0)
-            self._textview.set_yalign(0)
+            self._textview = MultiLineLabel(label=field.value, xalign=0)
+
         else:
             self._textview = Gtk.TextView()
             self._textview.set_wrap_mode(Gtk.WrapMode.WORD_CHAR)
             self._textview.get_buffer().set_text(field.value)
             self._textview.get_buffer().connect('changed', self._changed)
 
-        self._widget.add(self._textview)
+        if self.read_only and length < 500:
+            self._widget = self._textview
+        else:
+            self._widget = Gtk.ScrolledWindow()
+            self._widget.set_policy(Gtk.PolicyType.NEVER,
+                                    Gtk.PolicyType.AUTOMATIC)
+            self._widget.set_propagate_natural_height(True)
+            self._widget.set_min_content_height(100)
+
+            self._widget.add(self._textview)
 
     def _changed(self, widget: Gtk.TextBuffer) -> None:
         self._field.value = widget.get_text(*widget.get_bounds(), False)
         self._validate()
 
 
-class ImageMediaField():
+class ImageMediaField:
     def __init__(self,
                  uri: Uri,
                  form_grid: FormGrid,
@@ -872,3 +1004,86 @@ class DataFormDialog(Gtk.Dialog):
         if response == Gtk.ResponseType.OK:
             self._submit_callback(self._form.get_submit_form(), self._node)
         self.destroy()
+
+
+class DataFormReportedTable(Gtk.Grid):
+    '''
+    A widget representing the results of a command as a table, when the data
+    form contains a <reported> element.
+    '''
+
+    def __init__(self, form_node: MultipleDataForm) -> None:
+        super().__init__(row_spacing=10)
+        self._form_node = form_node
+
+        self.title = form_node.title
+        self.instructions = form_node.instructions
+
+        self._process_form()
+        self._build_list_store()
+        self._build_treeview()
+        self._build_scrolled_window()
+
+    def _process_form(self) -> None:
+        self._column_names: list[str] = []
+        self._column_types: list[str] = []
+        for field in self._form_node.reported.iter_fields():
+            self._column_types.append(field.type_)
+            self._column_names.append(field.label)
+
+        rows: list[list[str]] = [[f.value for f in record.fields]
+                                 for record in self._form_node.items]
+
+        self._rows = rows
+
+    def _build_list_store(self) -> None:
+        self._list_store = Gtk.ListStore(*(str for _ in self._column_names))
+        for row in self._rows:
+            self._list_store.append(row)
+
+    def _build_treeview(self) -> None:
+        self._treeview = Gtk.TreeView(model=self._list_store.filter_new())
+        self._treeview.connect('row-activated', self._on_row_activate)
+
+        for i, column_title in enumerate(self._column_names):
+            renderer = Gtk.CellRendererText()
+            column = Gtk.TreeViewColumn(column_title, renderer, text=i)
+            self._treeview.append_column(column)
+
+    def _build_scrolled_window(self) -> None:
+        scrolled_window = Gtk.ScrolledWindow()
+        scrolled_window.set_vexpand(True)
+        scrolled_window.set_hexpand(True)
+        scrolled_window.add(self._treeview)
+
+        title = f'<big>{GLib.markup_escape_text(self._form_node.title)}</big>'
+        self.attach(Gtk.Label(label=title, use_markup=True), 0, 0, 1, 1)
+        self.attach(scrolled_window, 0, 2, 1, 1)
+
+    def _on_row_activate(
+        self,
+        _tree_view: Gtk.TreeView,
+        path: Gtk.TreePath,
+        column: Gtk.TreeViewColumn,
+    ) -> None:
+        row = self._rows[int(path.to_string())]
+        col = self._column_names.index(column.get_title())
+        # If the activated cell is a JID, start a chat with it.
+        if self._column_types[col] == 'jid-single':
+            dest: str | None = row[col]
+        else:
+            # else, try to find a column that is a JID
+            for i, t in enumerate(self._column_types):
+                if t == 'jid-single':
+                    dest = row[i]
+                    break
+            else:
+                dest = None
+        if dest:
+            app.app.activate_action(
+                'start-chat', GLib.Variant('as', [dest, '']))
+
+    def validate(self, is_valid: bool) -> None:
+        viewport = cast(Gtk.Viewport, self.get_parent())
+        dataform_widget = cast(DataFormWidget, viewport.get_parent())
+        dataform_widget.emit('is-valid', is_valid)
