@@ -5,65 +5,57 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
-from typing import Optional
-
+import logging
 import os
 import time
-import logging
-from functools import partial
+import uuid
+from datetime import datetime
+from datetime import timezone
 from enum import IntEnum
 from enum import unique
-from datetime import datetime
+from functools import partial
+from pathlib import Path
 
-from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GLib
+from gi.repository import Gtk
 from gi.repository import Pango
-
 from nbxmpp.protocol import JID
 
 from gajim.common import app
 from gajim.common import ged
-from gajim.common import helpers
+from gajim.common import types
 from gajim.common.events import FileRequestSent
 from gajim.common.events import Notification
-from gajim.common.const import KindConstant
-from gajim.common.i18n import _
-from gajim.common.file_props import FilesProp
 from gajim.common.file_props import FileProp
-from gajim.common.helpers import open_file
+from gajim.common.file_props import FilesProp
 from gajim.common.helpers import file_is_locked
-from gajim.common.helpers import AdditionalDataDict
+from gajim.common.i18n import _
 from gajim.common.modules.bytestream import is_transfer_active
 from gajim.common.modules.bytestream import is_transfer_paused
 from gajim.common.modules.bytestream import is_transfer_stopped
 from gajim.common.modules.contacts import BareContact
-from gajim.common import types
+from gajim.common.storage.archive import models as mod
+from gajim.common.storage.archive.const import ChatDirection
+from gajim.common.storage.archive.const import MessageState
+from gajim.common.storage.archive.const import MessageType
+from gajim.common.util.datetime import utc_now
+from gajim.common.util.text import get_random_string
+from gajim.common.util.uri import show_in_folder
 
-from .dialogs import DialogButton
-from .dialogs import ConfirmationDialog
-from .dialogs import ErrorDialog
-from .filechoosers import FileSaveDialog
-from .file_transfer_send import SendFileDialog
-from .tooltips import FileTransfersTooltip
-from .builder import get_builder
-from .util import format_eta
+from gajim.gtk.builder import get_builder
+from gajim.gtk.dialogs import ConfirmationDialog
+from gajim.gtk.dialogs import DialogButton
+from gajim.gtk.dialogs import ErrorDialog
+from gajim.gtk.filechoosers import FileSaveDialog
+from gajim.gtk.tooltips import FileTransfersTooltip
+from gajim.gtk.util import format_eta
 
-log = logging.getLogger('gajim.gui.filetransfer')
+log = logging.getLogger('gajim.gtk.filetransfer')
 
 
 @unique
@@ -369,7 +361,7 @@ class FileTransfersWindow:
             os.remove(file_props.file_name)
 
             # Request the file to the sender
-            sid = helpers.get_random_string()
+            sid = get_random_string()
             new_file_props = FilesProp.getNewFileProp(account, sid)
             new_file_props.file_name = file_props.file_name
             new_file_props.name = file_props.name
@@ -403,16 +395,9 @@ class FileTransfersWindow:
                                text=_('_Download Again'),
                                callback=_on_yes)]).show()
 
-    def show_file_send_request(self,
-                               account: str,
-                               contact: types.BareContact
-                               ) -> None:
-        send_callback = partial(self.send_file, account, contact)
-        SendFileDialog(contact, send_callback, self.window)
-
     def send_file(self,
                   account: str,
-                  contact: types.BareContact,
+                  contact: types.ResourceContact,
                   resource_jid: JID,
                   file_path: str,
                   file_desc: str = ''
@@ -436,16 +421,24 @@ class FileTransfersWindow:
         if file_props is None:
             return False
 
-        # Insert file request into DB
-        additional_data = AdditionalDataDict()
-        additional_data.set_value('gajim', 'type', 'jingle')
-        additional_data.set_value('gajim', 'sid', file_props.sid)
-        app.storage.archive.insert_into_logs(
-            account,
-            contact.jid.bare,
-            time.time(),
-            KindConstant.FILE_TRANSFER_OUTGOING,
-            additional_data=additional_data)
+        ft_data = mod.FileTransfer(
+            state=1,
+            source=[mod.JingleFT(type='jingleft', sid=file_props.sid)],
+        )
+
+        message_data = mod.Message(
+            account_=account,
+            remote_jid_=contact.jid.new_as_bare(),
+            resource=resource_jid.resource,
+            type=MessageType.CHAT,
+            direction=ChatDirection.OUTGOING,
+            timestamp=utc_now(),
+            state=MessageState.ACKNOWLEDGED,
+            id=str(uuid.uuid4()),
+            filetransfers=[ft_data],
+        )
+
+        app.storage.archive.insert_object(message_data)
 
         client = app.get_client(account)
         client.get_module('Jingle').start_file_transfer(
@@ -473,7 +466,8 @@ class FileTransfersWindow:
                                  contact: BareContact,
                                  file_props: FileProp
                                  ) -> None:
-        def _on_accepted(account, contact, file_props, file_path):
+        def _on_accepted(account, contact, file_props, file_paths):
+            file_path = file_paths[0]
             if os.path.exists(file_path):
                 app.settings.set('last_save_dir', os.path.dirname(file_path))
 
@@ -570,27 +564,27 @@ class FileTransfersWindow:
         _str += str(percent) + '%          \n'
         return _str
 
-    def _get_eta_and_speed(self, full_size, transfered_size, file_props):
-        if not file_props.transfered_size:
+    def _get_eta_and_speed(self, full_size, transferred_size, file_props):
+        if not file_props.transferred_size:
             return 0., 0.
 
         if file_props.elapsed_time == 0:
             return 0., 0.
 
-        if len(file_props.transfered_size) == 1:
-            speed = round(float(transfered_size) / file_props.elapsed_time)
+        if len(file_props.transferred_size) == 1:
+            speed = round(float(transferred_size) / file_props.elapsed_time)
         else:
-            # first and last are (time, transfered_size)
-            first = file_props.transfered_size[0]
-            last = file_props.transfered_size[-1]
-            transfered = last[1] - first[1]
+            # first and last are (time, transferred_size)
+            first = file_props.transferred_size[0]
+            last = file_props.transferred_size[-1]
+            transferred = last[1] - first[1]
             tim = last[0] - first[0]
             if tim == 0:
                 return 0., 0.
-            speed = round(float(transfered) / tim)
+            speed = round(float(transferred) / tim)
         if speed == 0.:
             return 0., 0.
-        remaining_size = full_size - transfered_size
+        remaining_size = full_size - transferred_size
         eta = remaining_size / speed
         return eta, speed
 
@@ -615,9 +609,9 @@ class FileTransfersWindow:
             event.file_props.sid,
             event.file_props.received_len)
 
-    def set_progress(self, typ, sid, transfered_size, iter_=None):
+    def set_progress(self, typ, sid, transferred_size, iter_=None):
         '''
-        Change the progress of a transfer with new transfered size
+        Change the progress of a transfer with new transferred size
         '''
         if time.time() - self._last_progress_update < 0.5:
             # Update window every 500ms only
@@ -632,7 +626,7 @@ class FileTransfersWindow:
         if full_size == 0:
             percent = 0
         else:
-            percent = round(float(transfered_size) / full_size * 100, 1)
+            percent = round(float(transferred_size) / full_size * 100, 1)
         if iter_ is None:
             iter_ = self.get_iter_by_sid(typ, sid)
         if iter_ is not None:
@@ -640,30 +634,31 @@ class FileTransfersWindow:
             if self.model[iter_][Column.PERCENT] == 0 and int(percent > 0):
                 just_began = True
             text = self._format_percent(percent)
-            if transfered_size == 0:
+            if transferred_size == 0:
                 text += '0'
             else:
-                text += GLib.format_size_full(transfered_size, self.units)
+                text += GLib.format_size_full(transferred_size, self.units)
             text += '/' + GLib.format_size_full(full_size, self.units)
             # Kb/s
 
             # remaining time
             if file_props.offset:
-                transfered_size -= file_props.offset
+                transferred_size -= file_props.offset
                 full_size -= file_props.offset
 
             if file_props.elapsed_time > 0:
-                file_props.transfered_size.append((file_props.last_time,
-                                                   transfered_size))
-            if len(file_props.transfered_size) > 6:
-                file_props.transfered_size.pop(0)
-            eta, speed = self._get_eta_and_speed(full_size, transfered_size,
+                file_props.transferred_size.append((file_props.last_time,
+                                                   transferred_size))
+            if len(file_props.transferred_size) > 6:
+                file_props.transferred_size.pop(0)
+            eta, speed = self._get_eta_and_speed(full_size, transferred_size,
                                                  file_props)
 
             self.model.set(iter_, Column.PROGRESS, text)
             self.model.set(iter_, Column.PERCENT, int(percent))
             text = format_eta(eta)
             text += '\n'
+            # Translators:
             # This should make the string KB/s,
             # where 'KB' part is taken from %s.
             # Only the 's' after / (which means second) should be translated.
@@ -683,7 +678,7 @@ class FileTransfersWindow:
             if file_props.connected is False:
                 status = 'stop'
             self.model.set(iter_, 0, self.icons[status])
-            if transfered_size == full_size:
+            if transferred_size == full_size:
                 # If we are receiver and this is a jingle session
                 if (file_props.type_ == 'r' and
                         file_props.session_type == 'jingle' and
@@ -710,7 +705,7 @@ class FileTransfersWindow:
     @staticmethod
     def __convert_date(epoch: float) -> str:
         # Converts date-time from seconds from epoch to iso 8601
-        dt = datetime.utcfromtimestamp(epoch)
+        dt = datetime.fromtimestamp(epoch, timezone.utc)
         return dt.isoformat() + 'Z'
 
     def get_send_file_props(self,
@@ -719,7 +714,7 @@ class FileTransfersWindow:
                             file_path: str,
                             file_name: str,
                             file_desc: str = ''
-                            ) -> Optional[FileProp]:
+                            ) -> FileProp | None:
         '''
         Create new file_props object and set initial file transfer
         properties in it
@@ -740,7 +735,7 @@ class FileTransfersWindow:
 
         file_props = FilesProp.getNewFileProp(
             account,
-            sid=helpers.get_random_string())
+            sid=get_random_string())
         mod_date = os.path.getmtime(file_path)
         file_props.file_name = file_path
         file_props.name = file_name
@@ -886,7 +881,7 @@ class FileTransfersWindow:
     def _on_cleanup_button_clicked(self, widget):
         i = len(self.model) - 1
         while i >= 0:
-            iter_ = self.model.get_iter((i))
+            iter_ = self.model.get_iter(i)
             sid = self.model[iter_][Column.SID]
             file_props = FilesProp.getFilePropByType(sid[0], sid[1:])
             if is_transfer_stopped(file_props):
@@ -922,7 +917,7 @@ class FileTransfersWindow:
             file_props.paused = True
             self.set_status(file_props, 'pause')
             # Reset that to compute speed only when we resume
-            file_props.transfered_size = []
+            file_props.transferred_size = []
             self._toggle_pause_continue(False)
 
     def _on_cancel_button_clicked(self, widget):
@@ -1012,7 +1007,7 @@ class FileTransfersWindow:
                                                            int(event.y))[0]
         except TypeError:
             self._ui.transfers_list.get_selection().unselect_all()
-        if event.button == 3:  # Right click
+        if event.button == Gdk.BUTTON_SECONDARY:
             if path:
                 self._ui.transfers_list.get_selection().select_path(path)
                 iter_ = self.model.get_iter(path)
@@ -1029,9 +1024,8 @@ class FileTransfersWindow:
         file_props = FilesProp.getFilePropByType(sid[0], sid[1:])
         if not file_props.file_name:
             return
-        path = os.path.split(file_props.file_name)[0]
-        if os.path.exists(path) and os.path.isdir(path):
-            open_file(path)
+
+        show_in_folder(Path(file_props.file_name))
 
     def _on_cancel_menuitem_activate(self, widget):
         self._on_cancel_button_clicked(widget)
