@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from typing import Any
 
@@ -30,10 +20,10 @@ from gajim.common.const import MUC_DISCO_ERRORS
 from gajim.common.events import AccountConnected
 from gajim.common.events import AccountDisconnected
 from gajim.common.ged import EventHelper
-from gajim.common.helpers import get_random_muc_localpart
 from gajim.common.helpers import to_user_string
-from gajim.common.helpers import validate_jid
 from gajim.common.i18n import _
+from gajim.common.util.jid import validate_jid
+from gajim.common.util.muc import get_random_muc_localpart
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.dialogs import ErrorDialog
