@@ -1,24 +1,14 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
-from typing import Optional
+from __future__ import annotations
 
 import logging
 import time
 
 from gi.repository import Gdk
-from gi.repository import GLib
+from gi.repository import GObject
 from gi.repository import Gtk
 from nbxmpp import JID
 from nbxmpp.namespaces import Namespace
@@ -29,12 +19,14 @@ from gajim.common import app
 from gajim.common.const import AvatarSize
 from gajim.common.const import RFC5646_LANGUAGE_TAGS
 from gajim.common.const import XmppUriQuery
-from gajim.common.helpers import get_groupchat_name
-from gajim.common.helpers import open_uri
 from gajim.common.i18n import _
 from gajim.common.i18n import p_
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.util.muc import get_groupchat_name
+from gajim.common.util.uri import open_uri
 
 from gajim.gtk.builder import get_builder
+from gajim.gtk.contact_name_widget import ContactNameWidget
 from gajim.gtk.util import make_href_markup
 
 log = logging.getLogger('gajim.gtk.groupchat_info')
@@ -50,12 +42,12 @@ MUC_FEATURES = {
         _('This group chat is restricted '
           'to members only')),
     'muc_nonanonymous': (
-        'feather-shield-off-symbolic',
+        'feather-eye-symbolic',
         p_('Group chat feature', 'Not Anonymous'),
         _('All other group chat participants '
           'can see your XMPP address')),
     'muc_semianonymous': (
-        'feather-shield-symbolic',
+        'feather-eye-off-symbolic',
         p_('Group chat feature', 'Semi-Anonymous'),
         _('Only moderators can see your XMPP address')),
     'muc_moderated': (
@@ -69,11 +61,11 @@ MUC_FEATURES = {
         _('Participants entering this group chat are '
           'allowed to send messages')),
     'muc_public': (
-        'feather-eye-symbolic',
+        'lucide-megaphone-symbolic',
         p_('Group chat feature', 'Public'),
         _('Group chat can be found via search')),
     'muc_hidden': (
-        'feather-eye-off-symbolic',
+        'lucide-megaphone-off-symbolic',
         p_('Group chat feature', 'Hidden'),
         _('This group chat can not be found via search')),
     'muc_passwordprotected': (
@@ -104,10 +96,16 @@ MUC_FEATURES = {
 
 
 class GroupChatInfoScrolled(Gtk.ScrolledWindow):
+
+    __gsignals__ = {
+        'name-updated': (GObject.SignalFlags.RUN_LAST, None, (str,)),
+    }
+
     def __init__(self,
-                 account: Optional[str] = None,
+                 account: str | None = None,
                  width: int = 300,
-                 minimal: bool = False
+                 minimal: bool = False,
+                 edit_mode: bool = False
                  ) -> None:
         Gtk.ScrolledWindow.__init__(self)
         self.set_size_request(width, -1)
@@ -126,23 +124,32 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
                             Gtk.PolicyType.AUTOMATIC)
 
         self._account = account
-        self._info: Optional[DiscoInfo] = None
+        self._contact: GroupchatContact | None = None
+        self._info: DiscoInfo | None = None
 
         self._ui = get_builder('groupchat_info_scrolled.ui')
         self.add(self._ui.info_grid)
         self._ui.connect_signals(self)
+
+        self._contact_name_widget = ContactNameWidget(edit_mode=edit_mode)
+        self._contact_name_widget.connect('name-updated', self._on_contact_name_updated)
+
+        self._ui.name_box.add(self._contact_name_widget)
+
         self.show_all()
 
-    def get_account(self) -> Optional[str]:
+    def get_account(self) -> str | None:
         return self._account
 
     def set_account(self, account: str) -> None:
         self._account = account
 
-    def get_jid(self) -> Optional[JID]:
+    def get_jid(self) -> JID | None:
+        if self._contact is not None:
+            return self._contact.jid
         return self._info.jid
 
-    def set_subject(self, muc_subject: Optional[MucSubject]) -> None:
+    def set_subject(self, muc_subject: MucSubject | None) -> None:
         if muc_subject is None:
             return
 
@@ -157,11 +164,24 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
         self._ui.author_label.set_visible(has_author)
 
         has_subject = bool(muc_subject.text)
-        subject = GLib.markup_escape_text(muc_subject.text)
-        self._ui.subject.set_markup(make_href_markup(subject))
+        self._ui.subject.set_markup(make_href_markup(muc_subject.text))
         self._ui.subject.set_visible(has_subject)
         self._ui.subject_label.set_visible(has_subject)
 
+    def set_info_from_contact(self, contact: GroupchatContact) -> None:
+        self._contact = contact
+
+        self._contact_name_widget.set_contact(contact)
+
+        disco_info = contact.get_disco()
+        if disco_info is not None:
+            self.set_from_disco_info(disco_info)
+            return
+
+        self._ui.address.set_text(str(contact.jid))
+        self._ui.address.set_tooltip_text(str(contact.jid))
+        self._ui.address_copy_button.set_sensitive(True)
+
     def set_from_disco_info(self, info: DiscoInfo) -> None:
         self._info = info
         # Set name
@@ -177,18 +197,18 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
                 self.get_scale_factor())
             self._ui.avatar_image.set_from_surface(surface)
 
-        self._ui.name.set_text(name)
-        self._ui.name.set_visible(True)
+        self._contact_name_widget.update_displayed_name(name or '')
 
         # Set description
         has_desc = bool(info.muc_description)
-        self._ui.description.set_text(info.muc_description or '')
+        self._ui.description.set_markup(make_href_markup(info.muc_description))
         self._ui.description.set_visible(has_desc)
         self._ui.description_label.set_visible(has_desc)
 
         # Set address
         self._ui.address.set_text(str(info.jid))
         self._ui.address.set_tooltip_text(str(info.jid))
+        self._ui.address_copy_button.set_sensitive(True)
 
         if self._minimal:
             return
@@ -232,6 +252,12 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
 
         self._add_features(info.features)
 
+    def enable_edit_mode(self) -> None:
+        self._contact_name_widget.enable_edit_mode()
+
+    def _on_contact_name_updated(self, _widget: ContactNameWidget, name: str) -> None:
+        self.emit('name-updated', name)
+
     def _add_features(self, features: list[str]) -> None:
         grid = self._ui.info_grid
         for row in range(30, 9, -1):
@@ -258,8 +284,12 @@ class GroupChatInfoScrolled(Gtk.ScrolledWindow):
         grid.show_all()
 
     def _on_copy_address(self, _button: Gtk.Button) -> None:
+        if self._contact is not None:
+            jid = self._contact.jid
+        else:
+            jid = self._info.jid
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-        clipboard.set_text(self._info.jid.to_iri(XmppUriQuery.JOIN.value), -1)
+        clipboard.set_text(jid.to_iri(XmppUriQuery.JOIN.value), -1)
 
     @staticmethod
     def _on_activate_log_link(button: Gtk.LinkButton) -> int:
