@@ -1,21 +1,10 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 from typing import TYPE_CHECKING
 
 import logging
@@ -38,13 +27,16 @@ from gajim.common.const import Direction
 from gajim.common.const import Display
 from gajim.common.const import SimpleClientState
 from gajim.common.ged import EventHelper
-from gajim.common.helpers import open_file
-from gajim.common.helpers import open_uri
 from gajim.common.helpers import play_sound
-from gajim.common.helpers import show_in_folder
 from gajim.common.i18n import _
 from gajim.common.modules.bytestream import is_transfer_active
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.storage.archive.const import MessageType
+from gajim.common.util.uri import open_file
+from gajim.common.util.uri import open_uri
+from gajim.common.util.uri import show_in_folder
 from gajim.plugins.manifest import PluginManifest
 from gajim.plugins.repository import PluginRepository
 
@@ -56,18 +48,19 @@ from gajim.gtk.chat_list import ChatList
 from gajim.gtk.chat_list_row import ChatListRow
 from gajim.gtk.chat_stack import ChatStack
 from gajim.gtk.const import MAIN_WIN_ACTIONS
-from gajim.gtk.dialogs import ConfirmationCheckDialog
 from gajim.gtk.dialogs import ConfirmationDialog
 from gajim.gtk.dialogs import DialogButton
 from gajim.gtk.dialogs import ErrorDialog
 from gajim.gtk.dialogs import InputDialog
 from gajim.gtk.filechoosers import FileSaveDialog
+from gajim.gtk.main_menu_button import MainMenuButton
 from gajim.gtk.main_stack import MainStack
 from gajim.gtk.structs import AccountJidParam
 from gajim.gtk.structs import actionmethod
 from gajim.gtk.structs import AddChatActionParams
 from gajim.gtk.structs import ChatListEntryParam
-from gajim.gtk.structs import RetractMessageParam
+from gajim.gtk.structs import DeleteMessageParam
+from gajim.gtk.structs import ModerateMessageParam
 from gajim.gtk.util import get_app_window
 from gajim.gtk.util import open_window
 from gajim.gtk.util import resize_window
@@ -88,7 +81,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         EventHelper.__init__(self)
         self.set_application(app.app)
         self.set_title('Gajim')
-        self.set_default_icon_name('org.gajim.Gajim')
+        self.set_default_icon_name('gajim')
 
         app.window = self
 
@@ -107,6 +100,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
         self._chat_page = self._main_stack.get_chat_page()
 
+        self._ui.left_grid.attach(MainMenuButton(), 0, 0, 1, 1)
+
         self._app_page = self._main_stack.get_app_page()
         self._app_side_bar = AppSideBar(self._app_page)
         self._ui.app_box.add(self._app_side_bar)
@@ -121,6 +116,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self.connect('notify::is-active', self._on_window_active)
         self.connect('delete-event', self._on_window_delete)
         self.connect('window-state-event', self._on_window_state_changed)
+        self.connect_after('key-press-event', self._on_key_press_event)
 
         self._ui.connect_signals(self)
 
@@ -159,9 +155,13 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         app.app.systray.connect_unread_widget(chat_list_stack,
                                               'unread-count-changed')
 
+        self.set_show_menubar(app.settings.get_app_setting('show_main_menu'))
+
         for client in app.get_clients():
             client.connect_signal('state-changed',
                                   self._on_client_state_changed)
+            client.connect_signal('resume-successful',
+                                  self._on_client_resume_successful)
 
     def get_action(self, name: str) -> Gio.SimpleAction:
         action = self.lookup_action(name)
@@ -238,6 +238,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self._main_stack.add_account_page(event.account)
         client = app.get_client(event.account)
         client.connect_signal('state-changed', self._on_client_state_changed)
+        client.connect_signal('resume-successful',
+                              self._on_client_resume_successful)
 
     def _on_account_disabled(self, event: events.AccountDisabled) -> None:
         workspace_id = self._workspace_side_bar.get_first_workspace()
@@ -249,6 +251,47 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
     def update_account_unread_count(self, account: str, count: int) -> None:
         self._account_side_bar.update_unread_count(account, count)
 
+    def _on_key_press_event(
+        self,
+        _window: MainWindow,
+        event: Gdk.EventKey
+    ) -> bool:
+
+        # event.get_state() behaves different on Linux and Windows.
+        # On Linux its not set in the case that only a modifier key
+        # is pressed.
+
+        # Filter out modifier not used for shortcuts like Numlock (MOD2)
+        modifier = event.get_state() & Gtk.accelerator_get_default_mod_mask()
+        accel_name = Gtk.accelerator_name(event.keyval, modifier)
+
+        log.info('Captured key pressed: %s', accel_name)
+
+        if event.keyval in (
+            Gdk.KEY_Control_L,
+            Gdk.KEY_Control_R,
+            Gdk.KEY_Alt_L,
+            Gdk.KEY_Alt_R,
+            Gdk.KEY_Shift_L,
+            Gdk.KEY_Shift_R,
+        ):
+            return False
+
+        focused_widget = self.get_focus()
+        if (isinstance(focused_widget, Gtk.TextView)
+            and focused_widget.props.editable):
+                return False
+
+        if isinstance(focused_widget, Gtk.Entry):
+            return False
+
+        message_input = self.get_chat_stack().get_message_input()
+        if not message_input.get_mapped() or not message_input.is_sensitive():
+            return False
+
+        message_input.grab_focus()
+        return self.propagate_key_event(event)
+
     def _on_client_state_changed(self,
                                  client: Client,
                                  _signal_name: str,
@@ -257,6 +300,12 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         app.app.set_account_actions_state(client.account, state.is_connected)
         app.app.update_app_actions_state()
 
+    def _on_client_resume_successful(self,
+                                     client: Client,
+                                     _signal_name: str) -> None:
+
+        app.app.update_feature_actions_state(client.account)
+
     def _on_allow_gajim_update(self,
                                event: events.AllowGajimUpdateCheck) -> None:
         self.add_app_message(event.name)
@@ -288,7 +337,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
               'conversations will not be encrypted. Connecting PLAIN is '
               'strongly discouraged.') % {
                   'account': event.account,
-                  'server': app.get_hostname_from_account(event.account)},
+                  'server': app.get_hostname_from_account(
+                    event.account, prefer_custom=True)},
             [DialogButton.make('Cancel',
                                text=_('_Abort'),
                                callback=event.abort),
@@ -415,7 +465,11 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             ('switch-workspace-7', self._on_action),
             ('switch-workspace-8', self._on_action),
             ('switch-workspace-9', self._on_action),
+            ('increase-app-font-size', self._on_app_font_size_action),
+            ('decrease-app-font-size', self._on_app_font_size_action),
+            ('reset-app-font-size', self._on_app_font_size_action),
             ('toggle-chat-list', self._on_action),
+            ('toggle-menu-bar', self._on_action),
             ('preview-download', self._on_preview_action),
             ('preview-open', self._on_preview_action),
             ('preview-save-as', self._on_preview_action),
@@ -423,7 +477,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             ('preview-copy-link', self._on_preview_action),
             ('preview-open-link', self._on_preview_action),
             ('copy-message', self._on_copy_message),
-            ('retract-message', self._on_retract_message),
+            ('moderate-message', self._on_moderate_message),
             ('delete-message-locally', self._on_delete_message_locally),
             ('add-workspace', self._add_workspace),
             ('edit-workspace', self._edit_workspace),
@@ -441,7 +495,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def _on_action(self,
                    action: Gio.SimpleAction,
-                   _param: Optional[GLib.Variant]) -> Optional[int]:
+                   _param: GLib.Variant | None) -> int | None:
 
         action_name = action.get_name()
         log.info('Activate action: %s', action_name)
@@ -476,6 +530,9 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                                             control.contact.jid)
                 return None
 
+        if action_name == 'escape' and app.settings.get('escape_key_closes'):
+            self.emit('delete-event', Gdk.Event())
+
         if action_name == 'restore-chat':
             self._chat_page.restore_chat()
 
@@ -502,8 +559,41 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         elif action_name == 'toggle-chat-list':
             self._toggle_chat_list()
 
+        elif action_name == 'toggle-menu-bar':
+            show_menu_bar = not self.get_show_menubar()
+            app.settings.set_app_setting('show_main_menu', show_menu_bar)
+            self.set_show_menubar(show_menu_bar)
+
         return None
 
+    def _on_app_font_size_action(
+            self,
+            action: Gio.SimpleAction,
+            _param: GLib.Variant
+            ) -> None:
+
+        action_name = action.get_name()
+        if action_name == 'reset-app-font-size':
+            app.settings.set_app_setting('app_font_size', None)
+            app.css_config.apply_app_font_size()
+            return
+
+        app_font_size = app.settings.get('app_font_size')
+        new_app_font_size = app_font_size
+        if action_name == 'increase-app-font-size':
+            new_app_font_size = app_font_size + 0.125
+        elif action_name == 'decrease-app-font-size':
+            new_app_font_size = app_font_size - 0.125
+
+        # Clamp font size
+        new_app_font_size = max(min(1.5, new_app_font_size), 1.0)
+
+        if new_app_font_size == app_font_size:
+            return
+
+        app.settings.set_app_setting('app_font_size', new_app_font_size)
+        app.css_config.apply_app_font_size()
+
     def _on_preview_action(self,
                            action: Gio.SimpleAction,
                            param: GLib.Variant
@@ -532,6 +622,13 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
         elif action_name == 'preview-save-as':
             def _on_ok(paths: list[str]) -> None:
+                if not paths:
+                    ErrorDialog(
+                        _('Could not save file'),
+                        _('Could not save file to selected directory.'),
+                        transient_for=self)
+                    return
+
                 target = paths[0]
                 assert preview is not None
                 assert preview.orig_path is not None
@@ -553,7 +650,16 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                         transient_for=self)
                     return
 
-                shutil.copyfile(preview.orig_path, target_path)
+                try:
+                    shutil.copyfile(preview.orig_path, target_path)
+                except PermissionError as e:
+                    ErrorDialog(
+                        _('Could not save file'),
+                        _('You do not have permissions for this directory.\n'
+                          'Error: %s.') % e,
+                        transient_for=self)
+                    return
+
                 app.settings.set('last_save_dir', str(target_path.parent))
 
             if not preview.orig_exists:
@@ -629,37 +735,43 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         clip.set_text(param.get_string(), -1)
 
     @actionmethod
-    def _on_retract_message(self,
+    def _on_moderate_message(self,
                             _action: Gio.SimpleAction,
-                            params: RetractMessageParam
+                            params: ModerateMessageParam
                             ) -> None:
 
-        def _on_retract(reason: str) -> None:
+        def _on_moderate(reason: str) -> None:
             client = app.get_client(params.account)
-            client.get_module('MUC').retract_message(
-                params.jid, params.stanza_id, reason or None)
+            client.get_module('MUC').moderate_message(
+                params.namespace,
+                params.jid,
+                params.stanza_id,
+                reason or None
+            )
 
         InputDialog(
-            _('Retract Message'),
-            _('Retract message?'),
-            _('Why do you want to retract this message?'),
+            _('Moderate Message'),
+            _('Moderate message?'),
+            _('Why do you want to moderate this message?'),
             [DialogButton.make('Cancel'),
              DialogButton.make('Remove',
-                               text=_('_Retract'),
-                               callback=_on_retract)],
+                               text=_('_Moderate'),
+                               callback=_on_moderate)],
             input_str=_('Spam'),
             transient_for=app.window).show()
 
+    @actionmethod
     def _on_delete_message_locally(self,
                                    _action: Gio.SimpleAction,
-                                   param: GLib.Variant
+                                   params: DeleteMessageParam
                                    ) -> None:
 
         def _on_delete() -> None:
-            log_line_id = param.get_uint32()
-            app.storage.archive.delete_message_from_logs(log_line_id)
-            control = self.get_control()
-            control.remove_message(log_line_id)
+            app.storage.archive.delete_message(params.pk)
+            app.ged.raise_event(
+                events.MessageDeleted(account=params.account,
+                                      jid=params.jid,
+                                      pk=params.pk))
 
         ConfirmationDialog(
             _('Delete Message'),
@@ -690,33 +802,17 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                           _event: Gdk.Event
                           ) -> int:
 
+        if app.settings.get('confirm_on_window_delete'):
+            open_window('QuitDialog')
+            return Gdk.EVENT_STOP
+
         action = app.settings.get('action_on_close')
         if action == 'hide':
             self.hide()
-            return Gdk.EVENT_STOP
-
-        if action == 'minimize':
+        elif action == 'minimize':
             self.minimize()
-            return Gdk.EVENT_STOP
-
-        if not app.settings.get('confirm_on_window_delete'):
+        else:
             self.quit()
-            return Gdk.EVENT_STOP
-
-        def _on_ok(is_checked: bool) -> None:
-            if is_checked:
-                app.settings.set('confirm_on_window_delete', False)
-            self.quit()
-
-        ConfirmationCheckDialog(
-            _('Quit Gajim'),
-            _('You are about to quit Gajim'),
-            _('Are you sure you want to quit Gajim?'),
-            _('_Don’t ask again'),
-            [DialogButton.make('Cancel'),
-             DialogButton.make('Remove',
-                               text=_('_Quit'),
-                               callback=_on_ok)]).show()
 
         return Gdk.EVENT_STOP
 
@@ -755,7 +851,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self._account_side_bar.activate_account_page(account)
         self._main_stack.show_account(account)
 
-    def get_active_workspace(self) -> Optional[str]:
+    def get_active_workspace(self) -> str | None:
         return self._workspace_side_bar.get_active_workspace()
 
     def is_chat_active(self, account: str, jid: JID) -> bool:
@@ -801,7 +897,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             self.add_workspace(workspace_id)
 
     def add_workspace(self,
-                      workspace_id: Optional[str] = None,
+                      workspace_id: str | None = None,
                       switch: bool = True) -> str:
 
         if workspace_id is None:
@@ -967,7 +1063,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                  jid: JID,
                  type_: str,
                  select: bool = False,
-                 message: Optional[str] = None
+                 message: str | None = None
                  ) -> None:
 
         workspace_id = self._get_suitable_workspace(account, jid)
@@ -1032,8 +1128,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def add_app_message(self,
                         category: str,
-                        new_version: Optional[str] = None,
-                        new_setup_url: Optional[str] = None
+                        new_version: str | None = None,
+                        new_setup_url: str | None = None
                         ) -> None:
 
         self._app_page.add_app_message(category, new_version, new_setup_url)
@@ -1096,16 +1192,19 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             # Read marker must be sent only once
             return
 
-        last_message = app.storage.archive.get_last_conversation_line(
+        last_message = app.storage.archive.get_last_conversation_row(
             account, jid)
-        if last_message is None:
+        if last_message is None or last_message.id is None:
             return
 
         client = app.get_client(account)
         contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
+
         client.get_module('ChatMarkers').send_displayed_marker(
             contact,
-            last_message.message_id,
+            last_message.id,
             last_message.stanza_id)
 
     def _on_window_active(self,
@@ -1126,7 +1225,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def get_preferred_ft_method(self,
                                 contact: types.ChatContactT
-                                ) -> Optional[str]:
+                                ) -> str | None:
 
         ft_pref = app.settings.get_account_setting(
             contact.account,
@@ -1154,21 +1253,22 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
     def show_add_join_groupchat(self,
                                 account: str,
                                 jid: str,
-                                nickname: Optional[str] = None,
-                                password: Optional[str] = None
+                                nickname: str | None = None,
+                                password: str | None = None
                                 ) -> None:
 
-        if not self.chat_exists(account, JID.from_string(jid)):
+        jid_ = JID.from_string(jid)
+        if not self.chat_exists(account, jid_):
             client = app.get_client(account)
             client.get_module('MUC').join(
-                jid, nick=nickname, password=password)
+                jid_, nick=nickname, password=password)
 
-        self.add_group_chat(account, JID.from_string(jid), select=True)
+        self.add_group_chat(account, jid_, select=True)
 
     def start_chat_from_jid(self,
                             account: str,
                             jid: str,
-                            message: Optional[str] = None
+                            message: str | None = None
                             ) -> None:
 
         jid_ = JID.from_string(jid)
@@ -1189,20 +1289,20 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         open_window('ContactInfo', account=account, contact=contact)
 
     @staticmethod
-    def execute_command(account: str, jid: str) -> None:
-        # TODO: Resource?
-        open_window('AdHocCommands', account=account, jid=jid)
+    def execute_command(account: str, jids: list[str]) -> None:
+        open_window('AdHocCommands', account=account, jids=jids)
 
     def block_contact(self, account: str, jid: str) -> None:
         client = app.get_client(account)
 
         contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
         if contact.is_blocked:
             client.get_module('Blocking').unblock([jid])
             return
 
         # TODO: Keep "confirm_block" setting?
-        def _block_contact(report: Optional[str] = None) -> None:
+        def _block_contact(report: str | None = None) -> None:
             client.get_module('Blocking').block([contact.jid], report)
             self._chat_page.remove_chat(account, contact.jid)
 
@@ -1229,6 +1329,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             client.get_module('Roster').delete_item(jid)
 
         contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
         sec_text = _('You are about to remove %(name)s (%(jid)s) from '
                      'your contact list.\n') % {
                          'name': contact.name,
@@ -1262,27 +1364,24 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self._set_startup_finished()
 
     def _on_message_received(self, event: events.MessageReceived) -> None:
-        if not self.chat_exists(event.account, event.jid):
-            if not event.properties.body:
-                # Don’t open control on chatstate etc.
-                return
+        if self.chat_exists(event.account, event.jid):
+            return
 
-            if event.properties.is_muc_pm:
-                self.add_private_chat(event.account,
-                                      event.properties.jid)
+        if event.m_type == MessageType.PM:
+            self.add_private_chat(event.account,
+                                  event.jid)
 
-            else:
-                jid = event.properties.jid.new_as_bare()
-                self.add_chat(event.account, jid, 'contact')
+        else:
+            self.add_chat(event.account, event.jid, 'contact')
 
     def _on_read_state_sync(self, event: events.ReadStateSync) -> None:
-        last_message = app.storage.archive.get_last_conversation_line(
+        last_message = app.storage.archive.get_last_conversation_row(
             event.account, event.jid)
 
         if last_message is None:
             return
 
-        if event.marker_id not in (last_message.message_id,
+        if event.marker_id not in (last_message.id,
                                    last_message.stanza_id):
             return
 
@@ -1314,7 +1413,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         app.settings.set('mainwin_height', window_height)
         app.settings.save()
 
-        def on_continue2(message: Optional[str]) -> None:
+        def on_continue2(message: str | None) -> None:
             if 'file_transfers' not in app.interface.instances:
                 app.app.start_shutdown(message=message)
                 return
@@ -1342,7 +1441,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                 return
             app.app.start_shutdown(message=message)
 
-        def on_continue(message: Optional[str]) -> None:
+        def on_continue(message: str | None) -> None:
             if message is None:
                 # user pressed Cancel to change status message dialog
                 return
