@@ -1,19 +1,8 @@
-
 # Copyright (C) 2009-2014 Yann Leboulanger <asterix AT lagaule.org>
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -34,10 +23,6 @@ from gajim.common import app
 from gajim.common import types
 from gajim.common.const import URIType
 from gajim.common.const import XmppUriQuery
-from gajim.common.helpers import filesystem_path_from_uri
-from gajim.common.helpers import is_affiliation_change_allowed
-from gajim.common.helpers import is_retraction_allowed
-from gajim.common.helpers import is_role_change_allowed
 from gajim.common.helpers import jid_is_blocked
 from gajim.common.i18n import _
 from gajim.common.i18n import get_short_lang_code
@@ -47,16 +32,22 @@ from gajim.common.modules.contacts import can_add_to_roster
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.preview import Preview
+from gajim.common.storage.archive.const import MessageState
 from gajim.common.structs import URI
+from gajim.common.util.muc import is_affiliation_change_allowed
+from gajim.common.util.muc import is_moderation_allowed
+from gajim.common.util.muc import is_role_change_allowed
 from gajim.common.util.text import escape_iri_path_segment
+from gajim.common.util.uri import filesystem_path_from_uri
 
 from gajim.gtk.const import MuteState
 from gajim.gtk.structs import AccountJidParam
 from gajim.gtk.structs import AddChatActionParams
 from gajim.gtk.structs import ChatListEntryParam
+from gajim.gtk.structs import DeleteMessageParam
+from gajim.gtk.structs import ModerateMessageParam
 from gajim.gtk.structs import MuteContactParam
 from gajim.gtk.structs import RemoveHistoryActionParams
-from gajim.gtk.structs import RetractMessageParam
 from gajim.gtk.util import GajimMenu
 from gajim.gtk.util import MenuItemListT
 
@@ -64,6 +55,40 @@ UriMenuItemsT = list[tuple[str, list[str], str]]
 UriMenuBuilderT = Callable[[URI, str], UriMenuItemsT]
 
 
+def get_main_menu() -> GajimMenu:
+    main_menu = GajimMenu()
+
+    gajim_menu_items: MenuItemListT = [
+        (_('_Start / Join Chat…'), 'app.start-chat', GLib.Variant('as', ['', ''])),
+        (_('Create _Group Chat…'), 'app.create-groupchat', GLib.Variant('s', '')),
+        (_('Pl_ugins'), 'app.plugins', None),
+        (_('_Preferences'), 'app.preferences', None),
+        (_('_Quit'), 'app.quit', None),
+    ]
+    main_menu.append_submenu('_Gajim', GajimMenu.from_list(gajim_menu_items))
+
+    main_menu.add_submenu(_('_Accounts'))
+
+    view_menu_items: MenuItemListT = [
+        (_('_Debug Console'), 'app.xml-console', None),
+        (_('_File Transfer'), 'app.file-transfer', None),
+        (_('_Toggle Menu Bar'), 'win.toggle-menu-bar', None),
+    ]
+    main_menu.append_submenu(_('_View'), GajimMenu.from_list(view_menu_items))
+
+    help_menu_items: MenuItemListT = [
+        (_('_Wiki (Online)'), 'app.content', None),
+        (_('FA_Q (Online)'), 'app.faq', None),
+        (_('_Privacy Policy (Online)'), 'app.privacy-policy', None),
+        (_('Join Support Chat'), 'app.join-support-chat', None),
+        (_('_Keyboard Shortcuts'), 'app.shortcuts', None),
+        (_('_Features'), 'app.features', None),
+        (_('_About'), 'app.about', None),
+    ]
+    main_menu.append_submenu(_('_Help'), GajimMenu.from_list(help_menu_items))
+
+    return main_menu
+
 def get_self_contact_menu(contact: types.BareContact) -> GajimMenu:
     account = contact.account
     jid = contact.jid
@@ -97,10 +122,14 @@ def get_singlechat_menu(contact: types.BareContact) -> GajimMenu:
         params = AccountJidParam(account=account, jid=contact.jid)
         menu.add_item(_('Add to Contact List…'), 'win.add-to-roster', params)
 
+    jids = [str(c.jid) for c in contact.get_resources()]
+    if not jids:
+        jids = [contact.jid.bare]
+
     menu.add_item(
         _('Execute Command…'),
         f'app.{account}-execute-command',
-        contact.jid.bare,
+        GLib.Variant.new_strv(jids),
     )
 
     return menu
@@ -144,6 +173,9 @@ def get_account_menu(account: str) -> GajimMenu:
 
     client = app.get_client(account)
     server_jid = client.get_own_jid().domain
+    assert server_jid is not None
+
+    server_jid = GLib.Variant.new_strv([server_jid])
 
     menuitems: MenuItemListT = [
         (_('Profile'), f'app.{account}-profile', account),
@@ -158,7 +190,7 @@ def get_account_menu(account: str) -> GajimMenu:
         (_('Archiving Preferences'), f'app.{account}-archive', account),
         (_('Blocking List'), f'app.{account}-blocking', account),
         (_('PEP Configuration'), f'app.{account}-pep-config', account),
-        (_('Synchronise History…'), f'app.{account}-sync-history', account),
+        (_('Synchronize History…'), f'app.{account}-sync-history', account),
     ]
 
     if app.settings.get('developer_modus'):
@@ -188,9 +220,9 @@ def build_accounts_menu() -> None:
         return
 
     if len(accounts_list) > 1:
-        modify_accounts_item = Gio.MenuItem.new(_('_Modify Accounts…'),
+        manage_accounts_item = Gio.MenuItem.new(_('_Manage Accounts…'),
                                                 'app.accounts::')
-        acc_menu.append_item(modify_accounts_item)
+        acc_menu.append_item(manage_accounts_item)
         add_contact_item = Gio.MenuItem.new(_('Add _Contact…'),
                                             'app.add-contact::')
         acc_menu.append_item(add_contact_item)
@@ -199,9 +231,9 @@ def build_accounts_menu() -> None:
             acc_menu.append_submenu(label, get_account_menu(acc))
     else:
         acc_menu = get_account_menu(accounts_list[0])
-        modify_account_item = Gio.MenuItem.new(_('_Modify Account…'),
+        manage_account_item = Gio.MenuItem.new(_('_Manage Account…'),
                                                'app.accounts::')
-        acc_menu.insert_item(0, modify_account_item)
+        acc_menu.insert_item(0, manage_account_item)
         add_contact_item = Gio.MenuItem.new(_('Add _Contact…'),
                                             'app.add-contact::')
         acc_menu.insert_item(1, add_contact_item)
@@ -413,17 +445,27 @@ def populate_uri_context_menu(menu: Gtk.Menu, account: str, uri: URI) -> None:
         menu.append(menuitem)
 
 
-def get_roster_menu(account: str, jid: str, gateway: bool = False) -> GajimMenu:
+def get_roster_menu(
+    account: str,
+    contact: BareContact,
+    gateway: bool = False
+) -> GajimMenu:
 
     block_label = _('Block…')
-    if jid_is_blocked(account, jid):
+    if jid_is_blocked(account, str(contact.jid)):
         block_label = _('Unblock')
 
-    value = str(jid)
+    value = str(contact.jid)
+
+    jids = [str(c.jid) for c in contact.get_resources()]
+    if not jids:
+        jids = [contact.jid.bare]
+
+    jids_v = GLib.Variant.new_strv(jids)
 
     menuitems: MenuItemListT = [
         (_('Details'), f'app.{account}-contact-info', value),
-        (_('Execute Command…'), f'app.{account}-execute-command', value),
+        (_('Execute Command…'), f'app.{account}-execute-command', jids_v),
         (block_label, f'app.{account}-block-contact', value),
         (_('Remove…'), f'app.{account}-remove-contact', value),
     ]
@@ -533,7 +575,7 @@ def get_chat_list_row_menu(workspace_id: str,
 
     if app.window.get_chat_unread_count(account, jid, include_silent=True):
         params = AccountJidParam(account=account, jid=jid)
-        menu.add_item(_('Mark as read'), 'win.mark-as-read', params)
+        menu.add_item(_('Mark as read'), f'app.{account}-mark-as-read', params)
 
     if contact.is_muted:
         menu.add_item(
@@ -683,14 +725,17 @@ def get_chat_row_menu(contact: types.ChatContactT,
                       timestamp: datetime,
                       message_id: str | None,
                       stanza_id: str | None,
-                      log_line_id: int | None
+                      pk: int | None,
+                      corrected_pk: int | None,
+                      state: MessageState,
+                      is_moderated: bool
                       ) -> GajimMenu:
 
     menu_items: MenuItemListT = []
 
+    # Text can be an empty string,
+    # e.g. if a preview has not been loaded yet
     if text:
-        # Text can be an empty string,
-        # e.g. if a preview has not been loaded yet
         timestamp_formatted = timestamp.strftime(
             app.settings.get('date_time_format'))
 
@@ -707,20 +752,27 @@ def get_chat_row_menu(contact: types.ChatContactT,
 
         show_quote = True
         if isinstance(contact, GroupchatContact):
-            if contact.is_joined:
+            if contact.is_joined and state == MessageState.ACKNOWLEDGED:
                 self_contact = contact.get_self()
                 assert self_contact is not None
                 show_quote = not self_contact.role.is_visitor
             else:
                 show_quote = False
-        if show_quote:
+
+        show_reply = True
+        if isinstance(contact, GroupchatContact):
+            if contact.is_joined and stanza_id is not None:
+                self_contact = contact.get_self()
+                assert self_contact is not None
+                show_reply = not self_contact.role.is_visitor
+            else:
+                show_reply = False
+
+        if not show_reply and show_quote:
+            # Use XEP-0393 quotes if XEP-0461 message reply is not available
             menu_items.append((
                 p_('Message row action', 'Quote…'), 'win.quote', text))
 
-    menu_items.append(
-        (p_('Message row action', 'Select Messages…'),
-         'win.activate-message-selection',
-         GLib.Variant('u', log_line_id or 0)))
 
     show_correction = False
     if message_id is not None:
@@ -730,33 +782,51 @@ def get_chat_row_menu(contact: types.ChatContactT,
         menu_items.append((
             p_('Message row action', 'Correct…'), 'win.correct-message', None))
 
-    show_retract = False
     if isinstance(contact, GroupchatContact) and contact.is_joined:
         resource_contact = contact.get_resource(name)
         self_contact = contact.get_self()
         assert self_contact is not None
-        is_allowed = is_retraction_allowed(self_contact, resource_contact)
+        is_allowed = is_moderation_allowed(self_contact, resource_contact)
 
         disco_info = app.storage.cache.get_last_disco_info(contact.jid)
         assert disco_info is not None
 
-        if disco_info.has_message_moderation and is_allowed:
-            show_retract = True
-    if show_retract and stanza_id is not None:
-        param = RetractMessageParam(
+        if (
+            disco_info.has_message_moderation and
+            stanza_id is not None and
+            state == MessageState.ACKNOWLEDGED and
+            not is_moderated and
+            is_allowed
+        ):
+
+            ns = disco_info.moderation_namespace
+            assert ns is not None
+            param = ModerateMessageParam(
+                account=contact.account,
+                jid=contact.jid,
+                stanza_id=stanza_id,
+                namespace=ns)
+            menu_items.append((
+                p_('Message row action', 'Moderate…'),
+                'win.moderate-message',
+                param))
+
+    menu_items.append(
+        (p_('Message row action', 'Select Messages…'),
+         'win.activate-message-selection',
+         GLib.Variant('u', corrected_pk or 0)))
+
+    if pk is not None and state == MessageState.ACKNOWLEDGED:
+        param = DeleteMessageParam(
             account=contact.account,
             jid=contact.jid,
-            stanza_id=stanza_id)
-        menu_items.append((
-            p_('Message row action', 'Retract…'),
-            'win.retract-message',
-            param))
+            pk=pk)
 
-    if log_line_id is not None:
         menu_items.append(
-            (p_('Message row action', 'Delete Message Locally…'),
-            'win.delete-message-locally',
-            GLib.Variant('u', log_line_id or 0)))
+            (p_('Message row action',
+             'Delete Message Locally…'),
+             'win.delete-message-locally',
+              param))
 
     return GajimMenu.from_list(menu_items)
 
