@@ -1,21 +1,10 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import cast
-from typing import Optional
 
 import locale
 import logging
@@ -45,7 +34,7 @@ class ResourceSelector(Gtk.ScrolledWindow):
 
     def __init__(self,
                  contact: BareContact,
-                 constraints: Optional[list[str]] = None) -> None:
+                 constraints: list[str] | None = None) -> None:
         Gtk.ScrolledWindow.__init__(self)
         self.set_shadow_type(Gtk.ShadowType.IN)
         self.set_size_request(-1, 200)
@@ -154,7 +143,7 @@ class ResourceRow(Gtk.ListBoxRow):
 
     @staticmethod
     def _get_client_identity(disco_info: DiscoInfo
-                             ) -> tuple[Optional[str], Optional[str]]:
+                             ) -> tuple[str | None, str | None]:
         for identity in disco_info.identities:
             if identity.category == 'client':
                 return identity.name, identity.type
