@@ -1,63 +1,49 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
 from typing import Literal
-from typing import Optional
 
 import locale
 import logging
-
-from enum import IntEnum
 from collections import defaultdict
+from enum import IntEnum
 
 from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
-
 from nbxmpp import JID
 from nbxmpp.const import PresenceShow
 from nbxmpp.namespaces import Namespace
 
 from gajim.common import app
 from gajim.common import ged
-from gajim.common import types
 from gajim.common.const import AvatarSize
-from gajim.common.const import StyleAttr
 from gajim.common.const import PresenceShowExt
+from gajim.common.const import StyleAttr
 from gajim.common.events import ApplicationEvent
 from gajim.common.events import RosterPush
 from gajim.common.events import RosterReceived
-from gajim.common.helpers import event_filter
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
+from gajim.common.util.decorators import event_filter
 
-from .menus import get_roster_menu
-from .dialogs import ConfirmationDialog
-from .dialogs import DialogButton
-from .tooltips import RosterTooltip
-from .util import EventHelper
-from .builder import get_builder
-from .util import open_window
+from gajim.gtk.builder import get_builder
+from gajim.gtk.dialogs import ConfirmationDialog
+from gajim.gtk.dialogs import DialogButton
+from gajim.gtk.menus import get_roster_menu
+from gajim.gtk.tooltips import ContactTooltip
+from gajim.gtk.util import EventHelper
+from gajim.gtk.util import GajimPopover
+from gajim.gtk.util import open_window
 
-log = logging.getLogger('gajim.gui.roster')
+log = logging.getLogger('gajim.gtk.roster')
 
 
-HANDLED_EVENTS = []
-
 DEFAULT_GROUP = _('Contacts')
 
 
@@ -81,7 +67,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         self._client = app.get_client(account)
         self._contacts = self._client.get_module('Contacts')
 
-        self._roster_tooltip = RosterTooltip()
+        self._roster_tooltip = ContactTooltip()
 
         self._ui = get_builder('roster.ui')
         self._ui.roster_treeview.set_model(None)
@@ -117,8 +103,6 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         self._ui.connect_signals(self)
 
         self.register_events([
-            ('account-connected', ged.CORE, self._on_account_state),
-            ('account-disconnected', ged.CORE, self._on_account_state),
             ('roster-received', ged.CORE, self._on_roster_received),
             ('theme-update', ged.GUI2, self._on_theme_update),
             ('roster-push', ged.GUI2, self._on_roster_push),
@@ -135,69 +119,36 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         self._filter_enabled = False
         self._filter_string = ''
 
-        self._add_actions()
+        app.settings.connect_signal('showoffline', self._on_setting_changed)
+        app.settings.connect_signal('sort_by_show_in_roster',
+                                    self._on_setting_changed)
+
+        self._action_signal_ids = self._connect_actions()
         self._initial_draw()
 
-    def _add_actions(self):
-        actions = [
-            ('contact-info', self._on_contact_info),
-            ('modify-gateway', self._on_modify_gateway),
-            ('execute-command', self._on_execute_command),
-            ('block-contact', self._on_block_contact),
-            ('remove-contact', self._on_remove_contact),
+    def _connect_actions(self) -> dict[Gio.Action, int]:
+        app_actions = [
+            (f'{self._account}-contact-info', self._on_contact_info),
+            (f'{self._account}-modify-gateway', self._on_modify_gateway),
+            (f'{self._account}-execute-command', self._on_execute_command),
+            (f'{self._account}-block-contact', self._on_block_contact),
+            (f'{self._account}-remove-contact', self._on_remove_contact),
         ]
-        for action in actions:
+
+        signal_ids: dict[Gio.Action, int] = {}
+        for action in app_actions:
             action_name, func = action
-            act = Gio.SimpleAction.new(
-                f'{action_name}-{self._account}', GLib.VariantType.new('s'))
-            act.connect('activate', func)
-            app.window.add_action(act)
+            action = app.app.lookup_action(action_name)
+            assert action is not None
+            signal_id = action.connect('activate', func)
+            signal_ids[action] = signal_id
 
-        action = Gio.SimpleAction.new_stateful(
-            f'show-offline-{self._account}',
-            None,
-            GLib.Variant.new_boolean(app.settings.get('showoffline')))
-        action.connect('change-state', self._on_show_offline)
-        app.window.add_action(action)
+        return signal_ids
 
-        action = Gio.SimpleAction.new_stateful(
-            f'sort-by-show-{self._account}',
-            None,
-            GLib.Variant.new_boolean(
-                app.settings.get('sort_by_show_in_roster')))
-        action.connect('change-state', self._on_sort_by_show)
-        app.window.add_action(action)
-
-    def update_actions(self):
-        online = app.account_is_connected(self._account)
-        blocking_support = self._client.get_module('Blocking').supported
-
-        actions = [('contact-info', online),
-                   ('modify-gateway', online),
-                   ('execute-command', online),
-                   ('block-contact', online and blocking_support),
-                   ('remove-contact', online)]
-
-        for action, enabled in actions:
-            act = app.window.lookup_action(f'{action}-{self._account}')
-            assert act is not None
-            act.set_enabled(enabled)
-
-    def _remove_actions(self):
-        actions = [
-            'contact-info',
-            'modify-gateway',
-            'execute-command',
-            'block-contact',
-            'remove-contact',
-            'show-offline',
-            'sort-by-show',
-        ]
-        for action in actions:
-            app.window.remove_action(f'{action}-{self._account}')
-
-    def _on_account_state(self, _event: ApplicationEvent) -> None:
-        self.update_actions()
+    def _get_contact(self, jid: str) -> BareContact:
+        contact = self._client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
+        return contact
 
     def _on_theme_update(self, _event: ApplicationEvent) -> None:
         self.redraw()
@@ -300,24 +251,12 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
         contact = self._contacts.get_bare_contact(
             model[iter_][Column.JID_OR_GROUP])
-        value, widget = self._roster_tooltip.get_tooltip(path, contact)
+        assert isinstance(contact, BareContact)
+        value, widget = self._roster_tooltip.get_tooltip(contact)
         tooltip.set_custom(widget)
         return value
 
-    def _on_show_offline(self,
-                         action: Gio.SimpleAction,
-                         value: Optional[GLib.Variant]) -> None:
-
-        assert value is not None
-        action.set_state(value)
-        app.settings.set('showoffline', value.get_boolean())
-        self._refilter()
-
-    def _on_sort_by_show(self,
-                         action: Gio.SimpleAction,
-                         value: GLib.Variant) -> None:
-        action.set_state(value)
-        app.settings.set('sort_by_show_in_roster', value.get_boolean())
+    def _on_setting_changed(self, *args: Any) -> None:
         self._refilter()
 
     def _on_contact_info(self,
@@ -338,7 +277,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
                             _action: Gio.SimpleAction,
                             param: GLib.Variant) -> None:
 
-        app.window.execute_command(self._account, param.get_string())
+        app.window.execute_command(self._account, param.get_strv())
 
     def _on_block_contact(self,
                           _action: Gio.SimpleAction,
@@ -352,6 +291,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
         jid = JID.from_string(param.get_string())
         selected_contact = self._contacts.get_contact(jid)
+        assert isinstance(selected_contact, BareContact)
         if selected_contact.is_gateway:
             # Check for transport users in roster and warn about removing the
             # transport if there are any
@@ -363,7 +303,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
             def _on_remove():
                 self._client.get_module('Gateway').unsubscribe(
-                    selected_contact.jid)
+                    str(selected_contact.jid))
 
             if has_transport_contacts:
                 ConfirmationDialog(
@@ -401,7 +341,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
                                       treeview: Gtk.TreeView,
                                       event: Gdk.EventButton) -> None:
 
-        if event.button not in (2, 3):
+        if event.button == Gdk.BUTTON_PRIMARY:
             return
 
         pos = treeview.get_path_at_pos(int(event.x), int(event.y))
@@ -421,11 +361,11 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
         jid = self._store[iter_][Column.JID_OR_GROUP]
 
-        if event.button == 3:  # right click
+        if event.button == Gdk.BUTTON_SECONDARY:
             self._show_contact_menu(jid, treeview, event)
 
         jid = JID.from_string(jid)
-        if event.button == 2:  # middle click
+        if event.button == Gdk.BUTTON_MIDDLE:
             app.window.add_chat(self._account, jid, 'contact', select=True)
 
     @staticmethod
@@ -438,20 +378,13 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
                            event: Gdk.EventButton) -> None:
 
         contact = self._contacts.get_bare_contact(jid)
+        assert isinstance(contact, BareContact)
         gateway_register = contact.is_gateway and contact.supports(
             Namespace.REGISTER)
+
         menu = get_roster_menu(
-            self._account, jid, gateway=gateway_register)
-
-        rectangle = Gdk.Rectangle()
-        rectangle.x = int(event.x)
-        rectangle.y = int(event.y)
-        rectangle.width = rectangle.height = 1
-
-        popover = Gtk.Popover.new_from_model(self, menu)
-        popover.set_relative_to(treeview)
-        popover.set_position(Gtk.PositionType.RIGHT)
-        popover.set_pointing_to(rectangle)
+            self._account, contact, gateway=gateway_register)
+        popover = GajimPopover(menu, relative_to=treeview, event=event)
         popover.popup()
 
     def set_search_string(self, text: str) -> None:
@@ -472,17 +405,14 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
         return self._filter_string in model[iter_][Column.TEXT].lower()
 
-    def _get_contact_visible(self, contact: types.BareContact) -> bool:
+    def _get_contact_visible(self, contact: BareContact) -> bool:
         if self._filter_enabled:
             return self._filter_string in contact.name.lower()
 
         if app.settings.get('showoffline'):
             return True
 
-        if contact.show is PresenceShowExt.OFFLINE:
-            return False
-
-        return True
+        return contact.show is not PresenceShowExt.OFFLINE
 
     def _set_model(self) -> None:
         self._roster.set_model(self._modelfilter)
@@ -510,21 +440,20 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
     def _initial_draw(self) -> None:
         for contact in self._client.get_module('Roster').iter_contacts():
-            self._connect_contact_signals(contact)
             self._add_or_update_contact(contact)
 
         self._enable_sort(True)
         self._set_model()
         self._roster.expand_all()
 
-    def _connect_contact_signals(self, contact: types.BareContact) -> None:
+    def _connect_contact_signals(self, contact: BareContact) -> None:
         contact.connect('presence-update', self._on_contact_update)
         contact.connect('caps-update', self._on_contact_update)
         contact.connect('avatar-update', self._on_contact_update)
         contact.connect('blocking-update', self._on_contact_update)
 
     def _on_contact_update(self,
-                           contact: types.BareContact,
+                           contact: BareContact,
                            _signal_name: str) -> None:
 
         self._draw_contact(contact)
@@ -536,13 +465,11 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
     @event_filter(['account'])
     def _on_roster_push(self, event: RosterPush) -> None:
         contact = self._contacts.get_contact(str(event.item.jid))
+        assert isinstance(contact, BareContact)
 
         if event.item.subscription == 'remove':
-            contact.disconnect(self)
             self._remove_contact(contact)
         else:
-            if contact.jid not in self._contact_refs:
-                self._connect_contact_signals(contact)
             self._add_or_update_contact(contact)
 
     def _get_current_groups(self, jid: JID) -> set[str]:
@@ -567,7 +494,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         return group_iter
 
     def _add_contact_to_group(self,
-                              contact: types.BareContact,
+                              contact: BareContact,
                               group: str) -> None:
 
         group_iter = self._add_group(group)
@@ -579,13 +506,13 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         self._contact_refs[contact.jid].append(ref)
 
     def _add_contact_to_groups(self,
-                               contact: types.BareContact,
+                               contact: BareContact,
                                groups: set[str]) -> None:
         for group in groups:
             self._add_contact_to_group(contact, group)
 
     def _remove_contact_from_groups(self,
-                                    contact: types.BareContact,
+                                    contact: BareContact,
                                     groups: set[str]) -> None:
         if not groups:
             return
@@ -605,7 +532,8 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
         self._check_for_empty_groups()
 
-    def _remove_contact(self, contact: types.BareContact) -> None:
+    def _remove_contact(self, contact: BareContact) -> None:
+        contact.disconnect(self)
         refs = self._contact_refs.pop(contact.jid)
         for ref in refs:
             iter_ = self._get_iter_from_ref(ref)
@@ -622,7 +550,8 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
             self._store.remove(group_iter)
             del self._group_refs[group]
 
-    def _add_or_update_contact(self, contact: types.BareContact) -> None:
+    def _add_or_update_contact(self, contact: BareContact) -> None:
+        self._connect_contact_signals(contact)
         new_groups = set(contact.groups or [DEFAULT_GROUP])
         groups = self._get_current_groups(contact.jid)
 
@@ -665,6 +594,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
             for child in group.iterchildren():
                 contact = self._contacts.get_bare_contact(
                     child[Column.JID_OR_GROUP])
+                assert isinstance(contact, BareContact)
                 is_visible = self._get_contact_visible(contact)
                 child[Column.VISIBLE] = is_visible
                 if is_visible:
@@ -673,14 +603,14 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
             group[Column.VISIBLE] = group_is_visible
         self._roster.expand_all()
 
-    def _draw_contact(self, contact: types.BareContact) -> None:
+    def _draw_contact(self, contact: BareContact) -> None:
         for ref in self._contact_refs[contact.jid]:
             self._draw_contact_row(ref, contact)
         self._roster.expand_all()
 
     def _draw_contact_row(self,
                           ref: Gtk.TreeRowReference,
-                          contact: types.BareContact):
+                          contact: BareContact):
 
         iter_ = self._get_iter_from_ref(ref)
 
@@ -705,7 +635,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         assert path is not None
         return self._store.get_iter(path)
 
-    def _get_group_iter(self, group_name: str) -> Optional[Gtk.TreeIter]:
+    def _get_group_iter(self, group_name: str) -> Gtk.TreeIter | None:
         try:
             ref = self._group_refs[group_name]
         except KeyError:
@@ -745,7 +675,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
                             model: Gtk.TreeModel,
                             iter1: Gtk.TreeIter,
                             iter2: Gtk.TreeIter,
-                            _user_data: Literal[None]):
+                            _user_data: Literal[None]) -> int:
         '''
         Compare two iterators to sort them
         '''
@@ -760,8 +690,10 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
 
             contact1 = self._contacts.get_bare_contact(
                 model[iter1][Column.JID_OR_GROUP])
+            assert isinstance(contact1, BareContact)
             contact2 = self._contacts.get_bare_contact(
                 model[iter2][Column.JID_OR_GROUP])
+            assert isinstance(contact2, BareContact)
 
             if contact1.show != contact2.show:
                 if contact1.show == PresenceShow.DND:
@@ -782,16 +714,29 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         self._group_refs.clear()
         self._store.clear()
 
-    def process_event(self, event: ApplicationEvent) -> None:
-        if event.name not in HANDLED_EVENTS:
-            return
+    def _disconnect_actions(self) -> None:
+        for action, signal_id in self._action_signal_ids.items():
+            action.disconnect(signal_id)
+        self._action_signal_ids.clear()
+
+    @staticmethod
+    def _dummy_tree_compare(
+        _model: Gtk.TreeModel,
+        _iter1: Gtk.TreeIter,
+        _iter2: Gtk.TreeIter,
+        _user_data: Any
+    ) -> int:
+        return 0
 
     def _on_destroy(self, _roster: Roster) -> None:
-        self._remove_actions()
+        app.settings.disconnect_signals(self)
+        self._disconnect_actions()
         self._contact_refs.clear()
         self._group_refs.clear()
         self._unset_model()
         self._enable_sort(False)
+        # Set dummy method, otherwise store will not get destroyed
+        self._store.set_sort_func(Column.TEXT, self._dummy_tree_compare)
         self._store.clear()
         self._roster_tooltip.destroy()
         del self._roster
