@@ -1,18 +1,7 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
-from typing import Optional
 
 from gi.repository import Gtk
 from nbxmpp.protocol import JID
@@ -20,6 +9,9 @@ from nbxmpp.protocol import JID
 from gajim.common import app
 from gajim.common import i18n
 from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.dialogs import InformationDialog
@@ -34,7 +26,7 @@ class RosterItemExchange(Gtk.ApplicationWindow):
                  action: str,
                  exchange_list: dict[str, list[str]],
                  jid_from: JID,
-                 message_body: Optional[str] = None
+                 message_body: str | None = None
                  ) -> None:
         Gtk.ApplicationWindow.__init__(self)
         self.set_application(app.app)
@@ -54,6 +46,8 @@ class RosterItemExchange(Gtk.ApplicationWindow):
 
         # Set label depending on action
         contact = self._client.get_module('Contacts').get_contact(jid_from)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
         type_label = ''
         if action == 'add':
             type_label = _('%(name)s (%(jid)s) would like to add some '
@@ -127,16 +121,9 @@ class RosterItemExchange(Gtk.ApplicationWindow):
     def _add(self) -> None:
         for jid in self._exchange_list:
             contact = self._client.get_module('Contacts').get_contact(jid)
+            assert isinstance(contact, BareContact)
             name = self._exchange_list[jid][0]
-            groups = ''
-            num_list = len(self._exchange_list[jid][1])
-            current = 0
-            for group in self._exchange_list[jid][1]:
-                current += 1
-                if current == num_list:
-                    groups = groups + group
-                else:
-                    groups = groups + group + ', '
+            groups = ', '.join(self._exchange_list[jid][1])
             if not contact.is_in_roster:
                 self.show_all()
                 assert isinstance(self._model, Gtk.ListStore)
@@ -153,21 +140,15 @@ class RosterItemExchange(Gtk.ApplicationWindow):
         for jid in self._exchange_list:
             is_right = True
             contact = self._client.get_module('Contacts').get_contact(jid)
+            assert isinstance(contact, BareContact)
             name = self._exchange_list[jid][0]
-            groups = ''
+            groups = ', '.join(self._exchange_list[jid][1])
 
             if name != contact.name:
                 is_right = False
-            num_list = len(self._exchange_list[jid][1])
-            current = 0
             for group in self._exchange_list[jid][1]:
-                current += 1
                 if group not in contact.groups:
                     is_right = False
-                if current == num_list:
-                    groups = groups + group
-                else:
-                    groups = groups + group + ', '
             if not is_right and contact.is_in_roster:
                 self.show_all()
                 assert isinstance(self._model, Gtk.ListStore)
@@ -183,16 +164,9 @@ class RosterItemExchange(Gtk.ApplicationWindow):
     def _delete(self) -> None:
         for jid in self._exchange_list:
             contact = self._client.get_module('Contacts').get_contact(jid)
+            assert isinstance(contact, BareContact)
             name = self._exchange_list[jid][0]
-            groups = ''
-            num_list = len(self._exchange_list[jid][1])
-            current = 0
-            for group in self._exchange_list[jid][1]:
-                current += 1
-                if current == num_list:
-                    groups = groups + group
-                else:
-                    groups = groups + group + ', '
+            groups = ', '.join(self._exchange_list[jid][1])
             if contact.is_in_roster:
                 self.show_all()
                 assert isinstance(self._model, Gtk.ListStore)
@@ -217,11 +191,15 @@ class RosterItemExchange(Gtk.ApplicationWindow):
                     # It is selected
                     contact = self._client.get_module('Contacts').get_contact(
                         self._jid_from)
+                    assert isinstance(
+                        contact,
+                        BareContact | GroupchatContact | GroupchatParticipant)
                     message = _('%(name)s %(jid)s suggested me to add you to '
                                 'my contact list.') % {
                                     'name': contact.name,
                                     'jid': self._jid_from}
                     # Keep same groups and same nickname
+                    groups: list[str] = []
                     groups = model[iter_][3].split(', ')
                     if groups == ['']:
                         groups = []
