@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -40,24 +30,28 @@ from gajim.common.const import AvatarSize
 from gajim.common.const import Direction
 from gajim.common.const import MUC_DISCO_ERRORS
 from gajim.common.const import URIType
-from gajim.common.helpers import get_group_chat_nick
-from gajim.common.helpers import parse_uri
 from gajim.common.helpers import to_user_string
-from gajim.common.helpers import validate_jid
 from gajim.common.i18n import _
 from gajim.common.i18n import get_rfc5646_lang
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.util import as_task
+from gajim.common.util.jid import validate_jid
+from gajim.common.util.muc import get_group_chat_nick
+from gajim.common.util.text import get_country_flag_from_code
+from gajim.common.util.uri import parse_uri
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.chat_filter import ChatFilter
 from gajim.gtk.groupchat_info import GroupChatInfoScrolled
 from gajim.gtk.groupchat_nick import NickChooser
 from gajim.gtk.menus import get_start_chat_row_menu
+from gajim.gtk.tooltips import ContactTooltip
 from gajim.gtk.util import AccountBadge
 from gajim.gtk.util import GajimPopover
-from gajim.gtk.util import get_icon_name
+from gajim.gtk.util import get_status_icon_name
+from gajim.gtk.util import GroupBadge
+from gajim.gtk.util import IdleBadge
 
 ContactT = BareContact | GroupchatContact
 
@@ -303,7 +297,8 @@ class StartChatDialog(Gtk.ApplicationWindow):
                 return Gdk.EVENT_STOP
 
             if self._current_listbox_is(Search.GLOBAL):
-                if self._ui.search_entry.is_focus():
+                if (self._ui.search_entry.is_focus() and
+                        self._ui.search_entry.get_text()):
                     self._global_search_listbox.remove_all()
                     self._start_search()
 
@@ -787,38 +782,60 @@ class ContactRow(Gtk.ListBoxRow):
         image.set_size_request(AvatarSize.CHAT, AvatarSize.CHAT)
         grid.add(image)
 
-        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
-        box.set_hexpand(True)
+        self._tooltip = ContactTooltip()
+        image.set_has_tooltip(True)
+        image.connect('query-tooltip', self._on_query_tooltip)
 
         if self.name is None:
             self.name = _('Start New Chat')
 
+        meta_box = Gtk.Box(
+            orientation=Gtk.Orientation.VERTICAL, valign=Gtk.Align.CENTER)
+
         self.name_label = Gtk.Label(label=self.name)
         self.name_label.set_ellipsize(Pango.EllipsizeMode.END)
         self.name_label.set_xalign(0)
         self.name_label.set_width_chars(20)
         self.name_label.set_halign(Gtk.Align.START)
-        self.name_label.get_style_context().add_class('bold16')
-        name_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=6)
-        name_box.add(self.name_label)
+        self.name_label.get_style_context().add_class('bold14')
+        meta_box.add(self.name_label)
+
+        if contact and not contact.is_groupchat:
+            if idle := contact.idle_datetime:
+                idle_badge = IdleBadge(idle)
+                meta_box.add(idle_badge)
+
+        if contact and not contact.is_groupchat and (status := contact.status):
+            self.status_label = Gtk.Label(
+                label=status,
+                ellipsize=Pango.EllipsizeMode.END,
+                xalign=0,
+                width_chars=22,
+                halign=Gtk.Align.START,
+            )
+            self.status_label.get_style_context().add_class('dim-label')
+            self.status_label.get_style_context().add_class('small-label')
+            meta_box.add(self.status_label)
+
+        grid.attach(meta_box, 1, 0, 1, 3)
+
+        badge_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
         if show_account:
             account_badge = AccountBadge(account)
             account_badge.set_halign(Gtk.Align.END)
             account_badge.set_valign(Gtk.Align.START)
             account_badge.set_hexpand(True)
-            name_box.add(account_badge)
-        box.add(name_box)
+            badge_box.add(account_badge)
 
-        self.jid_label = Gtk.Label(label=str(jid))
-        self.jid_label.set_tooltip_text(str(jid))
-        self.jid_label.set_ellipsize(Pango.EllipsizeMode.END)
-        self.jid_label.set_xalign(0)
-        self.jid_label.set_width_chars(22)
-        self.jid_label.set_halign(Gtk.Align.START)
-        self.jid_label.get_style_context().add_class('dim-label')
-        box.add(self.jid_label)
+        if contact and not contact.is_groupchat and not contact.is_pm_contact:
+            groups = contact.groups
+            for group in groups:
+                group_badge = GroupBadge(group)
+                badge_box.add(group_badge)
 
-        grid.add(box)
+        grid.attach(badge_box, 2, 0, 1, 3)
+
+        self._grid = grid
 
         eventbox = Gtk.EventBox()
         eventbox.connect('button-press-event', self._popup_menu)
@@ -826,6 +843,18 @@ class ContactRow(Gtk.ListBoxRow):
         self.add(eventbox)
         self.show_all()
 
+    def _on_query_tooltip(self,
+                          _img: Gtk.Image,
+                          _x_coord: int,
+                          _y_coord: int,
+                          _keyboard_mode: bool,
+                          tooltip: Gtk.Tooltip) -> bool:
+        if not isinstance(self.contact, BareContact):
+            return False
+        v, widget = self._tooltip.get_tooltip(self.contact)
+        tooltip.set_custom(widget)
+        return v
+
     def _popup_menu(self,
                     _widget: Gtk.EventBox,
                     event: Gdk.EventButton
@@ -845,7 +874,7 @@ class ContactRow(Gtk.ListBoxRow):
         if self.is_new:
             icon_name = 'avatar-default'
             if self.groupchat:
-                icon_name = get_icon_name('muc-inactive')
+                icon_name = get_status_icon_name('muc-inactive')
             return Gtk.Image.new_from_icon_name(icon_name, Gtk.IconSize.DND)
 
         scale = self.get_scale_factor()
@@ -855,7 +884,7 @@ class ContactRow(Gtk.ListBoxRow):
 
     def update_jid(self, jid: JID) -> None:
         self.jid = jid
-        self.jid_label.set_text(str(jid))
+        self._grid.set_tooltip_text(str(jid))
 
     def update_chat_type(self, groupchat: bool = False) -> None:
         self.is_new = False
@@ -949,27 +978,77 @@ class GlobalSearch(Gtk.ListBox):
 
 class ResultRow(Gtk.ListBoxRow):
     def __init__(self, item: MuclumbusItem) -> None:
-        Gtk.ListBoxRow.__init__(self)
-        self.set_activatable(True)
+        Gtk.ListBoxRow.__init__(self, activatable=True)
         self.get_style_context().add_class('start-chat-row')
+
         self.is_new = False
         self.jid = JID.from_string(item.jid)
         self.groupchat = True
 
-        name_label = Gtk.Label(label=item.name)
-        name_label.set_halign(Gtk.Align.START)
-        name_label.set_ellipsize(Pango.EllipsizeMode.END)
-        name_label.set_max_width_chars(40)
+        name_label = Gtk.Label(
+            label=item.name or self.jid.localpart or item.jid,
+            halign=Gtk.Align.START,
+            ellipsize=Pango.EllipsizeMode.END,
+            max_width_chars=40
+        )
         name_label.get_style_context().add_class('bold16')
-        jid_label = Gtk.Label(label=item.jid)
-        jid_label.set_halign(Gtk.Align.START)
-        jid_label.set_ellipsize(Pango.EllipsizeMode.END)
-        jid_label.set_max_width_chars(40)
-        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        box.add(name_label)
-        box.add(jid_label)
 
-        self.add(box)
+        description_label = Gtk.Label(
+            label=item.description,
+            halign=Gtk.Align.START,
+            ellipsize=Pango.EllipsizeMode.END,
+            max_width_chars=40
+        )
+        description_label.get_style_context().add_class('dim-label')
+
+        main_box = Gtk.Box(spacing=12)
+        self.add(main_box)
+
+        name_box = Gtk.Box(
+            orientation=Gtk.Orientation.VERTICAL,
+            hexpand=True,
+            tooltip_text=item.jid
+        )
+        name_box.add(name_label)
+        name_box.add(description_label)
+
+        main_box.add(name_box)
+
+        language_code = item.language.upper()
+        if len(language_code) < 2:
+            language_code = ''
+
+        if len(language_code) > 2:
+            # Fix if no ISO code has been provided
+            language_code = language_code[:2]
+
+        if language_code == 'EN':
+            # Fix for most used languages/countries
+            language_code = 'GB'
+
+        language_label = Gtk.Label(
+            label=get_country_flag_from_code(language_code),
+            tooltip_text=_('Language: %s') % item.language
+        )
+        main_box.add(language_label)
+
+        users_box = Gtk.Box(
+            spacing=6,
+            tooltip_text=_('Participants Count')
+        )
+        users_box.get_style_context().add_class('dim-label')
+
+        users_icon = Gtk.Image.new_from_icon_name(
+            'system-users-symbolic',
+            Gtk.IconSize.BUTTON
+        )
+        users_box.add(users_icon)
+
+        users_count_label = Gtk.Label(label=item.nusers)
+        users_box.add(users_count_label)
+
+        main_box.add(users_box)
+
         self.show_all()
 
 
