@@ -3,67 +3,60 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
-from typing import Union
 
 import logging
 import math
 import sys
 import textwrap
-from io import BytesIO
-from importlib import import_module
-from functools import wraps
+from datetime import datetime
 from functools import lru_cache
-from pathlib import Path
+from functools import wraps
+from importlib import import_module
 from re import Match
 
-from PIL import Image
+import cairo
 from gi.repository import Gdk
+from gi.repository import GdkPixbuf
 from gi.repository import Gio
+from gi.repository import GLib
 from gi.repository import Gtk
 from gi.repository import GtkSource
-from gi.repository import GLib
 from gi.repository import Pango
-from gi.repository import GdkPixbuf
-import cairo
-import nbxmpp
 from nbxmpp import JID
+from nbxmpp import util as nbxmpp_util
 from nbxmpp.structs import LocationData
 from nbxmpp.structs import TuneData
 
 from gajim.common import app
 from gajim.common import configpaths
-from gajim.common.i18n import _
-from gajim.common.helpers import URL_REGEX
-from gajim.common.const import LOCATION_DATA
+from gajim.common import types
+from gajim.common.const import AvatarSize
 from gajim.common.const import Display
-from gajim.common.const import StyleAttr
+from gajim.common.const import LOCATION_DATA
 from gajim.common.ged import EventHelper as CommonEventHelper
-from gajim.common.styling import PlainBlock
+from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.regex import URL_REGEX
+from gajim.common.storage.archive import models as mod
+from gajim.common.storage.archive.const import ChatDirection
+from gajim.common.storage.archive.const import MessageType
 from gajim.common.structs import VariantMixin
+from gajim.common.styling import PlainBlock
+from gajim.common.util.user_strings import format_idle_time
 
-from .const import WINDOW_MODULES
+from gajim.gtk.const import WINDOW_MODULES
+
+log = logging.getLogger('gajim.gtk.util')
 
 
-log = logging.getLogger('gajim.gui.util')
-
-
-MenuValueT = Union[None, str, GLib.Variant, VariantMixin]
+MenuValueT = None | str | GLib.Variant | VariantMixin
 MenuItemListT = list[tuple[str, str, MenuValueT]]
 
 
@@ -78,8 +71,8 @@ def icon_exists(name: str) -> bool:
 
 def load_icon_info(icon_name: str,
                    size: int,
-                   scale: Optional[int],
-                   flags: Gtk.IconLookupFlags) -> Optional[Gtk.IconInfo]:
+                   scale: int | None,
+                   flags: Gtk.IconLookupFlags) -> Gtk.IconInfo | None:
 
     if scale is None:
         scale = app.window.get_scale_factor()
@@ -105,44 +98,59 @@ def load_icon_info(icon_name: str,
 def load_icon_surface(
         icon_name: str,
         size: int = 16,
-        scale: Optional[int] = None,
+        scale: int | None = None,
         flags: Gtk.IconLookupFlags = Gtk.IconLookupFlags.FORCE_SIZE
-) -> Optional[cairo.ImageSurface]:
+) -> cairo.ImageSurface | None:
 
     icon_info = load_icon_info(icon_name, size, scale, flags)
     if icon_info is None:
         return None
-    return icon_info.load_surface(None)
+
+    try:
+        surface = icon_info.load_surface(None)
+    except GLib.Error as e:
+        log.error(
+            'Error loading icon surface for %s, %s, %s: %s',
+            icon_name,
+            size,
+            scale,
+            e
+        )
+        return None
+
+    return surface
 
 
 def load_icon_pixbuf(icon_name: str,
                      size: int = 16,
-                     scale: Optional[int] = None,
+                     scale: int | None = None,
                      flags: Gtk.IconLookupFlags = Gtk.IconLookupFlags.FORCE_SIZE
-                     ) -> Optional[GdkPixbuf.Pixbuf]:
+                     ) -> GdkPixbuf.Pixbuf | None:
 
     icon_info = load_icon_info(icon_name, size, scale, flags)
     if icon_info is None:
         return None
-    return icon_info.load_icon()
+
+    try:
+        icon = icon_info.load_icon()
+    except GLib.Error as e:
+        log.error(
+            'Error loading icon surface for %s, %s, %s: %s',
+            icon_name,
+            size,
+            scale,
+            e
+        )
+        return None
+
+    return icon
 
 
-def get_app_icon_list(scale_widget: Gtk.Widget) -> list[GdkPixbuf.Pixbuf]:
-    scale = scale_widget.get_scale_factor()
-    pixbufs: list[GdkPixbuf.Pixbuf] = []
-    for size in (16, 32, 48, 64, 128):
-        pixbuf = load_icon_pixbuf('org.gajim.Gajim', size=size, scale=scale)
-        if pixbuf is not None:
-            pixbufs.append(pixbuf)
-    return pixbufs
-
-
-def get_icon_name(name: str,
-                  transport: Optional[str] = None) -> str:
-    if transport is not None:
-        return f'{transport}-{name}'
-
-    return f'dcraven-{name}'
+def get_status_icon_name(name: str) -> str:
+    prefix = 'gajim'
+    if app.is_flatpak():
+        prefix = app.get_default_app_id()
+    return f'{prefix}-status-{name}'
 
 
 def load_user_iconsets() -> None:
@@ -165,7 +173,7 @@ def get_total_screen_geometry() -> tuple[int, int]:
     display = Gdk.Display.get_default()
     assert display is not None
     monitors = display.get_n_monitors()
-    for num in range(0, monitors):
+    for num in range(monitors):
         monitor = display.get_monitor(num)
         assert monitor is not None
         geometry = monitor.get_geometry()
@@ -225,7 +233,7 @@ def restore_main_window_position() -> None:
                 app.settings.get('mainwin_y_position'))
 
 
-def get_source_view_style_scheme() -> Optional[GtkSource.StyleScheme]:
+def get_source_view_style_scheme() -> GtkSource.StyleScheme | None:
     style_scheme_manager = GtkSource.StyleSchemeManager.get_default()
     if app.css_config.prefer_dark:
         return style_scheme_manager.get_scheme('solarized-dark')
@@ -256,10 +264,16 @@ def get_completion_liststore(entry: Gtk.Entry) -> Gtk.ListStore:
 def get_cursor(name: str) -> Gdk.Cursor:
     display = Gdk.Display.get_default()
     assert display is not None
-    cursor = Gdk.Cursor.new_from_name(display, name)
-    if cursor is not None:
-        return cursor
-    return Gdk.Cursor.new_from_name(display, 'default')
+    try:
+        cursor = Gdk.Cursor.new_from_name(display, name)
+        if cursor is not None:
+            return cursor
+    except TypeError as e:
+        log.exception(e)
+
+    cursor = Gdk.Cursor.new_from_name(display, 'default')
+    assert cursor is not None
+    return cursor
 
 
 def scroll_to_end(widget: Gtk.ScrolledWindow) -> bool:
@@ -272,7 +286,7 @@ def scroll_to_end(widget: Gtk.ScrolledWindow) -> bool:
         bool: The return value is False so it can be used with GLib.idle_add.
     '''
     adj_v = widget.get_vadjustment()
-    if adj_v is None:
+    if adj_v is None:  # pyright: ignore
         # This can happen when the Widget is already destroyed when called
         # from GLib.idle_add
         return False
@@ -322,7 +336,7 @@ def convert_rgba_to_hex(rgba: Gdk.RGBA) -> str:
     red = int(rgba.red * 255)
     green = int(rgba.green * 255)
     blue = int(rgba.blue * 255)
-    return '#%02x%02x%02x' % (red, green, blue)
+    return f'#{red:02x}{green:02x}{blue:02x}'
 
 
 def convert_rgb_to_hex(rgb_string: str) -> str:
@@ -350,6 +364,7 @@ def make_rgba(color_string: str) -> Gdk.RGBA:
 
 
 def get_monitor_scale_factor() -> int:
+    # Does not work on Wayland
     display = Gdk.Display.get_default()
     assert display is not None
     monitor = display.get_primary_monitor()
@@ -359,7 +374,7 @@ def get_monitor_scale_factor() -> int:
     return monitor.get_scale_factor()
 
 
-def get_primary_accel_mod() -> Optional[Gdk.ModifierType]:
+def get_primary_accel_mod() -> Gdk.ModifierType | None:
     '''
     Returns the primary Gdk.ModifierType modifier.
     cmd on osx, ctrl everywhere else.
@@ -404,14 +419,13 @@ def format_tune(data: TuneData) -> str:
     title = GLib.markup_escape_text(data.title or _('Unknown Title'))
     source = GLib.markup_escape_text(data.source or _('Unknown Source'))
 
-    tune_string = _('<b>"%(title)s"</b> by <i>%(artist)s</i>\n'
-                    'from <i>%(source)s</i>') % {'title': title,
-                                                 'artist': artist,
-                                                 'source': source}
-    return tune_string
+    return _('<b>"%(title)s"</b> by <i>%(artist)s</i>\n'
+             'from <i>%(source)s</i>') % {'title': title,
+                                          'artist': artist,
+                                          'source': source}
 
 
-def get_account_tune_icon_name(account: str) -> Optional[str]:
+def get_account_tune_icon_name(account: str) -> str | None:
     client = app.get_client(account)
     tune = client.get_module('UserTune').get_current_tune()
     return None if tune is None else 'audio-x-generic'
@@ -428,19 +442,18 @@ def format_location(location: LocationData) -> str:
         tag = LOCATION_DATA.get(attr)
         if tag is None:
             continue
-        location_string += '\n<b>%(tag)s</b>: %(text)s' % {
-            'tag': tag.capitalize(), 'text': text}
+        location_string += f'\n<b>{tag.capitalize()}</b>: {text}'
 
     return location_string.strip()
 
 
-def get_account_location_icon_name(account: str) -> Optional[str]:
+def get_account_location_icon_name(account: str) -> str | None:
     client = app.get_client(account)
     location = client.get_module('UserLocation').get_current_location()
     return None if location is None else 'applications-internet'
 
 
-def format_eta(time_: Union[int, float]) -> str:
+def format_eta(time_: int | float) -> str:
     times = {'minutes': 0, 'seconds': 0}
     time_ = int(time_)
     times['seconds'] = time_ % 60
@@ -456,12 +469,12 @@ def format_fingerprint(fingerprint: str) -> str:
     wordsize = fplen // 8
     buf = ''
     for char in range(0, fplen, wordsize):
-        buf += '{0} '.format(fingerprint[char:char + wordsize])
+        buf += f'{fingerprint[char:char + wordsize]} '
     buf = textwrap.fill(buf, width=36)
     return buf.rstrip().upper()
 
 
-def find_widget(name: str, container: Gtk.Container) -> Optional[Gtk.Widget]:
+def find_widget(name: str, container: Gtk.Container) -> Gtk.Widget | None:
     for child in container.get_children():
         if Gtk.Buildable.get_name(child) == name:
             return child
@@ -476,6 +489,7 @@ class MultiLineLabel(Gtk.Label):
         self.set_line_wrap(True)
         self.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
         self.set_single_line_mode(False)
+        self.set_selectable(True)
 
 
 class MaxWidthComboBoxText(Gtk.ComboBoxText):
@@ -492,19 +506,30 @@ class MaxWidthComboBoxText(Gtk.ComboBoxText):
         minimum_width, natural_width = Gtk.ComboBoxText.do_get_preferred_width(
             self)
 
-        if natural_width > self._max_width:
-            natural_width = self._max_width
-        if minimum_width > self._max_width:
-            minimum_width = self._max_width
+        natural_width = min(natural_width, self._max_width)
+        minimum_width = min(minimum_width, self._max_width)
         return minimum_width, natural_width
 
 
 def text_to_color(text: str) -> tuple[float, float, float]:
     if app.css_config.prefer_dark:
-        background = (0, 0, 0)  # RGB (0, 0, 0) black
+        lightness = 60
     else:
-        background = (1, 1, 1)  # RGB (255, 255, 255) white
-    return nbxmpp.util.text_to_color(text, background)  # type: ignore
+        lightness = 40
+    return nbxmpp_util.text_to_color(text, 100, lightness)
+
+
+def get_contact_color(contact: types.ChatContactT
+                      ) -> tuple[float, float, float]:
+
+    if isinstance(contact, GroupchatParticipant):
+        if contact.room.muc_context in (None, 'public'):
+            return text_to_color(contact.name)
+
+        if contact.real_jid is not None:
+            return text_to_color(str(contact.real_jid))
+
+    return text_to_color(str(contact.jid))
 
 
 def get_color_for_account(account: str) -> str:
@@ -534,98 +559,57 @@ def add_css_to_widget(widget: Any, css: str) -> None:
                          Gtk.STYLE_PROVIDER_PRIORITY_USER)
 
 
-def get_pixbuf_from_data(file_data: bytes) -> Optional[GdkPixbuf.Pixbuf]:
-    # TODO: This already exists in preview_helpery pixbuf_from_data
-    '''
-    Get image data and returns GdkPixbuf.Pixbuf
-    '''
-    pixbufloader = GdkPixbuf.PixbufLoader()
-    try:
-        pixbufloader.write(file_data)
-        pixbufloader.close()
-        pixbuf = pixbufloader.get_pixbuf()
-    except GLib.Error:
-        pixbufloader.close()
+def get_contact_name_for_message(
+        db_row: mod.Message,
+        contact: types.ChatContactT
+    ) -> str:
 
-        log.warning('loading avatar using pixbufloader failed, trying to '
-                    'convert avatar image using pillow')
-        try:
-            avatar = Image.open(BytesIO(file_data)).convert('RGBA')
-            array = GLib.Bytes.new(avatar.tobytes())  # type: ignore
-            width, height = avatar.size
-            pixbuf = GdkPixbuf.Pixbuf.new_from_bytes(
-                array, GdkPixbuf.Colorspace.RGB,
-                True, 8, width, height, width * 4)
-        except Exception:
-            log.warning('Could not use pillow to convert avatar image, '
-                        'image cannot be displayed', exc_info=True)
-            return None
+    if isinstance(contact, BareContact) and contact.is_self:
+        return _('Me')
 
-    return pixbuf
+    if db_row.type == MessageType.CHAT:
+        if db_row.direction == ChatDirection.INCOMING:
+            return contact.name
+        return app.nicks[contact.account]
+
+    elif db_row.type == MessageType.GROUPCHAT:
+        resource = db_row.resource
+        if resource is None:
+            # Fall back to MUC name if contact name is None
+            # (may be the case for service messages from the MUC)
+            return contact.name
+        return resource
+
+    elif db_row.type == MessageType.PM:
+        resource = db_row.resource
+        assert resource is not None
+        return resource
+
+    else:
+        raise ValueError
 
 
-def scale_with_ratio(size: int, width: int, height: int) -> tuple[int, int]:
-    if height == width:
-        return size, size
-    if height > width:
-        ratio = height / float(width)
-        return int(size / ratio), size
+def get_avatar_for_message(
+        db_row: mod.Message,
+        contact: types.ChatContactT,
+        scale: int,
+        size: AvatarSize
+    ) -> cairo.ImageSurface | None:
 
-    ratio = width / float(height)
-    return size, int(size / ratio)
+    if isinstance(contact, GroupchatContact):
+        name = get_contact_name_for_message(db_row, contact)
+        resource_contact = contact.get_resource(name)
+        return resource_contact.get_avatar(
+            size, scale, add_show=False)
 
+    if db_row.direction == ChatDirection.OUTGOING:
+        client = app.get_client(contact.account)
+        self_contact = client.get_module('Contacts').get_contact(
+            client.get_own_jid().bare)
+        assert isinstance(self_contact, BareContact)
+        return self_contact.get_avatar(size, scale, add_show=False)
 
-def scale_pixbuf(pixbuf: GdkPixbuf.Pixbuf,
-                 size: int) -> Optional[GdkPixbuf.Pixbuf]:
-    width, height = scale_with_ratio(size,
-                                     pixbuf.get_width(),
-                                     pixbuf.get_height())
-    return pixbuf.scale_simple(width, height,
-                               GdkPixbuf.InterpType.BILINEAR)
-
-
-def scale_pixbuf_from_data(data: bytes,
-                           size: int
-                           ) -> Optional[GdkPixbuf.Pixbuf]:
-    pixbuf = get_pixbuf_from_data(data)
-    assert pixbuf is not None
-    return scale_pixbuf(pixbuf, size)
-
-
-def load_pixbuf(path: Union[str, Path],
-                size: Optional[int] = None
-                ) -> Optional[GdkPixbuf.Pixbuf]:
-    try:
-        if size is None:
-            return GdkPixbuf.Pixbuf.new_from_file(str(path))
-        return GdkPixbuf.Pixbuf.new_from_file_at_scale(
-            str(path), size, size, True)
-
-    except GLib.Error:
-        try:
-            with open(path, 'rb') as im_handle:
-                img = Image.open(im_handle)
-                avatar = img.convert('RGBA')
-        except (NameError, OSError):
-            log.warning('Pillow convert failed: %s', path)
-            log.debug('Error', exc_info=True)
-            return None
-
-        array = GLib.Bytes.new(avatar.tobytes())  # type: ignore
-        width, height = avatar.size
-        pixbuf = GdkPixbuf.Pixbuf.new_from_bytes(
-            array, GdkPixbuf.Colorspace.RGB, True,
-            8, width, height, width * 4)
-        if size is not None:
-            width, height = scale_with_ratio(size, width, height)
-            return pixbuf.scale_simple(width,
-                                       height,
-                                       GdkPixbuf.InterpType.BILINEAR)
-        return pixbuf
-
-    except RuntimeError as error:
-        log.warning('Loading pixbuf failed: %s', error)
-        return None
+    return contact.get_avatar(size, scale, add_show=False)
 
 
 def get_thumbnail_size(pixbuf: GdkPixbuf.Pixbuf, size: int) -> tuple[int, int]:
@@ -636,28 +620,28 @@ def get_thumbnail_size(pixbuf: GdkPixbuf.Pixbuf, size: int) -> tuple[int, int]:
     if image_width > image_height:
         if image_width > size:
             image_height = math.ceil(
-                (size / float(image_width) * image_height))
+                size / float(image_width) * image_height)
             image_width = int(size)
     else:
         if image_height > size:
             image_width = math.ceil(
-                (size / float(image_height) * image_width))
+                size / float(image_height) * image_width)
             image_height = int(size)
 
     return image_width, image_height
 
 
-def make_href_markup(string: str) -> str:
-    url_color = app.css_config.get_value('.gajim-url', StyleAttr.COLOR)
-    assert isinstance(url_color, str)
-    color = convert_rgb_to_hex(url_color)
+def make_href_markup(string: str | None) -> str:
+    if not string:
+        return ''
+
+    string = GLib.markup_escape_text(string)
 
     def _to_href(match: Match[str]) -> str:
         url = match.group()
         if '://' not in url:
-            url = 'https://' + url
-        return '<a href="%s"><span foreground="%s">%s</span></a>' % (
-            url, color, match.group())
+            url = f'https://{url}'
+        return f'<a href="{url}">{match.group()}</a>'
 
     return URL_REGEX.sub(_to_href, string)
 
@@ -666,25 +650,25 @@ def get_app_windows(account: str) -> list[Gtk.Window]:
     windows: list[Gtk.Window] = []
     for win in app.app.get_windows():
         if hasattr(win, 'account'):
-            if win.account == account:  # type: ignore
+            if win.account == account:  # pyright: ignore
                 windows.append(win)
     return windows
 
 
 def get_app_window(name: str,
-                   account: Optional[str] = None,
-                   jid: Optional[Union[str, JID]] = None
-                   ) -> Optional[Gtk.Window]:
+                   account: str | None = None,
+                   jid: str | JID | None = None
+                   ) -> Gtk.Window | None:
     for win in app.app.get_windows():
         if type(win).__name__ != name:
             continue
 
         if account is not None:
-            if account != win.account:  # type: ignore
+            if account != win.account:  # pyright: ignore
                 continue
 
         if jid is not None:
-            if jid != win.jid:  # type: ignore
+            if jid != win.jid:  # pyright: ignore
                 continue
         return win
     return None
@@ -704,17 +688,16 @@ def open_window(name: str, **kwargs: Any) -> Any:
 
 
 def get_gtk_version() -> str:
-    gtk_ver = '%i.%i.%i' % (
+    return '%i.%i.%i' % (
         Gtk.get_major_version(),
         Gtk.get_minor_version(),
         Gtk.get_micro_version())
-    return gtk_ver
 
 
 class EventHelper(CommonEventHelper):
     def __init__(self):
         CommonEventHelper.__init__(self)
-        self.connect('destroy', self.__on_destroy)  # type: ignore
+        self.connect('destroy', self.__on_destroy)  # pyright: ignore
 
     def __on_destroy(self, *args: Any) -> None:
         self.unregister_events()
@@ -781,8 +764,48 @@ def wrap_with_event_box(klass: Any) -> Any:
     return klass_wrapper
 
 
+class GroupBadge(Gtk.Label):
+    def __init__(self, group: str) -> None:
+        Gtk.Label.__init__(
+            self,
+            ellipsize=Pango.EllipsizeMode.END,
+            no_show_all=True,
+            halign=Gtk.Align.END,
+            valign=Gtk.Align.CENTER,
+            hexpand=True,
+            tooltip_text=group,
+        )
+        self.set_size_request(50, -1)
+        self.get_style_context().add_class('group')
+        self.set_text(group)
+        self.show()
+
+
+class IdleBadge(Gtk.Label):
+    def __init__(self, idle: datetime | None = None) -> None:
+        Gtk.Label.__init__(
+            self,
+            halign=Gtk.Align.START,
+            hexpand=True,
+            ellipsize=Pango.EllipsizeMode.NONE,
+            no_show_all=True,
+        )
+        self.set_size_request(50, -1)
+        self.get_style_context().add_class('dim-label')
+        self.get_style_context().add_class('small-label')
+        if idle is not None:
+            self.set_idle(idle)
+
+        self.show()
+
+    def set_idle(self, idle: datetime) -> None:
+        self.set_text(_('Last seen: %s') % format_idle_time(idle))
+        format_string = app.settings.get('date_time_format')
+        self.set_tooltip_text(idle.strftime(format_string))
+
+
 class AccountBadge(Gtk.Label):
-    def __init__(self, account: Optional[str] = None) -> None:
+    def __init__(self, account: str | None = None) -> None:
         Gtk.Label.__init__(self)
         self.set_ellipsize(Pango.EllipsizeMode.END)
         self.set_max_width_chars(12)
@@ -797,9 +820,31 @@ class AccountBadge(Gtk.Label):
     def set_account(self, account: str) -> None:
         label = app.get_account_label(account)
         self.set_text(label)
+
+        style_context = self.get_style_context()
+        for style_class in style_context.list_classes():
+            if style_class != 'badge':
+                style_context.remove_class(style_class)
+
         account_class = app.css_config.get_dynamic_class(account)
-        self.get_style_context().add_class(account_class)
+        style_context.add_class(account_class)
+
         self.set_tooltip_text(_('Account: %s') % label)
+        app.settings.disconnect_signals(self)
+        app.settings.connect_signal(
+            'account_label',
+            self._on_account_label_changed,
+            account)
+
+    def _on_account_label_changed(self,
+                                  _value: str,
+                                  _setting: str,
+                                  account: str | None,
+                                  *args: Any
+                                  ) -> None:
+
+        assert account is not None
+        self.set_account(account)
 
 
 def make_pango_attributes(block: PlainBlock) -> Pango.AttrList:
@@ -812,6 +857,24 @@ def make_pango_attributes(block: PlainBlock) -> Pango.AttrList:
     return attrlist
 
 
+_grapheme_buffer = Gtk.TextBuffer()
+
+
+def get_first_graphemes(text: str, n: int) -> str:
+    # This should be possible with lower-level APIs like Pango.break_* or
+    # Pango.get_log_attrs, but their Python bindings seem totally broken.
+    # The reuse of one global buffer is to mitigate very probable memory leaks.
+    _grapheme_buffer.set_text(text)
+    cursor = _grapheme_buffer.get_start_iter()
+    cursor.forward_cursor_positions(n)
+    return _grapheme_buffer.get_slice(
+        _grapheme_buffer.get_start_iter(), cursor, False)
+
+
+def get_first_grapheme(text: str) -> str:
+    return get_first_graphemes(text, 1)
+
+
 def get_style_attribute_with_name(name: str) -> Pango.Attribute:
     if name == 'strong':
         return Pango.attr_weight_new(Pango.Weight.BOLD)
@@ -828,7 +891,7 @@ def get_style_attribute_with_name(name: str) -> Pango.Attribute:
     raise ValueError('unknown attribute %s' % name)
 
 
-def get_key_theme() -> Optional[str]:
+def get_key_theme() -> str | None:
     settings = Gtk.Settings.get_default()
     if settings is None:
         return None
@@ -836,15 +899,18 @@ def get_key_theme() -> Optional[str]:
 
 
 def make_menu_item(label: str,
-                   action: Optional[str] = None,
+                   action: str | None = None,
                    value: MenuValueT = None) -> Gio.MenuItem:
 
+    item = Gio.MenuItem.new(label)
+
     if value is None:
-        return Gio.MenuItem.new(label, action)
+        item.set_action_and_target_value(action, None)
+        return item
 
     item = Gio.MenuItem.new(label)
     if isinstance(value, str):
-        item.set_detailed_action(f'{action}({value})')
+        item.set_action_and_target_value(action, GLib.Variant('s', value))
     elif isinstance(value, VariantMixin):
         item.set_action_and_target_value(action, value.to_variant())
     else:
@@ -867,7 +933,7 @@ class GajimMenu(Gio.Menu):
     def add_item(self,
                  label: str,
                  action: str,
-                 value: Optional[MenuValueT] = None) -> None:
+                 value: MenuValueT | None = None) -> None:
         item = make_menu_item(label, action, value)
         self.append_item(item)
 
@@ -894,14 +960,16 @@ class GdkRectangle(Gdk.Rectangle):
 
 class GajimPopover(Gtk.Popover):
     def __init__(self,
-                 menu: Gio.MenuModel,
-                 relative_to: Optional[Gtk.Widget] = None,
+                 menu: Gio.MenuModel | None = None,
+                 relative_to: Gtk.Widget | None = None,
                  position: Gtk.PositionType = Gtk.PositionType.RIGHT,
-                 event: Optional[Gdk.EventButton] = None) -> None:
+                 event: Gdk.EventButton | None = None) -> None:
 
         Gtk.Popover.__init__(self)
 
-        self.bind_model(menu)
+        if menu is not None:
+            self.bind_model(menu)
+
         self.set_relative_to(relative_to)
         self.set_position(position)
         if event is not None:
