@@ -1,30 +1,23 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
 
-import os
 import logging
+import os
 import pickle
+from collections.abc import Generator
 
 from gajim.common import configpaths
 from gajim.common.events import ApplicationEvent
 from gajim.common.ged import HandlerFuncT
+from gajim.common.modules.base import BaseModule
 from gajim.common.types import PluginExtensionPoints
 
+from .manifest import PluginManifest
 
 log = logging.getLogger('gajim.p.plugin')
 
@@ -37,7 +30,7 @@ class GajimPlugin:
     __path__: str = ''
 
     encryption_name: str = ''
-    manifest: Any
+    manifest: PluginManifest
 
     gui_extension_points: PluginExtensionPoints = {}
     '''
@@ -77,8 +70,8 @@ class GajimPlugin:
     events: list[ApplicationEvent] = []
     '''
     New network event classes to be registered in Network Events Controller.
-
     '''
+    modules: list[BaseModule] = []
 
     def __init__(self) -> None:
         self.config = GajimPluginConfig(self)
@@ -144,9 +137,8 @@ class GajimPluginConfig:
     def __contains__(self, key: str) -> bool:
         return key in self.data
 
-    def __iter__(self):
-        for key in self.data.keys():
-            yield key
+    def __iter__(self) -> Generator[str, Any, None]:
+        yield from self.data
 
     def keys(self):
         return self.data.keys()
@@ -172,7 +164,7 @@ class GajimPluginConfig:
                     shelf = shelve.open(str(self.FILE_PATH))
                     for (key, value) in shelf.items():
                         self.data[key] = value
-                    if not isinstance(self.data, dict):
+                    if not isinstance(self.data, dict):  # pyright: ignore
                         raise GajimPluginException
                     shelf.close()
                     self.save()
